@@ -197,6 +197,14 @@ QString AbstractAspect::name() const {
 	return d->m_name;
 }
 
+QUuid AbstractAspect::uuid() const {
+	return d->m_uuid;
+}
+
+void AbstractAspect::setSuppressWriteUuid(bool suppress) {
+	d->m_suppressWriteUuid = suppress;
+}
+
 /*!
  * \brief AbstractAspect::setName
  * sets the name of the abstract aspect
@@ -205,7 +213,7 @@ QString AbstractAspect::name() const {
  * \param skipAutoUnique - if set to \true, don't check for uniqueness, the caller has to guarantee the uniqueness. default is \false.
  * \return returns, if the new name is valid or not
  */
-bool AbstractAspect::setName(const QString& value, NameHandling handling) {
+bool AbstractAspect::setName(const QString& value, NameHandling handling, QUndoCommand* /*parent*/) {
 	if (value.isEmpty())
 		return setName(QLatin1String("1"), handling);
 
@@ -224,11 +232,7 @@ bool AbstractAspect::setName(const QString& value, NameHandling handling) {
 			info(i18n(R"(Intended name "%1" was changed to "%2" in order to avoid name collision.)", value, new_name));
 	} else
 		new_name = value;
-
-	exec(new PropertyChangeCommand<QString>(i18n("%1: rename to %2", d->m_name, new_name), &d->m_name, new_name),
-		 "aspectDescriptionAboutToChange",
-		 "aspectDescriptionChanged",
-		 Q_ARG(const AbstractAspect*, this));
+	exec(new AspectNameChangeCmd(this->d, new_name));
 	return true;
 }
 
@@ -242,7 +246,7 @@ void AbstractAspect::setComment(const QString& value) {
 	exec(new PropertyChangeCommand<QString>(i18n("%1: change comment", d->m_name), &d->m_comment, value),
 		 "aspectDescriptionAboutToChange",
 		 "aspectDescriptionChanged",
-		 Q_ARG(const AbstractAspect*, this));
+		 QArgument<const AbstractAspect*>("const AbstractAspect*", this));
 }
 
 void AbstractAspect::setCreationTime(const QDateTime& time) {
@@ -314,6 +318,7 @@ QMenu* AbstractAspect::createContextMenu() {
 	// TODO: activate this again when the functionality is implemented
 	// 	menu->addAction( KStandardAction::cut(this) );
 
+	QAction* actionDuplicate = nullptr;
 	if (!isFixed() && m_type != AspectType::Project && m_type != AspectType::CantorWorksheet) {
 		// copy action:
 		// don't allow to copy fixed aspects
@@ -324,8 +329,8 @@ QMenu* AbstractAspect::createContextMenu() {
 		// duplicate action:
 		// don't allow to duplicate legends in the plots
 		if (m_type != AspectType::CartesianPlotLegend) {
-			auto* actionDuplicate = new QAction(QIcon::fromTheme(QLatin1String("edit-copy")), i18n("Duplicate Here"), this);
-			actionDuplicate->setShortcut(Qt::CTRL + Qt::Key_D);
+			actionDuplicate = new QAction(QIcon::fromTheme(QLatin1String("edit-copy")), i18n("Duplicate Here"), this);
+			actionDuplicate->setShortcut(Qt::CTRL | Qt::Key_D);
 			connect(actionDuplicate, &QAction::triggered, this, &AbstractAspect::duplicate);
 			menu->addAction(actionDuplicate);
 		}
@@ -340,7 +345,10 @@ QMenu* AbstractAspect::createContextMenu() {
 	if (t != AspectType::AbstractAspect && pasteTypes().indexOf(t) != -1) {
 		auto* action = KStandardAction::paste(this);
 		action->setText(i18n("Paste '%1'", name));
-		menu->addAction(action);
+		if (actionDuplicate)
+			menu->insertAction(actionDuplicate, action);
+		else
+			menu->addAction(action);
 		connect(action, &QAction::triggered, this, &AbstractAspect::paste);
 	}
 	menu->addSeparator();
@@ -396,7 +404,7 @@ QMenu* AbstractAspect::createContextMenu() {
 	// rename and delete actions:
 	menu->addAction(QIcon::fromTheme(QLatin1String("edit-rename")), i18n("Rename"), this, &AbstractAspect::renameRequested);
 	if (m_type != AspectType::Project)
-		menu->addAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Delete"), this, &AbstractAspect::remove);
+		menu->addAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Delete"), this, QOverload<>::of(&AbstractAspect::remove));
 
 	// move up and down actions:
 	// don't shown them for worksheet elements since they implement their own "Drawing order" menu
@@ -404,7 +412,7 @@ QMenu* AbstractAspect::createContextMenu() {
 		const auto* parent = parentAspect();
 		int count = parent->childCount<AbstractAspect>();
 		if (count > 1) {
-			auto* moveMenu = new QMenu(i18n("Move"));
+			auto* moveMenu = new QMenu(i18n("Move"), menu);
 			moveMenu->setIcon(QIcon::fromTheme(QStringLiteral("layer-bottom")));
 			if (parent->indexOfChild<AbstractAspect>(this) != 0)
 				moveMenu->addAction(QIcon::fromTheme(QStringLiteral("draw-arrow-up")), i18n("Up"), this, &AbstractAspect::moveUp);
@@ -490,6 +498,12 @@ Project* AbstractAspect::project() {
 	return parentAspect() ? parentAspect()->project() : nullptr;
 }
 
+void AbstractAspect::setProjectChanged(bool changed) {
+	auto* p = project();
+	if (p)
+		p->setChanged(changed);
+}
+
 /**
  * \brief Return the path that leads from the top-most Aspect (usually a Project) to me.
  */
@@ -500,50 +514,69 @@ QString AbstractAspect::path() const {
 /**
  * \brief Add the given Aspect to my list of children.
  */
-void AbstractAspect::addChild(AbstractAspect* child) {
+void AbstractAspect::addChild(AbstractAspect* child, QUndoCommand* parent) {
 	Q_CHECK_PTR(child);
 
 	const QString new_name = uniqueNameFor(child->name());
-	beginMacro(i18n("%1: add %2", name(), new_name));
+	bool execute = false;
+	if (!parent) {
+		execute = true;
+		parent = new QUndoCommand(i18n("%1: add %2", name(), new_name));
+	}
 	if (new_name != child->name()) {
 		info(i18n(R"(Renaming "%1" to "%2" in order to avoid name collision.)", child->name(), new_name));
-		child->setName(new_name);
+		child->setName(new_name, NameHandling::AutoUnique, parent);
 	}
 
-	exec(new AspectChildAddCmd(d, child, d->m_children.count()));
-	endMacro();
+	new AspectChildAddCmd(d, child, d->m_children.count(), parent);
+
+	if (execute)
+		exec(parent);
 }
 
 /**
  * \brief Add the given Aspect to my list of children without any checks and without putting this step onto the undo-stack
  */
 void AbstractAspect::addChildFast(AbstractAspect* child) {
-	Q_EMIT aspectAboutToBeAdded(this, nullptr, child); // TODO: before-pointer is 0 here, also in the commands classes. why?
+	Q_EMIT childAspectAboutToBeAdded(this, nullptr, child); // TODO: before-pointer is 0 here, also in the commands classes. why?
 	d->insertChild(d->m_children.count(), child);
 	child->finalizeAdd();
 	// PERFTRACE(Q_FUNC_INFO);
-	Q_EMIT aspectAdded(child);
+	Q_EMIT childAspectAdded(child);
 	// print_callstack();
 }
 
 /**
  * \brief Insert the given Aspect at a specific position in my list of children.
  */
-void AbstractAspect::insertChildBefore(AbstractAspect* child, AbstractAspect* before) {
+void AbstractAspect::insertChildBefore(AbstractAspect* child, AbstractAspect* before, QUndoCommand* parent) {
+	insertChild(child, d->indexOfChild(before), parent);
+}
+
+void AbstractAspect::insertChild(AbstractAspect* child, int index, QUndoCommand* parent) {
 	Q_CHECK_PTR(child);
 
-	QString new_name = uniqueNameFor(child->name());
-	beginMacro(before ? i18n("%1: insert %2 before %3", name(), new_name, before->name()) : i18n("%1: insert %2 before end", name(), new_name));
-	if (new_name != child->name()) {
-		info(i18n(R"(Renaming "%1" to "%2" in order to avoid name collision.)", child->name(), new_name));
-		child->setName(new_name);
-	}
-	int index = d->indexOfChild(before);
 	if (index == -1)
 		index = d->m_children.count();
 
-	exec(new AspectChildAddCmd(d, child, index));
-	endMacro();
+	QString new_name = uniqueNameFor(child->name());
+	bool execute = false;
+	if (!parent) {
+		execute = true;
+		const auto* before = this->child<AbstractAspect>(index);
+		parent =
+			new QUndoCommand(before ? i18n("%1: insert %2 before %3", name(), new_name, before->name()) : i18n("%1: insert %2 before end", name(), new_name));
+	}
+
+	if (new_name != child->name()) {
+		info(i18n(R"(Renaming "%1" to "%2" in order to avoid name collision.)", child->name(), new_name));
+		child->setName(new_name, NameHandling::AutoUnique, parent);
+	}
+
+	new AspectChildAddCmd(d, child, index, parent);
+
+	if (execute)
+		exec(parent);
 }
 
 /**
@@ -557,10 +590,10 @@ void AbstractAspect::insertChildBeforeFast(AbstractAspect* child, AbstractAspect
 	if (index == -1)
 		index = d->m_children.count();
 
-	Q_EMIT aspectAboutToBeAdded(this, nullptr, child);
+	Q_EMIT childAspectAboutToBeAdded(this, nullptr, child);
 	d->insertChild(index, child);
 	child->finalizeAdd();
-	Q_EMIT aspectAdded(child);
+	Q_EMIT childAspectAdded(child);
 }
 
 /**
@@ -570,12 +603,26 @@ void AbstractAspect::insertChildBeforeFast(AbstractAspect* child, AbstractAspect
  * i.e., the aspect is deleted by the undo command.
  * \sa reparent()
  */
-void AbstractAspect::removeChild(AbstractAspect* child) {
+void AbstractAspect::removeChild(AbstractAspect* child, QUndoCommand* parent) {
 	// QDEBUG(Q_FUNC_INFO << ", CHILD =" << child << ", PARENT =" << child->parentAspect())
 
-	beginMacro(i18n("%1: remove %2", name(), child->name()));
-	exec(new AspectChildRemoveCmd(d, child));
-	endMacro();
+	bool execute = false;
+	if (!parent) {
+		execute = true;
+		parent = new QUndoCommand(i18n("%1: remove %2", name(), child->name()));
+	}
+
+	new AspectChildRemoveCmd(d, child, parent);
+
+	if (execute)
+		exec(parent);
+}
+
+void AbstractAspect::moveChild(AbstractAspect* child, int steps, QUndoCommand* parent) {
+	auto* command = new AspectChildMoveCmd(d, child, steps, parent);
+	if (!parent)
+		exec(command);
+	// otherwise handled by parent
 }
 
 /**
@@ -594,9 +641,9 @@ void AbstractAspect::removeAllChildren() {
 	}
 
 	while (current) {
-		Q_EMIT aspectAboutToBeRemoved(current);
+		Q_EMIT childAspectAboutToBeRemoved(current);
 		exec(new AspectChildRemoveCmd(d, current));
-		Q_EMIT aspectRemoved(this, nextSibling, current);
+		Q_EMIT childAspectRemoved(this, nextSibling, current);
 
 		current = nextSibling;
 		if (i != children_list.constEnd() && ++i != children_list.constEnd())
@@ -652,33 +699,25 @@ const QVector<AbstractAspect*>& AbstractAspect::children() const {
 /**
  * \brief Remove me from my parent's list of children.
  */
-void AbstractAspect::remove() {
+void AbstractAspect::remove(QUndoCommand* parent) {
 	if (parentAspect())
-		parentAspect()->removeChild(this);
+		parentAspect()->removeChild(this, parent);
+}
+
+void AbstractAspect::remove() {
+	remove(nullptr);
 }
 
 void AbstractAspect::moveUp() {
 	auto* parent = parentAspect();
-	int index = parent->indexOfChild<AbstractAspect>(this);
-	auto* sibling = parent->child<AbstractAspect>(index - 1);
-	beginMacro(i18n("%1: move up", name()));
-	setMoved(true);
-	remove();
-	parent->insertChildBefore(this, sibling);
-	setMoved(false);
-	endMacro();
+	if (parent)
+		parent->moveChild(this, -1);
 }
 
 void AbstractAspect::moveDown() {
 	auto* parent = parentAspect();
-	int index = parent->indexOfChild<AbstractAspect>(this);
-	auto* sibling = parent->child<AbstractAspect>(index + 2);
-	beginMacro(i18n("%1: move down", name()));
-	setMoved(true);
-	remove();
-	parent->insertChildBefore(this, sibling);
-	setMoved(false);
-	endMacro();
+	if (parent)
+		parent->moveChild(this, 1);
 }
 
 /*!
@@ -713,7 +752,7 @@ bool AbstractAspect::pasted() const {
  * copies the aspect to the clipboard. The standard XML-serialization
  * via AbstractAspect::load() is used.
  */
-void AbstractAspect::copy() const {
+void AbstractAspect::copy() {
 	QString output;
 	QXmlStreamWriter writer(&output);
 	writer.writeStartDocument();
@@ -727,9 +766,18 @@ void AbstractAspect::copy() const {
 	writer.writeAttribute(QStringLiteral("value"), QString::number(static_cast<int>(m_type)));
 	writer.writeEndElement();
 
+	setSuppressWriteUuid(true);
+	const auto& children = this->children(AspectType::AbstractAspect, {ChildIndexFlag::IncludeHidden, ChildIndexFlag::Recursive});
+	for (const auto& child : children)
+		child->setSuppressWriteUuid(true);
+
 	// write the aspect itself
 	save(&writer);
 
+	for (const auto& child : children)
+		child->setSuppressWriteUuid(false);
+	setSuppressWriteUuid(false);
+
 	writer.writeEndElement(); // end the root-element
 	writer.writeEndDocument();
 	QApplication::clipboard()->setText(output);
@@ -890,7 +938,7 @@ void AbstractAspect::writeCommentElement(QXmlStreamWriter* writer) const {
  * \brief Load comment from an XML element
  */
 bool AbstractAspect::readCommentElement(XmlStreamReader* reader) {
-	setComment(reader->readElementText());
+	d->m_comment = reader->readElementText();
 	return true;
 }
 
@@ -900,6 +948,8 @@ bool AbstractAspect::readCommentElement(XmlStreamReader* reader) {
 void AbstractAspect::writeBasicAttributes(QXmlStreamWriter* writer) const {
 	writer->writeAttribute(QLatin1String("creation_time"), creationTime().toString(QLatin1String("yyyy-dd-MM hh:mm:ss:zzz")));
 	writer->writeAttribute(QLatin1String("name"), name());
+	if (!d->m_suppressWriteUuid)
+		writer->writeAttribute(QLatin1String("uuid"), uuid().toString());
 }
 
 /**
@@ -925,11 +975,15 @@ bool AbstractAspect::readBasicAttributes(XmlStreamReader* reader) {
 	} else {
 		QDateTime creation_time = QDateTime::fromString(str, QLatin1String("yyyy-dd-MM hh:mm:ss:zzz"));
 		if (creation_time.isValid())
-			d->m_creation_time = creation_time;
+			d->m_creation_time = std::move(creation_time);
 		else
 			d->m_creation_time = QDateTime::currentDateTime();
 	}
 
+	str = attribs.value(QLatin1String("uuid")).toString();
+	if (!str.isEmpty()) {
+		d->m_uuid = QUuid(str);
+	}
 	return true;
 }
 
@@ -971,7 +1025,7 @@ void AbstractAspect::exec(QUndoCommand* cmd) {
 		}
 
 		if (project())
-			project()->setChanged(true);
+			setProjectChanged(true);
 	} else {
 		cmd->redo();
 		delete cmd;
@@ -1117,7 +1171,7 @@ QString AbstractAspect::uniqueNameFor(const QString& name, const QStringList& na
 	if (last_non_digit >= 0 && base[last_non_digit].category() != QChar::Separator_Space)
 		base.append(QLatin1Char(' '));
 
-	int new_nr = name.rightRef(name.size() - base.size()).toInt();
+	int new_nr = QStringView(name).right(name.size() - base.size()).toInt();
 	QString new_name;
 	do
 		new_name = base + QString::number(++new_nr);
@@ -1129,10 +1183,18 @@ QString AbstractAspect::uniqueNameFor(const QString& name, const QStringList& na
 void AbstractAspect::connectChild(AbstractAspect* child) {
 	connect(child, &AbstractAspect::aspectDescriptionAboutToChange, this, &AbstractAspect::aspectDescriptionAboutToChange);
 	connect(child, &AbstractAspect::aspectDescriptionChanged, this, &AbstractAspect::aspectDescriptionChanged);
-	connect(child, &AbstractAspect::aspectAboutToBeAdded, this, &AbstractAspect::aspectAboutToBeAdded);
-	connect(child, &AbstractAspect::aspectAdded, this, &AbstractAspect::aspectAdded);
-	connect(child, &AbstractAspect::aspectAboutToBeRemoved, this, &AbstractAspect::aspectAboutToBeRemoved);
-	connect(child, &AbstractAspect::aspectRemoved, this, &AbstractAspect::aspectRemoved);
+	connect(child,
+			QOverload<const AbstractAspect*, const AbstractAspect*, const AbstractAspect*>::of(&AbstractAspect::childAspectAboutToBeAdded),
+			this,
+			QOverload<const AbstractAspect*, const AbstractAspect*, const AbstractAspect*>::of(&AbstractAspect::childAspectAboutToBeAdded));
+	connect(child, &AbstractAspect::childAspectAdded, this, &AbstractAspect::childAspectAdded);
+	connect(child,
+			QOverload<const AbstractAspect*>::of(&AbstractAspect::childAspectAboutToBeRemoved),
+			this,
+			QOverload<const AbstractAspect*>::of(&AbstractAspect::childAspectAboutToBeRemoved));
+	connect(child, &AbstractAspect::childAspectRemoved, this, &AbstractAspect::childAspectRemoved);
+	connect(child, &AbstractAspect::childAspectAboutToBeMoved, this, &AbstractAspect::childAspectAboutToBeMoved);
+	connect(child, &AbstractAspect::childAspectMoved, this, &AbstractAspect::childAspectMoved);
 	connect(child, &AbstractAspect::aspectHiddenAboutToChange, this, &AbstractAspect::aspectHiddenAboutToChange);
 	connect(child, &AbstractAspect::aspectHiddenChanged, this, &AbstractAspect::aspectHiddenChanged);
 	connect(child, &AbstractAspect::statusInfo, this, &AbstractAspect::statusInfo);
@@ -1141,9 +1203,9 @@ void AbstractAspect::connectChild(AbstractAspect* child) {
 	connect(child, &AbstractAspect::deselected, this, &AbstractAspect::childDeselected);
 }
 
-//##############################################################################
-//######################  Private implementation ###############################
-//##############################################################################
+// ##############################################################################
+// ######################  Private implementation ###############################
+// ##############################################################################
 AbstractAspectPrivate::AbstractAspectPrivate(AbstractAspect* owner, const QString& name)
 	: m_name(name.isEmpty() ? QLatin1String("1") : name)
 	, q(owner) {
@@ -1172,14 +1234,3 @@ int AbstractAspectPrivate::indexOfChild(const AbstractAspect* child) const {
 
 	return -1;
 }
-
-int AbstractAspectPrivate::removeChild(AbstractAspect* child) {
-	// QDEBUG(Q_FUNC_INFO << " CHILD = " << child << ", PARENT =" << child->parentAspect())
-	int index = indexOfChild(child);
-	Q_ASSERT(index != -1);
-	m_children.removeAll(child);
-	QObject::disconnect(child, nullptr, q, nullptr);
-	child->setParentAspect(nullptr);
-	// QDEBUG(Q_FUNC_INFO << " DONE. CHILD = " << child)
-	return index;
-}
