@@ -14,6 +14,7 @@
 
 #include <QDateTime>
 #include <QList>
+#include <QUuid>
 
 class AbstractAspect;
 
@@ -24,7 +25,6 @@ public:
 
 	void insertChild(int index, AbstractAspect*);
 	int indexOfChild(const AbstractAspect*) const;
-	int removeChild(AbstractAspect*);
 
 public:
 	QVector<AbstractAspect*> m_children;
@@ -39,6 +39,8 @@ public:
 	bool m_undoAware{true};
 	bool m_isLoading{false};
 	bool m_pasted{false};
+	QUuid m_uuid{QUuid::createUuid()};
+	bool m_suppressWriteUuid{false};
 };
 
 #endif // ifndef ASPECT_PRIVATE_H
