@@ -45,6 +45,9 @@ QWidget* Workbook::view() const {
 	if (!m_partView) {
 		m_view = new WorkbookView(const_cast<Workbook*>(this));
 		m_partView = m_view;
+		connect(this, &Workbook::viewAboutToBeDeleted, [this]() {
+			m_view = nullptr;
+		});
 	}
 	return m_partView;
 }
@@ -161,9 +164,9 @@ void Workbook::processDropEvent(const QVector<quintptr>& vec) {
 		aspect->reparent(this);
 	}
 }
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 
 //! Save as XML
 void Workbook::save(QXmlStreamWriter* writer) const {
