@@ -151,6 +151,7 @@ void ColumnSetModeCmd::redo() {
 		m_old_out_filter = m_col->outputFilter();
 
 		// do the conversion
+		m_col->setLabelsMode(m_mode); // must be done before setColumnMode, because setColumnMode() sends signal to dock
 		m_col->setColumnMode(m_mode);
 
 		// save new values
@@ -171,6 +172,7 @@ void ColumnSetModeCmd::redo() {
 void ColumnSetModeCmd::undo() {
 	// reset to old values
 	m_col->replaceModeData(m_old_mode, m_old_data, m_old_in_filter, m_old_out_filter);
+	// setLabelsMode will be done in replaceModeData()
 
 	m_undone = true;
 }
@@ -386,6 +388,8 @@ ColumnInsertRowsCmd::ColumnInsertRowsCmd(ColumnPrivate* col, int before, int cou
  */
 void ColumnInsertRowsCmd::redo() {
 	m_col->insertRows(m_before, m_count);
+	m_col->m_owner->updateFormula(); // only needed in redo
+	finalize();
 }
 
 /**
@@ -393,6 +397,12 @@ void ColumnInsertRowsCmd::redo() {
  */
 void ColumnInsertRowsCmd::undo() {
 	m_col->removeRows(m_before, m_count);
+	finalize();
+}
+
+void ColumnInsertRowsCmd::finalize() const {
+	if (!m_col->m_owner->m_suppressDataChangedSignal)
+		Q_EMIT m_col->m_owner->dataChanged(m_col->m_owner);
 }
 
 /** ***************************************************************************
@@ -472,6 +482,7 @@ void ColumnRemoveRowsCmd::redo() {
 		m_formulas = m_col->formulaAttribute();
 	}
 	m_col->removeRows(m_first, m_count);
+	finalize();
 }
 
 /**
@@ -482,6 +493,12 @@ void ColumnRemoveRowsCmd::undo() {
 	m_col->copy(m_backup, 0, m_first, m_data_row_count);
 	m_col->resizeTo(m_old_size);
 	m_col->replaceFormulas(m_formulas);
+	finalize();
+}
+
+void ColumnRemoveRowsCmd::finalize() const {
+	if (!m_col->m_owner->m_suppressDataChangedSignal)
+		Q_EMIT m_col->m_owner->dataChanged(m_col->m_owner);
 }
 
 /** ***************************************************************************
@@ -677,13 +694,16 @@ ColumnSetGlobalFormulaCmd::ColumnSetGlobalFormulaCmd(ColumnPrivate* col,
 													 QString formula,
 													 QStringList variableNames,
 													 QVector<Column*> variableColumns,
-													 bool autoUpdate)
-	: QUndoCommand()
+													 bool autoUpdate,
+													 bool autoResize,
+													 QUndoCommand* parent)
+	: QUndoCommand(parent)
 	, m_col(col)
 	, m_newFormula(std::move(formula))
 	, m_newVariableNames(std::move(variableNames))
 	, m_newVariableColumns(std::move(variableColumns))
-	, m_newAutoUpdate(autoUpdate) {
+	, m_newAutoUpdate(autoUpdate)
+	, m_newAutoResize(autoResize) {
 	setText(i18n("%1: set formula", col->name()));
 }
 
@@ -695,6 +715,7 @@ void ColumnSetGlobalFormulaCmd::redo() {
 			m_variableColumns << d.m_column;
 		}
 		m_autoUpdate = m_col->formulaAutoUpdate();
+		m_autoResize = m_col->formulaAutoResize();
 		m_copied = true;
 	}
 
@@ -703,14 +724,14 @@ void ColumnSetGlobalFormulaCmd::redo() {
 		if (i < m_newVariableColumns.size()) // names may be defined but without column
 			formulaData << Column::FormulaData(m_newVariableNames.at(i), m_newVariableColumns.at(i));
 
-	m_col->setFormula(m_newFormula, formulaData, m_newAutoUpdate);
+	m_col->setFormula(m_newFormula, formulaData, m_newAutoUpdate, m_newAutoResize);
 }
 
 void ColumnSetGlobalFormulaCmd::undo() {
 	QVector<Column::FormulaData> formulaData;
 	for (int i = 0; i < m_variableNames.count(); i++)
 		formulaData << Column::FormulaData(m_variableNames.at(i), m_variableColumns.at(i));
-	m_col->setFormula(m_formula, formulaData, m_autoUpdate);
+	m_col->setFormula(m_formula, formulaData, m_autoUpdate, m_autoResize);
 }
 
 /** ***************************************************************************
