@@ -30,8 +30,8 @@
 Datapicker::Datapicker(const QString& name, const bool loading)
 	: AbstractPart(name, AspectType::Datapicker)
 	, m_transform(new Transform()) {
-	connect(this, &Datapicker::aspectAdded, this, &Datapicker::handleAspectAdded);
-	connect(this, &Datapicker::aspectAboutToBeRemoved, this, &Datapicker::handleAspectAboutToBeRemoved);
+	connect(this, &Datapicker::childAspectAdded, this, &Datapicker::handleAspectAdded);
+	connect(this, &Datapicker::childAspectAboutToBeRemoved, this, &Datapicker::handleAspectAboutToBeRemoved);
 
 	if (!loading)
 		init();
@@ -71,6 +71,9 @@ QWidget* Datapicker::view() const {
 	if (!m_partView) {
 		m_view = new DatapickerView(const_cast<Datapicker*>(this));
 		m_partView = m_view;
+		connect(this, &Datapicker::viewAboutToBeDeleted, [this]() {
+			m_view = nullptr;
+		});
 	}
 	return m_partView;
 }
@@ -316,9 +319,9 @@ void Datapicker::handleChildAspectAdded(const AbstractAspect* aspect) {
 	}
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 
 //! Save as XML
 void Datapicker::save(QXmlStreamWriter* writer) const {
