@@ -24,7 +24,6 @@
 #include "kdefrontend/worksheet/ExportWorksheetDialog.h"
 
 #include <QBuffer>
-#include <QDesktopWidget>
 #include <QDir>
 #include <QFileInfo>
 #include <QGraphicsScene>
@@ -32,6 +31,7 @@
 #include <QPrintDialog>
 #include <QPrintPreviewDialog>
 #include <QPrinter>
+#include <QScreen>
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -52,8 +52,9 @@ DatapickerImage::DatapickerImage(const QString& name, bool loading)
 	, saturationBins(new int[ImageEditor::colorAttributeMax(ColorAttributes::Saturation) + 1])
 	, valueBins(new int[ImageEditor::colorAttributeMax(ColorAttributes::Value) + 1])
 	, intensityBins(new int[ImageEditor::colorAttributeMax(ColorAttributes::Intensity) + 1])
-	, d(new DatapickerImagePrivate(this))
+	, d_ptr(new DatapickerImagePrivate(this))
 	, m_segments(new Segments(this)) {
+	Q_D(DatapickerImage);
 	if (!loading)
 		init();
 	else {
@@ -76,33 +77,34 @@ DatapickerImage::~DatapickerImage() {
 	delete[] intensityBins;
 	delete[] foregroundBins;
 	delete m_segments;
-	delete d;
+	delete d_ptr;
 }
 
 void DatapickerImage::init() {
+	Q_D(DatapickerImage);
 	KConfig config;
-	KConfigGroup group = config.group("DatapickerImage");
+	KConfigGroup group = config.group(QStringLiteral("DatapickerImage"));
 
 	// general properties
-	d->fileName = group.readEntry("FileName", QString());
-	d->rotationAngle = group.readEntry("RotationAngle", 0.0);
-	d->minSegmentLength = group.readEntry("MinSegmentLength", 30);
-	d->pointSeparation = group.readEntry("PointSeparation", 30);
-	d->axisPoints.type = static_cast<GraphType>(group.readEntry("GraphType", static_cast<int>(GraphType::Linear)));
-	d->axisPoints.ternaryScale = group.readEntry("TernaryScale", 1);
+	d->fileName = group.readEntry(QStringLiteral("FileName"), QString());
+	d->rotationAngle = group.readEntry(QStringLiteral("RotationAngle"), 0.0);
+	d->minSegmentLength = group.readEntry(QStringLiteral("MinSegmentLength"), 30);
+	d->pointSeparation = group.readEntry(QStringLiteral("PointSeparation"), 30);
+	d->axisPoints.type = static_cast<GraphType>(group.readEntry(QStringLiteral("GraphType"), static_cast<int>(GraphType::Linear)));
+	d->axisPoints.ternaryScale = group.readEntry(QStringLiteral("TernaryScale"), 1);
 
 	// edit image settings
 	d->plotImageType = DatapickerImage::PlotImageType::OriginalImage;
-	d->settings.foregroundThresholdHigh = group.readEntry("ForegroundThresholdHigh", d->settings.foregroundThresholdHigh);
-	d->settings.foregroundThresholdLow = group.readEntry("ForegroundThresholdLow", d->settings.foregroundThresholdLow);
-	d->settings.hueThresholdHigh = group.readEntry("HueThresholdHigh", d->settings.hueThresholdHigh);
-	d->settings.hueThresholdLow = group.readEntry("HueThresholdLow", d->settings.hueThresholdLow);
-	d->settings.intensityThresholdHigh = group.readEntry("IntensityThresholdHigh", d->settings.intensityThresholdHigh);
-	d->settings.intensityThresholdLow = group.readEntry("IntensityThresholdLow", d->settings.intensityThresholdLow);
-	d->settings.saturationThresholdHigh = group.readEntry("SaturationThresholdHigh", d->settings.saturationThresholdHigh);
-	d->settings.saturationThresholdLow = group.readEntry("SaturationThresholdLow", d->settings.saturationThresholdLow);
-	d->settings.valueThresholdHigh = group.readEntry("ValueThresholdHigh", d->settings.valueThresholdHigh);
-	d->settings.valueThresholdLow = group.readEntry("ValueThresholdLow", d->settings.valueThresholdLow);
+	d->settings.foregroundThresholdHigh = group.readEntry(QStringLiteral("ForegroundThresholdHigh"), d->settings.foregroundThresholdHigh);
+	d->settings.foregroundThresholdLow = group.readEntry(QStringLiteral("ForegroundThresholdLow"), d->settings.foregroundThresholdLow);
+	d->settings.hueThresholdHigh = group.readEntry(QStringLiteral("HueThresholdHigh"), d->settings.hueThresholdHigh);
+	d->settings.hueThresholdLow = group.readEntry(QStringLiteral("HueThresholdLow"), d->settings.hueThresholdLow);
+	d->settings.intensityThresholdHigh = group.readEntry(QStringLiteral("IntensityThresholdHigh"), d->settings.intensityThresholdHigh);
+	d->settings.intensityThresholdLow = group.readEntry(QStringLiteral("IntensityThresholdLow"), d->settings.intensityThresholdLow);
+	d->settings.saturationThresholdHigh = group.readEntry(QStringLiteral("SaturationThresholdHigh"), d->settings.saturationThresholdHigh);
+	d->settings.saturationThresholdLow = group.readEntry(QStringLiteral("SaturationThresholdLow"), d->settings.saturationThresholdLow);
+	d->settings.valueThresholdHigh = group.readEntry(QStringLiteral("ValueThresholdHigh"), d->settings.valueThresholdHigh);
+	d->settings.valueThresholdLow = group.readEntry(QStringLiteral("ValueThresholdLow"), d->settings.valueThresholdLow);
 
 	// reference point symbol properties
 	d->symbol = new Symbol(QString());
@@ -115,7 +117,8 @@ void DatapickerImage::init() {
 		d->retransform();
 	});
 	d->symbol->init(group);
-	d->pointVisibility = group.readEntry("PointVisibility", true);
+
+	d->pointVisibility = group.readEntry(QStringLiteral("PointVisibility"), true);
 }
 
 /*!
@@ -206,14 +209,17 @@ void DatapickerImage::setSegmentsHoverEvent(const bool on) {
 }
 
 QGraphicsScene* DatapickerImage::scene() const {
+	Q_D(const DatapickerImage);
 	return d->m_scene;
 }
 
 QRectF DatapickerImage::pageRect() const {
+	Q_D(const DatapickerImage);
 	return d->m_scene->sceneRect();
 }
 
 void DatapickerImage::setPlotImageType(const DatapickerImage::PlotImageType type) {
+	Q_D(DatapickerImage);
 	d->plotImageType = type;
 	if (d->plotImageType == DatapickerImage::PlotImageType::ProcessedImage)
 		d->discretize();
@@ -221,24 +227,89 @@ void DatapickerImage::setPlotImageType(const DatapickerImage::PlotImageType type
 	Q_EMIT requestUpdate();
 }
 
-int DatapickerImage::currentSelectedReferencePoint() {
+int DatapickerImage::currentSelectedReferencePoint() const {
 	return m_currentRefPoint;
 }
 
-DatapickerImage::PlotImageType DatapickerImage::plotImageType() {
+DatapickerImage::PlotImageType DatapickerImage::plotImageType() const {
+	Q_D(const DatapickerImage);
 	return d->plotImageType;
 }
 
-bool DatapickerImage::setOriginalImage(const QString& fileName) {
-	originalPlotImage.load(fileName);
-	setEmbedded(true);
-	return d->uploadImage(QStringLiteral(""));
+class DatapickerImageSetOriginalImageCmd : public QUndoCommand {
+public:
+	DatapickerImageSetOriginalImageCmd(DatapickerImage::Private* target,
+									   const QImage& newImage,
+									   const QString& filename,
+									   bool embedded,
+									   const KLocalizedString& description,
+									   QUndoCommand* parent = nullptr)
+		: QUndoCommand(parent)
+		, m_img(newImage)
+		, m_filename(filename)
+		, m_embedded(embedded)
+		, m_target(target) {
+		setText(description.subs(m_target->name()).toString());
+	}
+	virtual void redo() override {
+		const QImage tmp = m_target->q->originalPlotImage;
+		const QString tmpFilename = m_target->q->fileName();
+		const bool tmpEmbedded = m_target->q->embedded();
+
+		if (m_embedded && !m_img.isNull())
+			m_target->q->originalPlotImage = m_img;
+		else
+			m_target->q->originalPlotImage.load(m_filename);
+		m_target->fileName = m_filename;
+		m_target->embedded = m_embedded;
+
+		if (tmpEmbedded)
+			m_img = tmp;
+		else
+			m_img = QImage();
+		m_filename = tmpFilename;
+		m_embedded = tmpEmbedded;
+		QUndoCommand::redo(); // redo all childs
+
+		finalize();
+		Q_EMIT m_target->q->fileNameChanged(m_target->fileName);
+		Q_EMIT m_target->q->embeddedChanged(m_target->embedded);
+	}
+
+	virtual void undo() override {
+		redo();
+	}
+
+	void finalize() {
+		m_target->updateImage();
+	}
+
+private:
+	QImage m_img;
+	QString m_filename;
+	bool m_embedded;
+	DatapickerImage::Private* m_target;
+};
+
+// 1. Image from clipboard, 2. file from path (embedded or not embedded)
+//     -> important to store qimage, because otherwise image is lost when doing undo
+// 1. Image from clipboard, 2. image from clipboard
+//     -> important to store qimage, because otherwise image is lost when doing undo
+// 1. Image from path (embedded or not embedded), 2. image from clipboard
+//     -> important to store qimage, because when doing redo after undo the image must be available
+// 1. Image from path (not embedded), 2. image from path
+//     -> storing qimage is not important
+// 1. Image from path (embedded), 2. image from path
+//     -> storing qimage is important because otherwise if undo and path is anymore valid image is anymore available
+
+void DatapickerImage::setImage(const QString& fileName, bool embedded) {
+	return setImage(QImage(), fileName, embedded);
 }
 
-bool DatapickerImage::setOriginalImage(const QImage& image) {
-	originalPlotImage = image;
-	setEmbedded(true);
-	return d->uploadImage(QStringLiteral(""));
+void DatapickerImage::setImage(const QImage& image, const QString& filename, bool embedded) {
+	Q_D(DatapickerImage);
+	if (image != originalPlotImage || filename != fileName() || embedded != this->embedded())
+		exec(new DatapickerImageSetOriginalImageCmd(d, image, filename, embedded, ki18n("%1: upload image")));
 }
 
 /* =============================== getter methods for background options ================================= */
@@ -254,19 +325,14 @@ BASIC_D_READER_IMPL(DatapickerImage, int, minSegmentLength, minSegmentLength)
 
 // symbols
 Symbol* DatapickerImage::symbol() const {
-	// 	Q_D(const DatapickerImage);
+	Q_D(const DatapickerImage);
 	return d->symbol;
 }
 
 BASIC_D_READER_IMPL(DatapickerImage, bool, pointVisibility, pointVisibility)
 /* ============================ setter methods and undo commands  for background options  ================= */
-STD_SETTER_CMD_IMPL_F_S(DatapickerImage, SetFileName, QString, fileName, updateFileName)
 void DatapickerImage::setFileName(const QString& fileName) {
-	if (fileName != d->fileName) {
-		beginMacro(i18n("%1: upload new image", name()));
-		exec(new DatapickerImageSetFileNameCmd(d, fileName, ki18n("%1: upload image")));
-		endMacro();
-	}
+	setImage(fileName, embedded());
 }
 
 class DatapickerImageSetRelativeFilePathCmd : public StandardSetterCmd<DatapickerImage::Private, bool> {
@@ -287,43 +353,56 @@ public:
 				fi.setFile(m_target->fileName);
 				filename = fi.absoluteFilePath();
 			}
-			m_target->q->setFileName(filename);
+			// setting relative is only possible if the image is not embedded!
+			m_target->q->setImage(filename, false);
 		}
-		emit m_target->q->relativeFilePathChanged(m_target->*m_field);
+		Q_EMIT m_target->q->relativeFilePathChanged(m_target->*m_field);
 	}
 };
 
 void DatapickerImage::setRelativeFilePath(bool relative) {
-	if (relative != d->isRelativeFilePath)
+	Q_D(DatapickerImage);
+	if (relative != d->isRelativeFilePath) {
+		beginMacro(i18n("%1: upload new image", name()));
 		exec(new DatapickerImageSetRelativeFilePathCmd(d, relative, ki18n("%1: upload image")));
+		endMacro();
+	}
 }
 
-STD_SETTER_CMD_IMPL_F_S(DatapickerImage, SetEmbedded, bool, embedded, updateFileName)
 void DatapickerImage::setEmbedded(bool embedded) {
-	if (embedded != d->embedded)
-		exec(new DatapickerImageSetEmbeddedCmd(d, embedded, ki18n("%1: embed image")));
+	Q_D(DatapickerImage);
+	if (embedded != d->embedded) {
+		if (embedded)
+			setImage(originalPlotImage, fileName(), true);
+		else
+			setImage(fileName(), false);
+	}
 }
 
 STD_SETTER_CMD_IMPL_S(DatapickerImage, SetRotationAngle, float, rotationAngle)
 void DatapickerImage::setRotationAngle(float angle) {
+	Q_D(DatapickerImage);
 	if (angle != d->rotationAngle)
 		exec(new DatapickerImageSetRotationAngleCmd(d, angle, ki18n("%1: set rotation angle")));
 }
 
 STD_SETTER_CMD_IMPL_S(DatapickerImage, SetAxisPoints, DatapickerImage::ReferencePoints, axisPoints)
 void DatapickerImage::setAxisPoints(const DatapickerImage::ReferencePoints& points) {
+	Q_D(DatapickerImage);
 	if (memcmp(&points, &d->axisPoints, sizeof(points)) != 0) // valgrind: Conditional jump or move depends on uninitialised value(s)
 		exec(new DatapickerImageSetAxisPointsCmd(d, points, ki18n("%1: set Axis points")));
 }
 
 STD_SETTER_CMD_IMPL_F_S(DatapickerImage, SetSettings, DatapickerImage::EditorSettings, settings, discretize)
 void DatapickerImage::setSettings(const DatapickerImage::EditorSettings& editorSettings) {
+	Q_D(DatapickerImage);
 	if (memcmp(&editorSettings, &d->settings, sizeof(editorSettings)) != 0)
 		exec(new DatapickerImageSetSettingsCmd(d, editorSettings, ki18n("%1: set editor settings")));
 }
 
 STD_SETTER_CMD_IMPL_F_S(DatapickerImage, SetMinSegmentLength, int, minSegmentLength, makeSegments)
 void DatapickerImage::setminSegmentLength(const int value) {
+	Q_D(DatapickerImage);
 	if (d->minSegmentLength != value)
 		exec(new DatapickerImageSetMinSegmentLengthCmd(d, value, ki18n("%1: set minimum segment length")));
 	;
@@ -331,6 +410,7 @@ void DatapickerImage::setminSegmentLength(const int value) {
 
 STD_SETTER_CMD_IMPL_F_S(DatapickerImage, SetPointVisibility, bool, pointVisibility, retransform)
 void DatapickerImage::setPointVisibility(const bool on) {
+	Q_D(DatapickerImage);
 	if (on != d->pointVisibility)
 		exec(new DatapickerImageSetPointVisibilityCmd(d, on, on ? ki18n("%1: set visible") : ki18n("%1: set invisible")));
 }
@@ -342,6 +422,7 @@ void DatapickerImage::setPrinting(bool on) const {
 }
 
 void DatapickerImage::setPlotPointsType(const PointsType pointsType) {
+	Q_D(DatapickerImage);
 	if (d->plotPointsType == pointsType)
 		return;
 
@@ -349,7 +430,7 @@ void DatapickerImage::setPlotPointsType(const PointsType pointsType) {
 
 	if (pointsType == DatapickerImage::PointsType::AxisPoints) {
 		// clear image
-		auto points = children<DatapickerPoint>(ChildIndexFlag::IncludeHidden);
+		const auto& points = children<DatapickerPoint>(ChildIndexFlag::IncludeHidden);
 		if (!points.isEmpty()) {
 			beginMacro(i18n("%1: remove all axis points", name()));
 
@@ -377,24 +458,25 @@ void DatapickerImage::setPlotPointsType(const PointsType pointsType) {
 }
 
 void DatapickerImage::setPointSeparation(const int value) {
+	Q_D(DatapickerImage);
 	d->pointSeparation = value;
 }
 
 void DatapickerImage::referencePointSelected(const DatapickerPoint* point) {
-	const auto points = children<DatapickerPoint>(AbstractAspect::ChildIndexFlag::IncludeHidden);
+	const auto& points = children<DatapickerPoint>(AbstractAspect::ChildIndexFlag::IncludeHidden);
 	for (int i = 0; i < points.count(); i++) {
 		if (points.at(i) == point) {
 			m_currentRefPoint = i;
-			emit referencePointSelected(i);
+			Q_EMIT referencePointSelected(i);
 			return;
 		}
 	}
 	m_currentRefPoint = -1;
 }
 
-//##############################################################################
-//######################  Private implementation ###############################
-//##############################################################################
+// ##############################################################################
+// ######################  Private implementation ###############################
+// ##############################################################################
 DatapickerImagePrivate::DatapickerImagePrivate(DatapickerImage* owner)
 	: q(owner)
 	, pageRect(0, 0, 1000, 1000)
@@ -408,17 +490,14 @@ QString DatapickerImagePrivate::name() const {
 void DatapickerImagePrivate::retransform() {
 	if (q->isLoading())
 		return;
-	auto points = q->children<DatapickerPoint>(AbstractAspect::ChildIndexFlag::IncludeHidden);
+
+	const auto& points = q->children<DatapickerPoint>(AbstractAspect::ChildIndexFlag::IncludeHidden);
 	for (auto* point : points)
 		point->retransform();
 }
 
-bool DatapickerImagePrivate::uploadImage(const QString& address) {
-	bool rc;
-	if (embedded)
-		rc = !q->originalPlotImage.isNull(); // already stored in originalPlotImage
-	else
-		rc = q->originalPlotImage.load(address);
+bool DatapickerImagePrivate::uploadImage() {
+	const bool rc = !q->originalPlotImage.isNull();
 
 	if (rc) {
 		// convert the image to 32bit-format if this is not the case yet
@@ -437,8 +516,8 @@ bool DatapickerImagePrivate::uploadImage(const QString& address) {
 		discretize();
 
 		// resize the screen
-		double w = Worksheet::convertToSceneUnits(q->originalPlotImage.width(), Worksheet::Unit::Inch) / QApplication::desktop()->physicalDpiX();
-		double h = Worksheet::convertToSceneUnits(q->originalPlotImage.height(), Worksheet::Unit::Inch) / QApplication::desktop()->physicalDpiX();
+		double w = Worksheet::convertToSceneUnits(q->originalPlotImage.width(), Worksheet::Unit::Inch) / QApplication::primaryScreen()->physicalDotsPerInchX();
+		double h = Worksheet::convertToSceneUnits(q->originalPlotImage.height(), Worksheet::Unit::Inch) / QApplication::primaryScreen()->physicalDotsPerInchY();
 		m_scene->setSceneRect(0, 0, w, h);
 		q->isLoaded = true;
 	}
@@ -472,17 +551,15 @@ DatapickerImagePrivate::~DatapickerImagePrivate() {
 	delete m_scene;
 }
 
-void DatapickerImagePrivate::updateFileName() {
+void DatapickerImagePrivate::updateImage() {
 	WAIT_CURSOR;
 	q->isLoaded = false;
 
-	QString address = fileName.trimmed();
-	if (address.isEmpty()) {
+	if (q->originalPlotImage.isNull()) {
 		// hide segments if they are visible
 		q->m_segments->setSegmentsVisible(false);
-	} else {
-		uploadImage(fileName);
-	}
+	} else
+		uploadImage();
 
 	auto points = q->parentAspect()->children<DatapickerPoint>(AbstractAspect::ChildIndexFlag::Recursive | AbstractAspect::ChildIndexFlag::IncludeHidden);
 	if (!points.isEmpty()) {
@@ -495,12 +572,13 @@ void DatapickerImagePrivate::updateFileName() {
 	RESET_CURSOR;
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 
 //! Save as XML
 void DatapickerImage::save(QXmlStreamWriter* writer) const {
+	Q_D(const DatapickerImage);
 	writer->writeStartElement(QStringLiteral("datapickerImage"));
 	writeBasicAttributes(writer);
 
@@ -577,7 +655,7 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 	if (!readBasicAttributes(reader))
 		return false;
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
+	Q_D(DatapickerImage);
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -610,91 +688,91 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("axisPointLogicalX1")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalX1")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalX1"));
 			else
 				d->axisPoints.logicalPos[0].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalY1")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalY1")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalY1"));
 			else
 				d->axisPoints.logicalPos[0].setY(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalZ1")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalZ1")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalZ1"));
 			else
 				d->axisPoints.logicalPos[0].setZ(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalX2")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalX2")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalX2"));
 			else
 				d->axisPoints.logicalPos[1].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalY2")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalY2")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalY2"));
 			else
 				d->axisPoints.logicalPos[1].setY(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalZ2")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalZ2")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalZ2"));
 			else
 				d->axisPoints.logicalPos[1].setZ(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalX3")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalX3")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalX3"));
 			else
 				d->axisPoints.logicalPos[2].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalY3")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalY3")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalY3"));
 			else
 				d->axisPoints.logicalPos[2].setY(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointLogicalZ3")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointLogicalZ3")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointLogicalZ3"));
 			else
 				d->axisPoints.logicalPos[2].setZ(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneX1")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneX1")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneX1"));
 			else
 				d->axisPoints.scenePos[0].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneY1")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneY1")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneY1"));
 			else
 				d->axisPoints.scenePos[0].setY(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneX2")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneX2")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneX2"));
 			else
 				d->axisPoints.scenePos[1].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneY2")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneY2")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneY2"));
 			else
 				d->axisPoints.scenePos[1].setY(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneX3")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneX3")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneX3"));
 			else
 				d->axisPoints.scenePos[2].setX(str.toDouble());
 
 			str = attribs.value(QStringLiteral("axisPointSceneY3")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("axisPointSceneY3")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("axisPointSceneY3"));
 			else
 				d->axisPoints.scenePos[2].setY(str.toDouble());
 
@@ -723,25 +801,25 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("pointRotationAngle")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("pointRotationAngle")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("pointRotationAngle"));
 			else
 				d->symbol->setRotationAngle(str.toDouble());
 
 			str = attribs.value(QStringLiteral("pointOpacity")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("pointOpacity")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("pointOpacity"));
 			else
 				d->symbol->setOpacity(str.toDouble());
 
 			str = attribs.value(QStringLiteral("pointSize")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("pointSize")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("pointSize"));
 			else
 				d->symbol->setSize(str.toDouble());
 
 			str = attribs.value(QStringLiteral("pointStyle")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("pointStyle")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("pointStyle"));
 			else
 				d->symbol->setStyle(static_cast<Symbol::Style>(str.toInt()));
 
@@ -749,26 +827,26 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 			QBrush brush;
 			str = attribs.value(QStringLiteral("brush_style")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("brush_style")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("brush_style"));
 			else
 				brush.setStyle(static_cast<Qt::BrushStyle>(str.toInt()));
 
 			QColor color;
 			str = attribs.value(QStringLiteral("brush_color_r")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("brush_color_r")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("brush_color_r"));
 			else
 				color.setRed(str.toInt());
 
 			str = attribs.value(QStringLiteral("brush_color_g")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("brush_color_g")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("brush_color_g"));
 			else
 				color.setGreen(str.toInt());
 
 			str = attribs.value(QStringLiteral("brush_color_b")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("brush_color_b")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("brush_color_b"));
 			else
 				color.setBlue(str.toInt());
 
@@ -779,25 +857,25 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 			QPen pen;
 			str = attribs.value(QStringLiteral("style")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("style")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("style"));
 			else
 				pen.setStyle(static_cast<Qt::PenStyle>(str.toInt()));
 
 			str = attribs.value(QStringLiteral("color_r")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("color_r")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("color_r"));
 			else
 				color.setRed(str.toInt());
 
 			str = attribs.value(QStringLiteral("color_g")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("color_g")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("color_g"));
 			else
 				color.setGreen(str.toInt());
 
 			str = attribs.value(QStringLiteral("color_b")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("color_b")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("color_b"));
 			else
 				color.setBlue(str.toInt());
 
@@ -805,7 +883,7 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("width")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("width")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("width"));
 			else
 				pen.setWidthF(str.toDouble());
 
@@ -825,13 +903,16 @@ bool DatapickerImage::load(XmlStreamReader* reader, bool preview) {
 				addChild(datapickerPoint);
 			}
 		} else { // unknown element
-			reader->raiseWarning(i18n("unknown element '%1'", reader->name().toString()));
+			reader->raiseUnknownElementWarning();
 			if (!reader->skipToEndElement())
 				return false;
 		}
 	}
 
-	d->uploadImage(d->fileName);
+	// No undo redo
+	if (originalPlotImage.isNull())
+		originalPlotImage.load(d->fileName);
+	d->uploadImage();
 	d->retransform();
 	return true;
 }
