@@ -29,7 +29,7 @@ MQTTClient::MQTTClient(const QString& name)
 	, m_client(new QMqttClient(this))
 	, m_willTimer(new QTimer(this)) {
 	// stop reading from the source before removing the child from the project
-	connect(this, &AbstractAspect::aspectAboutToBeRemoved, this, &MQTTClient::pauseReading);
+	connect(this, &AbstractAspect::childAspectAboutToBeRemoved, this, &MQTTClient::pauseReading);
 
 	connect(m_updateTimer, &QTimer::timeout, this, &MQTTClient::read);
 	connect(m_client, &QMqttClient::connected, this, &MQTTClient::onMQTTConnect);
@@ -981,9 +981,9 @@ void MQTTClient::stopWillTimer() const {
 	m_willTimer->stop();
 }
 
-//##############################################################################
-//#################################  SLOTS  ####################################
-//##############################################################################
+// ##############################################################################
+// #################################  SLOTS  ####################################
+// ##############################################################################
 
 /*!
  *\brief called periodically when update type is TimeInterval
@@ -1134,9 +1134,9 @@ void MQTTClient::subscriptionLoaded(const QString& name) {
 	}
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 /*!
   Saves as XML.
  */
