@@ -4,7 +4,7 @@
 	Description          : file I/O-filter related interface
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2009-2017 Alexander Semke <alexander.semke@web.de>
-	SPDX-FileCopyrightText: 2017 Stefan Gerlach <stefan.gerlach@uni.kn>
+	SPDX-FileCopyrightText: 2017-2024 Stefan Gerlach <stefan.gerlach@uni.kn>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -12,13 +12,16 @@
 #include "backend/datasources/filters/AbstractFileFilter.h"
 #include "backend/datasources/filters/SpiceFilter.h"
 #include "backend/datasources/filters/VectorBLFFilter.h"
+#include "backend/lib/hostprocess.h"
 #include "backend/lib/macros.h"
 
 #include <KLocalizedString>
+
 #include <QDateTime>
 #include <QImageReader>
 #include <QLocale>
 #include <QProcess>
+#include <QStandardPaths>
 
 bool AbstractFileFilter::isNan(const QString& s) {
 	const static QStringList nanStrings{QStringLiteral("NA"),
@@ -116,12 +119,29 @@ QStringList AbstractFileFilter::numberFormats() {
 }
 
 /*!
- * \brief AbstractFileFilter::lastErrors
- * Errors occured during last parse
- * \return
+ * Returns the last error that occured during the last parse step.
  */
-QStringList AbstractFileFilter::lastErrors() {
-	return QStringList();
+QString AbstractFileFilter::lastError() const {
+	return m_lastError;
+}
+
+void AbstractFileFilter::setLastError(const QString& error) {
+	m_lastError = error;
+}
+
+/*!
+ * Returns the list of warnings that occured during the last parse step.
+ */
+QStringList AbstractFileFilter::lastWarnings() const {
+	return m_lastWarnings;
+}
+
+void AbstractFileFilter::addWarning(const QString& warning) {
+	m_lastWarnings << warning;
+}
+
+void AbstractFileFilter::clearLastWarnings() {
+	m_lastWarnings.clear();
 }
 
 AbstractFileFilter::FileType AbstractFileFilter::fileType(const QString& fileName) {
@@ -129,10 +149,10 @@ AbstractFileFilter::FileType AbstractFileFilter::fileType(const QString& fileNam
 	QString fileInfo;
 #ifndef HAVE_WINDOWS
 	// check, if we can guess the file type by content
-	const QString fileFullPath = QStandardPaths::findExecutable(QLatin1String("file"));
+	const QString fileFullPath = safeExecutableName(QStringLiteral("file"));
 	if (!fileFullPath.isEmpty()) {
 		QProcess proc;
-		proc.start(fileFullPath, QStringList() << QStringLiteral("-b") << QStringLiteral("-z") << fileName);
+		startHostProcess(proc, fileFullPath, QStringList() << QStringLiteral("-b") << QStringLiteral("-z") << fileName);
 		if (!proc.waitForFinished(1000)) {
 			proc.kill();
 			DEBUG("ERROR: reading file type of file" << STDSTRING(fileName));
@@ -154,9 +174,13 @@ AbstractFileFilter::FileType AbstractFileFilter::fileType(const QString& fileNam
 		fileType = FileType::JSON;
 	} else if (SpiceFilter::isSpiceFile(fileName))
 		fileType = FileType::Spice;
-#ifdef HAVE_EXCEL // before ASCII, because XLSX is XML and XML is ASCII
+#ifdef HAVE_QXLSX // before ASCII, because XLSX is XML and XML is ASCII
 	else if (fileInfo.contains(QLatin1String("Microsoft Excel")) || fileName.endsWith(QLatin1String("xlsx"), Qt::CaseInsensitive))
-		fileType = FileType::Excel;
+		fileType = FileType::XLSX;
+#endif
+#ifdef HAVE_ORCUS // before ASCII, because ODS is XML and XML is ASCII
+	else if (fileInfo.contains(QLatin1String("OpenDocument Spreadsheet")) || fileName.endsWith(QLatin1String("ods"), Qt::CaseInsensitive))
+		fileType = FileType::Ods;
 #endif
 	else if (fileInfo.contains(QLatin1String("ASCII")) || fileName.endsWith(QLatin1String("txt"), Qt::CaseInsensitive)
 			 || fileName.endsWith(QLatin1String("csv"), Qt::CaseInsensitive) || fileName.endsWith(QLatin1String("dat"), Qt::CaseInsensitive)
@@ -214,8 +238,38 @@ AbstractFileFilter::FileType AbstractFileFilter::fileType(const QString& fileNam
 /*!
   returns the list of all supported data file formats
 */
-QStringList AbstractFileFilter::fileTypes() {
+/*QStringList AbstractFileFilter::fileTypes() {
+	// TODO: #ifdef HAVE_QXLSX?
 	return (QStringList() << i18n("ASCII Data") << i18n("Binary Data") << i18n("Image") << i18n("Excel") << i18n("Hierarchical Data Format 5 (HDF5)")
 						  << i18n("Network Common Data Format (NetCDF)") << i18n("Flexible Image Transport System Data Format (FITS)") << i18n("JSON Data")
 						  << i18n("ROOT (CERN) Histograms") << i18n("Spice") << i18n("SAS, Stata or SPSS"));
+}*/
+
+QString AbstractFileFilter::convertFromNumberToColumn(int n) {
+	// main code from https://www.geeksforgeeks.org/find-excel-column-name-given-number/
+	// Function to print column name for a given column number
+
+	char str[1000]; // To store result (column name)
+	int i = 0; // To store current index in str which is result
+
+	while (n > 0) {
+		// Find remainder
+		int rem = n % 26;
+
+		// If remainder is 0, then a 'Z' must be there in output
+		if (rem == 0) {
+			str[i++] = 'Z';
+			n = (n / 26) - 1;
+		} else // If remainder is non-zero
+		{
+			str[i++] = (rem - 1) + 'A';
+			n = n / 26;
+		}
+	}
+	str[i] = '\0';
+
+	// Reverse the string and print result
+	std::reverse(str, str + strlen(str));
+
+	return QLatin1String(str);
 }
