@@ -3,8 +3,8 @@
 	Project              : LabPlot
 	Description          : file I/O-filter related interface
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2009-2018 Alexander Semke <alexander.semke@web.de>
-	SPDX-FileCopyrightText: 2017 Stefan Gerlach <stefan.gerlach@uni.kn>
+	SPDX-FileCopyrightText: 2009-2024 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2017-2024 Stefan Gerlach <stefan.gerlach@uni.kn>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -15,11 +15,12 @@
 #include "backend/core/AbstractColumn.h"
 #include <QLocale>
 #include <QObject>
-#include <memory> // smart pointer
+#include <memory> // unique_ptr
 
 class AbstractDataSource;
 class XmlStreamReader;
 class QXmlStreamWriter;
+class KConfig;
 
 class AbstractFileFilter : public QObject {
 	Q_OBJECT
@@ -27,7 +28,7 @@ class AbstractFileFilter : public QObject {
 	Q_ENUMS(ImportMode)
 
 public:
-	enum class FileType { Ascii, Binary, Excel, Image, HDF5, NETCDF, FITS, JSON, ROOT, Spice, READSTAT, MATIO, VECTOR_BLF };
+	enum class FileType { Ascii, Binary, XLSX, Ods, Image, HDF5, NETCDF, FITS, JSON, ROOT, Spice, READSTAT, MATIO, VECTOR_BLF };
 	enum class ImportMode { Append, Prepend, Replace };
 
 	explicit AbstractFileFilter(FileType type)
@@ -40,17 +41,21 @@ public:
 	static AbstractColumn::ColumnMode columnMode(const QString& valueString, QString& dateTimeFormat, const QLocale& = QLocale());
 	static QString dateTimeFormat(const QString& valueString);
 	static QStringList numberFormats();
-	static AbstractFileFilter::FileType fileType(const QString&);
-	static QStringList fileTypes();
+	static FileType fileType(const QString&);
+	// static QStringList fileTypes();
+	static QString convertFromNumberToColumn(int n);
 
 	virtual void readDataFromFile(const QString& fileName, AbstractDataSource* = nullptr, ImportMode = ImportMode::Replace) = 0;
 	virtual void write(const QString& fileName, AbstractDataSource*) = 0;
 
-	virtual QStringList lastErrors();
+	QString lastError() const;
+	void setLastError(const QString&);
 
-	virtual void loadFilterSettings(const QString& filterName) = 0;
-	virtual void saveFilterSettings(const QString& filterName) const = 0;
+	QStringList lastWarnings() const;
+	void addWarning(const QString&);
+	void clearLastWarnings();
 
+	// save/load in the project XML
 	virtual void save(QXmlStreamWriter*) const = 0;
 	virtual bool load(XmlStreamReader*) = 0;
 
@@ -63,6 +68,8 @@ Q_SIGNALS:
 
 protected:
 	const FileType m_type;
+	QString m_lastError;
+	QStringList m_lastWarnings;
 };
 
 #endif
