@@ -11,7 +11,6 @@
 #ifndef ASCIIFILTERPRIVATE_H
 #define ASCIIFILTERPRIVATE_H
 
-class KFilterDev;
 class AbstractDataSource;
 class AbstractColumn;
 class AbstractAspect;
@@ -56,7 +55,7 @@ public:
 	void setPreparedForMQTT(bool, MQTTTopic*, const QString&);
 #endif
 
-	const AsciiFilter* q;
+	AsciiFilter* const q;
 
 	QString commentCharacter{QStringLiteral("#")};
 	QString separatingCharacter{QStringLiteral("auto")};
@@ -65,7 +64,7 @@ public:
 	QLocale locale{QLocale::C};
 	bool autoModeEnabled{true};
 	bool headerEnabled{true}; // read header from file
-	int headerLine{0}; // line to read header from. 0: no header line
+	int headerLine{1}; // line to read header from
 	bool skipEmptyParts{false};
 	bool simplifyWhitespacesEnabled{false};
 	double nanValue{qQNaN()};
@@ -90,7 +89,8 @@ private:
 	int m_actualStartRow{1};
 	int m_actualRows{0};
 	int m_actualCols{0};
-	int m_prepared{false};
+	bool m_prepared{false};
+	bool m_firstRead{true};
 	int m_columnOffset{0}; // indexes the "start column" in the datasource. Data will be imported starting from this column.
 	std::vector<void*> m_dataContainer; // pointers to the actual data containers
 
