@@ -14,7 +14,6 @@
 #include <limits>
 
 class BinaryFilterPrivate;
-class QStringList;
 class QIODevice;
 
 class BinaryFilter : public AbstractFileFilter {
@@ -34,15 +33,11 @@ public:
 	static QString fileInfoString(const QString&);
 
 	// read data from any device
-	void
-	readDataFromDevice(QIODevice&, AbstractDataSource* = nullptr, AbstractFileFilter::ImportMode = AbstractFileFilter::ImportMode::Replace, int lines = -1);
-	void readDataFromFile(const QString& fileName, AbstractDataSource*, AbstractFileFilter::ImportMode = AbstractFileFilter::ImportMode::Replace) override;
+	void readDataFromDevice(QIODevice&, AbstractDataSource* = nullptr, ImportMode = ImportMode::Replace, int lines = -1);
+	void readDataFromFile(const QString& fileName, AbstractDataSource*, ImportMode = ImportMode::Replace) override;
 	void write(const QString& fileName, AbstractDataSource*) override;
 	QVector<QStringList> preview(const QString& fileName, int lines);
 
-	void loadFilterSettings(const QString&) override;
-	void saveFilterSettings(const QString&) const override;
-
 	void setVectors(const size_t);
 	size_t vectors() const;
 
