@@ -12,7 +12,6 @@
 #include "backend/datasources/filters/AbstractFileFilter.h"
 
 class ImageFilterPrivate;
-class QStringList;
 
 class ImageFilter : public AbstractFileFilter {
 	Q_OBJECT
@@ -27,13 +26,9 @@ public:
 	static QStringList importFormats();
 	static QString fileInfoString(const QString&);
 
-	void
-	readDataFromFile(const QString& fileName, AbstractDataSource* = nullptr, AbstractFileFilter::ImportMode = AbstractFileFilter::ImportMode::Replace) override;
+	void readDataFromFile(const QString& fileName, AbstractDataSource* = nullptr, ImportMode = ImportMode::Replace) override;
 	void write(const QString& fileName, AbstractDataSource*) override;
 
-	void loadFilterSettings(const QString&) override;
-	void saveFilterSettings(const QString&) const override;
-
 	void setImportFormat(const ImageFilter::ImportFormat);
 	ImageFilter::ImportFormat importFormat() const;
 
