@@ -11,21 +11,23 @@
 #ifndef EXPRESSIONPARSER_H
 #define EXPRESSIONPARSER_H
 
+#include "backend/gsl/parser.h"
 #include "backend/lib/Range.h"
 #include "backend/worksheet/plots/cartesian/XYEquationCurve.h"
 #include <QVector>
 
-class QStringList;
-
 class ExpressionParser {
 public:
 	static ExpressionParser* getInstance();
 	static int functionArgumentCount(const QString& functionName);
+	static QString parameters(const QString& functionName);
 	static QString functionArgumentString(const QString& functionName, const XYEquationCurve::EquationType);
 	QString functionDescription(const QString& function);
 	QString constantDescription(const QString& constant);
+	void setSpecialFunction1(const char* function_name, func_t1Payload funct, std::shared_ptr<Payload> payload);
+	void setSpecialFunction2(const char* function_name, func_t2Payload funct, std::shared_ptr<Payload> payload);
 
-	bool isValid(const QString& expr, const QStringList& vars);
+	static bool isValid(const QString& expr, const QStringList& vars = QStringList());
 	QStringList getParameter(const QString& expr, const QStringList& vars);
 	bool evaluateCartesian(const QString& expr,
 						   Range<double> range,
@@ -49,7 +51,7 @@ public:
 						   QVector<double>* yVector,
 						   const QStringList& paramNames,
 						   const QVector<double>& paramValues);
-	bool evaluateCartesian(const QString& expr, const QStringList& vars, const QVector<QVector<double>*>& xVectors, QVector<double>* yVector);
+	static bool evaluateCartesian(const QString& expr, const QStringList& vars, const QVector<QVector<double>*>& xVectors, QVector<double>* yVector);
 	bool evaluatePolar(const QString& expr, const QString& min, const QString& max, int count, QVector<double>* xVector, QVector<double>* yVector);
 	bool evaluateParametric(const QString& expr1,
 							const QString& expr2,
@@ -58,18 +60,19 @@ public:
 							int count,
 							QVector<double>* xVector,
 							QVector<double>* yVector);
+	QString errorMessage() const;
 
 	const QStringList& functions();
 	const QStringList& functionsGroups();
-	const QStringList& functionsNames();
-	const QVector<int>& functionsGroupIndices();
+	const QStringList& functionsDescriptions();
+	const QVector<FunctionGroups>& functionsGroupIndices();
 
 	const QStringList& constants();
 	const QStringList& constantsGroups();
 	const QStringList& constantsNames();
 	const QStringList& constantsValues();
 	const QStringList& constantsUnits();
-	const QVector<int>& constantsGroupIndices();
+	const QVector<ConstantGroups>& constantsGroupIndices();
 
 private:
 	ExpressionParser();
@@ -82,14 +85,14 @@ private:
 
 	QStringList m_functions;
 	QStringList m_functionsGroups;
-	QStringList m_functionsNames;
-	QVector<int> m_functionsGroupIndex;
+	QStringList m_functionsDescription;
+	QVector<FunctionGroups> m_functionsGroupIndex;
 
 	QStringList m_constants;
 	QStringList m_constantsGroups;
-	QStringList m_constantsNames;
+	QStringList m_constantsDescription;
 	QStringList m_constantsValues;
 	QStringList m_constantsUnits;
-	QVector<int> m_constantsGroupIndex;
+	QVector<ConstantGroups> m_constantsGroupIndex;
 };
 #endif
