@@ -4,7 +4,7 @@
 	Description          : Spreadsheet with a MxN matrix data model
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2008-2009 Tilman Benkert <thzs@gmx.net>
-	SPDX-FileCopyrightText: 2015-2017 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2015-2024 Alexander Semke <alexander.semke@web.de>
 	SPDX-FileCopyrightText: 2017-2020 Stefan Gerlach <stefan.gerlach@uni.kn>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
@@ -21,12 +21,14 @@
 #include "matrixcommands.h"
 
 #include <KConfig>
+#include <KConfigGroup>
 #include <KLocalizedString>
 
 #include <QHeaderView>
 #include <QPrintDialog>
 #include <QPrintPreviewDialog>
 #include <QPrinter>
+#include <QTimer>
 
 /*!
 	This class manages matrix based data (i.e., mathematically
@@ -40,7 +42,7 @@
 */
 Matrix::Matrix(int rows, int cols, const QString& name, const AbstractColumn::ColumnMode mode)
 	: AbstractDataSource(name, AspectType::Matrix)
-	, d(new MatrixPrivate(this, mode)) {
+	, d_ptr(new MatrixPrivate(this, mode)) {
 	// set initial number of rows and columns
 	appendColumns(cols);
 	appendRows(rows);
@@ -50,36 +52,37 @@ Matrix::Matrix(int rows, int cols, const QString& name, const AbstractColumn::Co
 
 Matrix::Matrix(const QString& name, bool loading, const AbstractColumn::ColumnMode mode)
 	: AbstractDataSource(name, AspectType::Matrix)
-	, d(new MatrixPrivate(this, mode)) {
+	, d_ptr(new MatrixPrivate(this, mode)) {
 	if (!loading)
 		init();
 }
 
 Matrix::~Matrix() {
-	delete d;
+	delete d_ptr;
 }
 
 void Matrix::init() {
+	Q_D(Matrix);
 	KConfig config;
-	KConfigGroup group = config.group("Matrix");
+	KConfigGroup group = config.group(QStringLiteral("Matrix"));
 
 	// matrix dimension
-	int rows = group.readEntry("RowCount", 10);
-	int cols = group.readEntry("ColumnCount", 10);
+	int rows = group.readEntry(QStringLiteral("RowCount"), 10);
+	int cols = group.readEntry(QStringLiteral("ColumnCount"), 10);
 	appendRows(rows);
 	appendColumns(cols);
 
 	// mapping to logical x- and y-coordinates
-	d->xStart = group.readEntry("XStart", 0.0);
-	d->xEnd = group.readEntry("XEnd", 1.0);
-	d->yStart = group.readEntry("YStart", 0.0);
-	d->yEnd = group.readEntry("YEnd", 1.0);
+	d->xStart = group.readEntry(QStringLiteral("XStart"), 0.0);
+	d->xEnd = group.readEntry(QStringLiteral("XEnd"), 1.0);
+	d->yStart = group.readEntry(QStringLiteral("YStart"), 0.0);
+	d->yEnd = group.readEntry(QStringLiteral("YEnd"), 1.0);
 
 	// format
-	QByteArray formatba = group.readEntry("NumericFormat", "f").toLatin1();
+	QByteArray formatba = group.readEntry(QStringLiteral("NumericFormat"), QStringLiteral("f")).toLatin1();
 	d->numericFormat = *formatba.data();
-	d->precision = group.readEntry("Precision", 3);
-	d->headerFormat = (Matrix::HeaderFormat)group.readEntry("HeaderFormat", static_cast<int>(HeaderFormat::HeaderRowsColumns));
+	d->precision = group.readEntry(QStringLiteral("Precision"), 3);
+	d->headerFormat = (Matrix::HeaderFormat)group.readEntry(QStringLiteral("HeaderFormat"), static_cast<int>(HeaderFormat::HeaderRowsColumns));
 }
 
 /*!
@@ -103,11 +106,21 @@ QWidget* Matrix::view() const {
 		m_view = new MatrixView(const_cast<Matrix*>(this));
 		m_partView = m_view;
 		m_model = m_view->model();
+		connect(this, &Matrix::viewAboutToBeDeleted, [this]() {
+			m_view = nullptr;
+		});
+
+		// navigate to the first cell and set the focus so the user can start directly entering new data
+		QTimer::singleShot(0, this, [=]() {
+			m_view->goToCell(0, 0);
+			m_view->setFocus();
+		});
 	}
 	return m_partView;
 }
 
 bool Matrix::exportView() const {
+#ifndef SDK
 	auto* dlg = new ExportSpreadsheetDialog(m_view);
 	dlg->setFileName(name());
 	dlg->setMatrixMode(true);
@@ -143,6 +156,9 @@ bool Matrix::exportView() const {
 	delete dlg;
 
 	return ret;
+#else
+	return 0;
+#endif
 }
 
 bool Matrix::printView() {
@@ -164,10 +180,11 @@ bool Matrix::printPreview() const {
 	return dlg->exec();
 }
 
-//##############################################################################
-//##########################  getter methods  ##################################
-//##############################################################################
+// ##############################################################################
+// ##########################  getter methods  ##################################
+// ##############################################################################
 void* Matrix::data() const {
+	Q_D(const Matrix);
 	return d->data;
 }
 
@@ -193,10 +210,11 @@ void Matrix::setChanged() {
 		m_model->setChanged();
 }
 
-//##############################################################################
-//#################  setter methods and undo commands ##########################
-//##############################################################################
+// ##############################################################################
+// #################  setter methods and undo commands ##########################
+// ##############################################################################
 void Matrix::setRowCount(int count) {
+	Q_D(const Matrix);
 	if (count == d->rowCount)
 		return;
 
@@ -208,6 +226,7 @@ void Matrix::setRowCount(int count) {
 }
 
 void Matrix::setColumnCount(int count) {
+	Q_D(const Matrix);
 	if (count == d->columnCount)
 		return;
 
@@ -220,42 +239,49 @@ void Matrix::setColumnCount(int count) {
 
 STD_SETTER_CMD_IMPL_F_S(Matrix, SetXStart, double, xStart, updateViewHeader)
 void Matrix::setXStart(double xStart) {
+	Q_D(Matrix);
 	if (xStart != d->xStart)
 		exec(new MatrixSetXStartCmd(d, xStart, ki18n("%1: x-start changed")));
 }
 
 STD_SETTER_CMD_IMPL_F_S(Matrix, SetXEnd, double, xEnd, updateViewHeader)
 void Matrix::setXEnd(double xEnd) {
+	Q_D(Matrix);
 	if (xEnd != d->xEnd)
 		exec(new MatrixSetXEndCmd(d, xEnd, ki18n("%1: x-end changed")));
 }
 
 STD_SETTER_CMD_IMPL_F_S(Matrix, SetYStart, double, yStart, updateViewHeader)
 void Matrix::setYStart(double yStart) {
+	Q_D(Matrix);
 	if (yStart != d->yStart)
 		exec(new MatrixSetYStartCmd(d, yStart, ki18n("%1: y-start changed")));
 }
 
 STD_SETTER_CMD_IMPL_F_S(Matrix, SetYEnd, double, yEnd, updateViewHeader)
 void Matrix::setYEnd(double yEnd) {
+	Q_D(Matrix);
 	if (yEnd != d->yEnd)
 		exec(new MatrixSetYEndCmd(d, yEnd, ki18n("%1: y-end changed")));
 }
 
 STD_SETTER_CMD_IMPL_S(Matrix, SetNumericFormat, char, numericFormat)
 void Matrix::setNumericFormat(char format) {
+	Q_D(Matrix);
 	if (format != d->numericFormat)
 		exec(new MatrixSetNumericFormatCmd(d, format, ki18n("%1: numeric format changed")));
 }
 
 STD_SETTER_CMD_IMPL_S(Matrix, SetPrecision, int, precision)
 void Matrix::setPrecision(int precision) {
+	Q_D(Matrix);
 	if (precision != d->precision)
 		exec(new MatrixSetPrecisionCmd(d, precision, ki18n("%1: precision changed")));
 }
 
 // TODO: make this undoable?
 void Matrix::setHeaderFormat(Matrix::HeaderFormat format) {
+	Q_D(Matrix);
 	d->headerFormat = format;
 	m_model->updateHeader();
 
@@ -267,6 +293,7 @@ void Matrix::setHeaderFormat(Matrix::HeaderFormat format) {
 
 // columns
 void Matrix::insertColumns(int before, int count) {
+	Q_D(Matrix);
 	if (count < 1 || before < 0 || before > columnCount())
 		return;
 	WAIT_CURSOR;
@@ -279,6 +306,7 @@ void Matrix::appendColumns(int count) {
 }
 
 void Matrix::removeColumns(int first, int count) {
+	Q_D(Matrix);
 	if (count < 1 || first < 0 || first + count > columnCount())
 		return;
 	WAIT_CURSOR;
@@ -306,6 +334,7 @@ void Matrix::removeColumns(int first, int count) {
 
 void Matrix::clearColumn(int c) {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixClearColumnCmd<double>(d, c));
@@ -330,6 +359,7 @@ void Matrix::clearColumn(int c) {
 
 // rows
 void Matrix::insertRows(int before, int count) {
+	Q_D(Matrix);
 	if (count < 1 || before < 0 || before > rowCount())
 		return;
 	WAIT_CURSOR;
@@ -345,6 +375,7 @@ void Matrix::removeRows(int first, int count) {
 	if (count < 1 || first < 0 || first + count > rowCount())
 		return;
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixRemoveRowsCmd<double>(d, first, count));
@@ -368,6 +399,7 @@ void Matrix::removeRows(int first, int count) {
 }
 
 void Matrix::clearRow(int r) {
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		for (int c = 0; c < columnCount(); ++c)
@@ -397,6 +429,7 @@ void Matrix::clearRow(int r) {
 //! Return the value in the given cell (needs explicit instantiation)
 template<typename T>
 T Matrix::cell(int row, int col) const {
+	Q_D(const Matrix);
 	return d->cell<T>(row, col);
 }
 template double Matrix::cell<double>(int row, int col) const;
@@ -412,6 +445,7 @@ QString Matrix::text(int row, int col) {
 }
 template<>
 QString Matrix::text<double>(int row, int col) {
+	Q_D(const Matrix);
 	return QLocale().toString(cell<double>(row, col), d->numericFormat, d->precision);
 }
 template<>
@@ -425,6 +459,7 @@ template QString Matrix::text<QDateTime>(int row, int col);
 //! Set the value of the cell (needs explicit instantiation)
 template<typename T>
 void Matrix::setCell(int row, int col, T value) {
+	Q_D(Matrix);
 	if (row < 0 || row >= rowCount())
 		return;
 	if (col < 0 || col >= columnCount())
@@ -438,6 +473,7 @@ template void Matrix::setCell<QString>(int row, int col, QString value);
 template void Matrix::setCell<QDateTime>(int row, int col, QDateTime value);
 
 void Matrix::clearCell(int row, int col) {
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixSetCellValueCmd<double>(d, row, col, 0.0));
@@ -482,79 +518,20 @@ void Matrix::setDimensions(int rows, int cols) {
 	RESET_CURSOR;
 }
 
-void Matrix::copy(Matrix* other) {
-	WAIT_CURSOR;
-	beginMacro(i18n("%1: copy %2", name(), other->name()));
-
-	int rows = other->rowCount();
-	int columns = other->columnCount();
-	setDimensions(rows, columns);
-
-	for (int i = 0; i < rows; i++)
-		setRowHeight(i, other->rowHeight(i));
-
-	for (int i = 0; i < columns; i++)
-		setColumnWidth(i, other->columnWidth(i));
-
-	d->suppressDataChange = true;
-	switch (d->mode) {
-	case AbstractColumn::ColumnMode::Double:
-		for (int i = 0; i < columns; i++)
-			setColumnCells(i, 0, rows - 1, other->columnCells<double>(i, 0, rows - 1));
-		break;
-	case AbstractColumn::ColumnMode::Text:
-		for (int i = 0; i < columns; i++)
-			setColumnCells(i, 0, rows - 1, other->columnCells<QString>(i, 0, rows - 1));
-		break;
-	case AbstractColumn::ColumnMode::Integer:
-		for (int i = 0; i < columns; i++)
-			setColumnCells(i, 0, rows - 1, other->columnCells<int>(i, 0, rows - 1));
-		break;
-	case AbstractColumn::ColumnMode::BigInt:
-		for (int i = 0; i < columns; i++)
-			setColumnCells(i, 0, rows - 1, other->columnCells<qint64>(i, 0, rows - 1));
-		break;
-	case AbstractColumn::ColumnMode::Day:
-	case AbstractColumn::ColumnMode::Month:
-	case AbstractColumn::ColumnMode::DateTime:
-		for (int i = 0; i < columns; i++)
-			setColumnCells(i, 0, rows - 1, other->columnCells<QDateTime>(i, 0, rows - 1));
-		break;
-	}
-
-	setCoordinates(other->xStart(), other->xEnd(), other->yStart(), other->yEnd());
-	setNumericFormat(other->numericFormat());
-	setPrecision(other->precision());
-	d->formula = other->formula();
-	d->suppressDataChange = false;
-	Q_EMIT dataChanged(0, 0, rows - 1, columns - 1);
-	if (m_view)
-		m_view->adjustHeaders();
-
-	endMacro();
-	RESET_CURSOR;
-}
-
-//! Duplicate the matrix inside its folder
-void Matrix::duplicate() {
-	Matrix* matrix = new Matrix(rowCount(), columnCount(), name());
-	matrix->copy(this);
-	if (folder())
-		folder()->addChild(matrix);
-}
-
 void Matrix::addRows() {
+	Q_D(Matrix);
 	if (!m_view)
 		return;
 	WAIT_CURSOR;
 	int count = m_view->selectedRowCount(false);
-	beginMacro(i18np("%1: add %2 rows", "%1: add %2 rows", name(), count));
+	beginMacro(i18np("%1: add %2 row", "%1: add %2 rows", name(), count));
 	exec(new MatrixInsertRowsCmd(d, rowCount(), count));
 	endMacro();
 	RESET_CURSOR;
 }
 
 void Matrix::addColumns() {
+	Q_D(Matrix);
 	if (!m_view)
 		return;
 	WAIT_CURSOR;
@@ -566,10 +543,12 @@ void Matrix::addColumns() {
 }
 
 void Matrix::setCoordinates(double x1, double x2, double y1, double y2) {
+	Q_D(Matrix);
 	exec(new MatrixSetCoordinatesCmd(d, x1, x2, y1, y2));
 }
 
 void Matrix::setFormula(const QString& formula) {
+	Q_D(Matrix);
 	exec(new MatrixSetFormulaCmd(d, formula));
 }
 
@@ -578,6 +557,7 @@ void Matrix::setFormula(const QString& formula) {
  * values that are saved when the matrix is saved. The view
  * has to take care of reading and applying these values */
 void Matrix::setRowHeight(int row, int height) {
+	Q_D(Matrix);
 	d->setRowHeight(row, height);
 }
 
@@ -586,20 +566,24 @@ void Matrix::setRowHeight(int row, int height) {
  * values that are saved when the matrix is saved. The view
  * has to take care of reading and applying these values */
 void Matrix::setColumnWidth(int col, int width) {
+	Q_D(Matrix);
 	d->setColumnWidth(col, width);
 }
 
 int Matrix::rowHeight(int row) const {
+	Q_D(const Matrix);
 	return d->rowHeight(row);
 }
 
 int Matrix::columnWidth(int col) const {
+	Q_D(const Matrix);
 	return d->columnWidth(col);
 }
 
 //! Return the values in the given cells as vector
 template<typename T>
 QVector<T> Matrix::columnCells(int col, int first_row, int last_row) {
+	Q_D(const Matrix);
 	return d->columnCells<T>(col, first_row, last_row);
 }
 
@@ -607,6 +591,7 @@ QVector<T> Matrix::columnCells(int col, int first_row, int last_row) {
 template<typename T>
 void Matrix::setColumnCells(int col, int first_row, int last_row, const QVector<T>& values) {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	exec(new MatrixSetColumnCellsCmd<T>(d, col, first_row, last_row, values));
 	RESET_CURSOR;
 }
@@ -614,6 +599,7 @@ void Matrix::setColumnCells(int col, int first_row, int last_row, const QVector<
 //! Return the values in the given cells as vector (needs explicit instantiation)
 template<typename T>
 QVector<T> Matrix::rowCells(int row, int first_column, int last_column) {
+	Q_D(const Matrix);
 	return d->rowCells<T>(row, first_column, last_column);
 }
 template QVector<double> Matrix::rowCells<double>(int row, int first_column, int last_column);
@@ -625,13 +611,14 @@ template QVector<QDateTime> Matrix::rowCells<QDateTime>(int row, int first_colum
 template<typename T>
 void Matrix::setRowCells(int row, int first_column, int last_column, const QVector<T>& values) {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	exec(new MatrixSetRowCellsCmd<T>(d, row, first_column, last_column, values));
 	RESET_CURSOR;
 }
 
 void Matrix::setData(void* data) {
 	bool isEmpty = false;
-
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		if (static_cast<QVector<QVector<double>>*>(data)->isEmpty())
@@ -667,12 +654,13 @@ QVector<AspectType> Matrix::dropableOn() const {
 	return vec;
 }
 
-//##############################################################################
-//#########################  Public slots  #####################################
-//##############################################################################
+// ##############################################################################
+// #########################  Public slots  #####################################
+// ##############################################################################
 //! Clear the whole matrix (i.e. reset all cells)
 void Matrix::clear() {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	beginMacro(i18n("%1: clear", name()));
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
@@ -699,6 +687,7 @@ void Matrix::clear() {
 
 void Matrix::transpose() {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixTransposeCmd<double>(d));
@@ -723,6 +712,7 @@ void Matrix::transpose() {
 
 void Matrix::mirrorHorizontally() {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixMirrorHorizontallyCmd<double>(d));
@@ -747,6 +737,7 @@ void Matrix::mirrorHorizontally() {
 
 void Matrix::mirrorVertically() {
 	WAIT_CURSOR;
+	Q_D(Matrix);
 	switch (d->mode) {
 	case AbstractColumn::ColumnMode::Double:
 		exec(new MatrixMirrorVerticallyCmd<double>(d));
@@ -769,9 +760,9 @@ void Matrix::mirrorVertically() {
 	RESET_CURSOR;
 }
 
-//##############################################################################
-//######################  Private implementation ###############################
-//##############################################################################
+// ##############################################################################
+// ######################  Private implementation ###############################
+// ##############################################################################
 
 MatrixPrivate::MatrixPrivate(Matrix* owner, const AbstractColumn::ColumnMode m)
 	: q(owner)
@@ -1020,10 +1011,11 @@ void MatrixPrivate::clearColumn(int col) {
 		Q_EMIT q->dataChanged(0, col, rowCount - 1, col);
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 void Matrix::save(QXmlStreamWriter* writer) const {
+	Q_D(const Matrix);
 	DEBUG(Q_FUNC_INFO);
 	writer->writeStartElement(QStringLiteral("matrix"));
 	writeBasicAttributes(writer);
@@ -1127,7 +1119,7 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 	if (!readBasicAttributes(reader))
 		return false;
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
+	Q_D(Matrix);
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -1151,19 +1143,19 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("mode")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("mode")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("mode"));
 			else
 				d->mode = AbstractColumn::ColumnMode(str.toInt());
 
 			str = attribs.value(QStringLiteral("headerFormat")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("headerFormat")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("headerFormat"));
 			else
 				d->headerFormat = Matrix::HeaderFormat(str.toInt());
 
 			str = attribs.value(QStringLiteral("numericFormat")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("numericFormat")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("numericFormat"));
 			else {
 				QByteArray formatba = str.toLatin1();
 				d->numericFormat = *formatba.data();
@@ -1171,7 +1163,7 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("precision")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("precision")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("precision"));
 			else
 				d->precision = str.toInt();
 
@@ -1180,37 +1172,37 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("columns")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("columns")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("columns"));
 			else
 				d->columnCount = str.toInt();
 
 			str = attribs.value(QStringLiteral("rows")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("rows")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("rows"));
 			else
 				d->rowCount = str.toInt();
 
 			str = attribs.value(QStringLiteral("x_start")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("x_start")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("x_start"));
 			else
 				d->xStart = str.toDouble();
 
 			str = attribs.value(QStringLiteral("x_end")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("x_end")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("x_end"));
 			else
 				d->xEnd = str.toDouble();
 
 			str = attribs.value(QStringLiteral("y_start")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("y_start")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("y_start"));
 			else
 				d->yStart = str.toDouble();
 
 			str = attribs.value(QStringLiteral("y_end")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("y_end")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("y_end"));
 			else
 				d->yEnd = str.toDouble();
 		} else if (!preview && reader->name() == QLatin1String("row_heights")) {
@@ -1280,7 +1272,7 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 			}
 			}
 		} else { // unknown element
-			reader->raiseWarning(i18n("unknown element '%1'", reader->name().toString()));
+			reader->raiseUnknownElementWarning();
 			if (!reader->skipToEndElement())
 				return false;
 		}
@@ -1289,16 +1281,18 @@ bool Matrix::load(XmlStreamReader* reader, bool preview) {
 	return true;
 }
 
-//##############################################################################
-//########################  Data Import  #######################################
-//##############################################################################
+// ##############################################################################
+// ########################  Data Import  #######################################
+// ##############################################################################
 int Matrix::prepareImport(std::vector<void*>& dataContainer,
 						  AbstractFileFilter::ImportMode mode,
 						  int actualRows,
 						  int actualCols,
-						  QStringList /*colNameList*/,
-						  QVector<AbstractColumn::ColumnMode> columnMode,
+						  const QStringList& /*colNameList*/,
+						  const QVector<AbstractColumn::ColumnMode>& columnMode,
+						  bool& ok,
 						  bool initializeDataContainer) {
+	Q_D(Matrix);
 	auto newColumnMode = columnMode.at(0); // only first column mode used
 	DEBUG(Q_FUNC_INFO << ", rows = " << actualRows << " cols = " << actualCols << ", mode = " << ENUM_TO_STRING(AbstractFileFilter, ImportMode, mode)
 					  << ", column mode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, newColumnMode))
@@ -1309,29 +1303,34 @@ int Matrix::prepareImport(std::vector<void*>& dataContainer,
 	setSuppressDataChangedSignal(true);
 
 	// resize the matrix
-	if (mode == AbstractFileFilter::ImportMode::Replace) {
-		clear();
-		setDimensions(actualRows, actualCols);
-	} else { // Append
-		// handle mismatch of modes
-		DEBUG(Q_FUNC_INFO << ", TODO: matrix mode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, d->mode)
-						  << ", columnMode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, columnMode.at(0)))
-		// TODO: no way to convert types yet!
-		if (d->mode != newColumnMode) {
-			DEBUG(Q_FUNC_INFO << ", WARNING mismatch of types in append mode!")
-		}
-		// catch some cases
-		if ((d->mode == AbstractColumn::ColumnMode::Integer || d->mode == AbstractColumn::ColumnMode::BigInt)
-			&& newColumnMode == AbstractColumn::ColumnMode::Double)
-			d->mode = newColumnMode;
-
-		columnOffset = columnCount();
-		actualCols += columnOffset;
-		DEBUG(Q_FUNC_INFO << ", col count = " << columnCount() << ", actualCols = " << actualCols)
-		if (rowCount() < actualRows)
+	try {
+		if (mode == AbstractFileFilter::ImportMode::Replace) {
+			clear();
 			setDimensions(actualRows, actualCols);
-		else
-			setDimensions(rowCount(), actualCols);
+		} else { // Append
+			// handle mismatch of modes
+			DEBUG(Q_FUNC_INFO << ", TODO: matrix mode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, d->mode)
+							  << ", columnMode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, columnMode.at(0)))
+			// TODO: no way to convert types yet!
+			if (d->mode != newColumnMode) {
+				DEBUG(Q_FUNC_INFO << ", WARNING mismatch of types in append mode!")
+			}
+			// catch some cases
+			if ((d->mode == AbstractColumn::ColumnMode::Integer || d->mode == AbstractColumn::ColumnMode::BigInt)
+				&& newColumnMode == AbstractColumn::ColumnMode::Double)
+				d->mode = newColumnMode;
+
+			columnOffset = columnCount();
+			actualCols += columnOffset;
+			DEBUG(Q_FUNC_INFO << ", col count = " << columnCount() << ", actualCols = " << actualCols)
+			if (rowCount() < actualRows)
+				setDimensions(actualRows, actualCols);
+			else
+				setDimensions(rowCount(), actualCols);
+		}
+	} catch (std::bad_alloc&) {
+		ok = false;
+		return 0;
 	}
 
 	DEBUG(Q_FUNC_INFO << ", actual rows/cols = " << actualRows << "/" << actualCols)
@@ -1385,6 +1384,7 @@ int Matrix::prepareImport(std::vector<void*>& dataContainer,
 		}
 	}
 
+	ok = true;
 	return columnOffset;
 }
 
