@@ -5,7 +5,7 @@
 						   show their values
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2020 Martin Marmsoler <martin.marmsoler@gmail.com>
-	SPDX-FileCopyrightText: 2020-2022 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2020-2023 Alexander Semke <alexander.semke@web.de>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -24,6 +24,9 @@
 #include "backend/worksheet/plots/cartesian/CustomPoint.h"
 #include "backend/worksheet/plots/cartesian/XYCurve.h"
 
+#include <KConfig>
+#include <KConfigGroup>
+
 #include <QAction>
 #include <QDateTime>
 #include <QGraphicsSceneMouseEvent>
@@ -31,19 +34,29 @@
 #include <QMenu>
 #include <QPainter>
 #include <QTextEdit>
+#include <qglobal.h>
+
+InfoElement::MarkerPoints_T::MarkerPoints_T(CustomPoint* custompoint, const XYCurve* curve, QString curvePath)
+	: customPoint(custompoint)
+	, curve(curve)
+	, curvePath(curvePath) {
+	if (customPoint)
+		visible = custompoint->isVisible();
+}
 
 InfoElement::InfoElement(const QString& name, CartesianPlot* plot)
 	: WorksheetElement(name, new InfoElementPrivate(this), AspectType::InfoElement) {
-	m_plot = plot;
+	Q_D(InfoElement);
+	d->m_plot = plot;
 
 	init();
 	setVisible(false);
 }
 
-InfoElement::InfoElement(const QString& name, CartesianPlot* p, const XYCurve* curve, double pos)
+InfoElement::InfoElement(const QString& name, CartesianPlot* p, const XYCurve* curve, double logicalPos)
 	: WorksheetElement(name, new InfoElementPrivate(this, curve), AspectType::InfoElement) {
 	Q_D(InfoElement);
-	m_plot = p;
+	d->m_plot = p;
 
 	init();
 
@@ -51,19 +64,19 @@ InfoElement::InfoElement(const QString& name, CartesianPlot* p, const XYCurve* c
 
 	if (curve) {
 		d->connectionLineCurveName = curve->name();
-		auto* custompoint = new CustomPoint(m_plot, curve->name());
+		auto* custompoint = new CustomPoint(d->m_plot, curve->name());
 		custompoint->setFixed(true);
 		custompoint->setCoordinateBindingEnabled(true);
 		custompoint->setCoordinateSystemIndex(curve->coordinateSystemIndex());
 		addChild(custompoint);
-		InfoElement::MarkerPoints_T markerpoint(custompoint, custompoint->path(), curve, curve->path());
+		InfoElement::MarkerPoints_T markerpoint(custompoint, curve, curve->path());
 		markerpoints.append(markerpoint);
 
 		// setpos after label was created
 		if (curve->xColumn() && curve->yColumn()) {
 			bool valueFound;
 			double xpos;
-			double y = curve->y(pos, xpos, valueFound);
+			double y = curve->y(logicalPos, xpos, valueFound);
 			if (valueFound) {
 				d->positionLogical = xpos;
 				d->m_index = curve->xColumn()->indexForValue(xpos);
@@ -121,15 +134,16 @@ InfoElement::~InfoElement() {
 }
 
 void InfoElement::init() {
-	cSystem = dynamic_cast<const CartesianCoordinateSystem*>(m_plot->coordinateSystem(m_cSystemIndex));
+	Q_D(InfoElement);
+	cSystem = dynamic_cast<const CartesianCoordinateSystem*>(d->m_plot->coordinateSystem(m_cSystemIndex));
 
 	initActions();
 	initMenus();
 
-	connect(this, &InfoElement::aspectRemoved, this, &InfoElement::childRemoved);
-	connect(this, &InfoElement::aspectAdded, this, &InfoElement::childAdded);
+	connect(this, &InfoElement::childAspectRemoved, this, &InfoElement::childRemoved);
+	connect(this, &InfoElement::childAspectAdded, this, &InfoElement::childAdded);
 
-	m_title = new TextLabel(i18n("Label"), m_plot);
+	m_title = new TextLabel(i18n("Label"), d->m_plot);
 	m_title->setHidden(true);
 	TextLabel::TextWrapper text;
 	text.allowPlaceholder = true;
@@ -140,12 +154,16 @@ void InfoElement::init() {
 	m_setTextLabelText = false;
 	addChild(m_title);
 
+	// use the line properties of axis line also for the info element lines
+	KConfig config;
+	const auto& group = config.group(QStringLiteral("Axis"));
+
 	// lines
-	Q_D(InfoElement);
 	d->verticalLine = new Line(QString());
 	d->verticalLine->setHidden(true);
 	d->verticalLine->setPrefix(QStringLiteral("VerticalLine"));
 	addChild(d->verticalLine);
+	d->verticalLine->init(group);
 	connect(d->verticalLine, &Line::updatePixmapRequested, [=] {
 		d->update();
 	});
@@ -157,22 +175,16 @@ void InfoElement::init() {
 	d->connectionLine->setHidden(true);
 	d->connectionLine->setPrefix(QStringLiteral("ConnectionLine"));
 	addChild(d->connectionLine);
+	d->connectionLine->init(group);
 	connect(d->connectionLine, &Line::updatePixmapRequested, [=] {
 		d->update();
 	});
 	connect(d->connectionLine, &Line::updateRequested, [=] {
 		d->updateConnectionLine();
 	});
-
-	// use the color for the axis line from the theme also for info element's lines
-	KConfig config;
-	InfoElement::loadThemeConfig(config);
 }
 
 void InfoElement::initActions() {
-	visibilityAction = new QAction(i18n("Visible"), this);
-	visibilityAction->setCheckable(true);
-	connect(visibilityAction, &QAction::triggered, this, &InfoElement::setVisible);
 }
 
 void InfoElement::initMenus() {
@@ -183,25 +195,31 @@ void InfoElement::initCurveConnections(const XYCurve* curve) {
 	connect(curve, &XYCurve::visibleChanged, this, &InfoElement::curveVisibilityChanged);
 	connect(curve, &XYCurve::coordinateSystemIndexChanged, this, &InfoElement::curveCoordinateSystemIndexChanged);
 	connect(curve, &XYCurve::dataChanged, this, &InfoElement::curveDataChanged);
-	connect(curve, &XYCurve::moveBegin, this, [this]() {
-		m_curveGetsMoved = true;
+	connect(curve, &XYCurve::xColumnChanged, [this](const AbstractColumn*) {
+		retransform();
 	});
-	connect(curve, &XYCurve::moveEnd, this, [this]() {
-		m_curveGetsMoved = false;
+	connect(curve, &XYCurve::yColumnChanged, [this](const AbstractColumn*) {
+		retransform();
 	});
+	connect(curve, &XYCurve::aspectAboutToBeRemoved, this, &InfoElement::curveDeleted);
+}
+
+void InfoElement::pointVisibleChanged(bool visible) {
+	const auto* point = QObject::sender();
+
+	if (m_suppressVisibleChange)
+		return;
+	for (auto& c : markerpoints) {
+		if (c.customPoint == point)
+			c.visible = visible;
+	}
 }
 
 QMenu* InfoElement::createContextMenu() {
 	if (!m_menusInitialized)
 		initMenus();
 
-	QMenu* menu = WorksheetElement::createContextMenu();
-	QAction* firstAction = menu->actions().at(1);
-
-	visibilityAction->setChecked(isVisible());
-	menu->insertAction(firstAction, visibilityAction);
-
-	return menu;
+	return WorksheetElement::createContextMenu();
 }
 
 /*!
@@ -222,10 +240,12 @@ void InfoElement::addCurve(const XYCurve* curve, CustomPoint* custompoint) {
 
 	if (!custompoint) {
 		m_suppressChildPositionChanged = true;
-		custompoint = new CustomPoint(m_plot, curve->name());
+		custompoint = new CustomPoint(d->m_plot, curve->name());
 		custompoint->setCoordinateBindingEnabled(true);
 		custompoint->setCoordinateSystemIndex(curve->coordinateSystemIndex());
+		setUndoAware(false);
 		addChild(custompoint);
+		setUndoAware(true);
 
 		if (curve->xColumn() && curve->yColumn()) {
 			bool valueFound;
@@ -251,7 +271,7 @@ void InfoElement::addCurve(const XYCurve* curve, CustomPoint* custompoint) {
 	if (d->m_index < 0 && curve->xColumn())
 		d->m_index = curve->xColumn()->indexForValue(custompoint->positionLogical().x());
 
-	struct MarkerPoints_T markerpoint = {custompoint, custompoint->path(), curve, curve->path()};
+	struct MarkerPoints_T markerpoint = {custompoint, curve, curve->path()};
 	markerpoints.append(markerpoint);
 
 	if (markerpoints.count() == 1) // first point
@@ -281,14 +301,15 @@ void InfoElement::addCurve(const XYCurve* curve, CustomPoint* custompoint) {
  * @param curvePath path from the curve
  * @param custompoint adding already created custom point
  */
-void InfoElement::addCurvePath(QString& curvePath, CustomPoint* custompoint) {
+void InfoElement::addCurvePath(const QString& curvePath, CustomPoint* custompoint) {
 	for (auto& markerpoint : markerpoints) {
 		if (curvePath == markerpoint.curvePath)
 			return;
 	}
 
+	Q_D(const InfoElement);
 	if (!custompoint) {
-		custompoint = new CustomPoint(m_plot, i18n("Symbol"));
+		custompoint = new CustomPoint(d->m_plot, i18n("Symbol"));
 		custompoint->setVisible(false);
 		m_suppressChildPositionChanged = true;
 		custompoint->setCoordinateBindingEnabled(true);
@@ -296,7 +317,7 @@ void InfoElement::addCurvePath(QString& curvePath, CustomPoint* custompoint) {
 		addChild(custompoint);
 	}
 
-	struct MarkerPoints_T markerpoint = {custompoint, custompoint->path(), nullptr, curvePath};
+	struct MarkerPoints_T markerpoint = {custompoint, nullptr, curvePath};
 	markerpoints.append(markerpoint);
 }
 
@@ -313,7 +334,6 @@ bool InfoElement::assignCurve(const QVector<XYCurve*>& curves) {
 			if (mp.curvePath == curve->path()) {
 				mp.curve = curve;
 				initCurveConnections(curve);
-				mp.customPoint->setVisible(curve->isVisible()); // initial visibility
 				mp.customPoint->setCoordinateSystemIndex(curve->coordinateSystemIndex());
 				break;
 			}
@@ -332,29 +352,64 @@ bool InfoElement::assignCurve(const QVector<XYCurve*>& curves) {
 	return success;
 }
 
+void InfoElement::curveDeleted(const AbstractAspect* aspect) {
+	auto curve = dynamic_cast<const XYCurve*>(aspect);
+	if (!curve)
+		return;
+
+	for (auto& mp : markerpoints) {
+		if (mp.curve == curve) {
+			disconnect(curve, nullptr, this, nullptr);
+			// No remove of the custompoint
+
+			Lock lock(m_suppressVisibleChange);
+			assert(mp.curvePath == curve->path());
+			mp.curve = nullptr;
+			mp.customPoint->setVisible(false);
+		}
+	}
+
+	updateValid();
+
+	if (curve->name() == connectionLineCurveName())
+		setConnectionLineNextValidCurve();
+}
+
+void InfoElement::setConnectionLineNextValidCurve() {
+	for (const auto& mp : markerpoints) {
+		if (mp.curve) {
+			setConnectionLineCurveName(mp.curve->name());
+			return;
+		}
+	}
+	setConnectionLineCurveName(QLatin1String());
+}
+
 /*!
  * Remove markerpoint from a curve
  * @param curve
  */
 void InfoElement::removeCurve(const XYCurve* curve) {
-	if (m_curveGetsMoved)
-		return;
-
 	for (const auto& mp : markerpoints) {
 		if (mp.curve == curve) {
 			disconnect(curve, nullptr, this, nullptr);
+			setUndoAware(false);
 			removeChild(mp.customPoint);
+			setUndoAware(true);
 		}
 	}
 
+	setUndoAware(false);
+	if (curve->name() == connectionLineCurveName())
+		setConnectionLineNextValidCurve();
+	setUndoAware(true);
+
 	m_title->setUndoAware(false);
 	m_title->setText(createTextLabelText());
 
 	// hide the label if now curves are selected
 	if (markerpoints.isEmpty()) {
-		m_title->setUndoAware(false);
 		m_title->setVisible(false); // hide in the worksheet view
-		m_title->setUndoAware(true);
 		Q_D(InfoElement);
 		d->update(); // redraw to remove all children graphic items belonging to InfoElement
 	}
@@ -378,15 +433,15 @@ void InfoElement::setZValue(qreal value) {
 /*!
  * Returns the amount of markerpoints. Used in the InfoElementDock to fill listWidget.
  */
-int InfoElement::markerPointsCount() {
+int InfoElement::markerPointsCount() const {
 	return markerpoints.length();
 }
 
-TextLabel::GluePoint InfoElement::gluePoint(int index) {
+TextLabel::GluePoint InfoElement::gluePoint(int index) const {
 	return m_title->gluePointAt(index);
 }
 
-int InfoElement::gluePointsCount() {
+int InfoElement::gluePointsCount() const {
 	return m_title->gluePointCount();
 }
 
@@ -394,7 +449,7 @@ int InfoElement::gluePointsCount() {
  * Returns the Markerpoint at index \p index. Used in the InfoElementDock to fill listWidget
  * @param index
  */
-InfoElement::MarkerPoints_T InfoElement::markerPointAt(int index) {
+InfoElement::MarkerPoints_T InfoElement::markerPointAt(int index) const {
 	return markerpoints.at(index);
 }
 
@@ -408,7 +463,7 @@ InfoElement::MarkerPoints_T InfoElement::markerPointAt(int index) {
  */
 TextLabel::TextWrapper InfoElement::createTextLabelText() {
 	// TODO: save positions of the variables in extra variables to replace faster, because replace takes long time
-	TextLabel::TextWrapper wrapper = m_title->text();
+	auto wrapper = m_title->text();
 	if (markerPointsCount() < 1) {
 		DEBUG(STDSTRING(wrapper.text))
 		DEBUG(STDSTRING(wrapper.textPlaceholder))
@@ -421,20 +476,19 @@ TextLabel::TextWrapper InfoElement::createTextLabelText() {
 
 	Q_D(const InfoElement);
 
-	QString text = wrapper.textPlaceholder;
-
 	// replace the placeholder for the x-value
 	QString xValueStr;
-	auto columnMode = markerpoints.at(0).curve->xColumn()->columnMode();
+	const auto columnMode = markerpoints.at(0).curve->xColumn()->columnMode();
 	if (columnMode == AbstractColumn::ColumnMode::Double || columnMode == AbstractColumn::ColumnMode::Integer
 		|| columnMode == AbstractColumn::ColumnMode::BigInt)
 		xValueStr = QString::number(d->positionLogical);
 	else if (columnMode == AbstractColumn::ColumnMode::Day || columnMode == AbstractColumn::ColumnMode::Month
 			 || columnMode == AbstractColumn::ColumnMode::DateTime) {
 		const auto& dateTime = QDateTime::fromMSecsSinceEpoch(d->positionLogical, Qt::UTC);
-		xValueStr = dateTime.toString(m_plot->rangeDateTimeFormat(Dimension::X));
+		xValueStr = dateTime.toString(d->m_plot->rangeDateTimeFormat(Dimension::X));
 	}
 
+	QString text = wrapper.textPlaceholder;
 	if (wrapper.mode == TextLabel::Mode::Text)
 		text.replace(QStringLiteral("&amp;(x)"), xValueStr);
 	else
@@ -495,26 +549,6 @@ void InfoElement::labelTextWrapperChanged(TextLabel::TextWrapper) {
 	d->retransform();
 }
 
-/*!
- * \brief InfoElement::moveElementBegin
- * Called, when a child is moved in front or behind another element.
- * Needed, because the child calls child removed, when moving and then
- * everything will be deleted
- */
-void InfoElement::moveElementBegin() {
-	m_suppressChildRemoved = true;
-}
-
-/*!
- * \brief InfoElement::moveElementEnd
- * Called, when a child is moved in front or behind another element.
- * Needed, because the child calls child removed, when moving and then
- * everything will be deleted
- */
-void InfoElement::moveElementEnd() {
-	m_suppressChildRemoved = false;
-}
-
 void InfoElement::curveCoordinateSystemIndexChanged(int /*index*/) {
 	auto* curve = static_cast<XYCurve*>(QObject::sender());
 	auto cSystemIndex = curve->coordinateSystemIndex();
@@ -635,8 +669,7 @@ void InfoElement::childAdded(const AbstractAspect* child) {
 		// Must be done after setCoordinateBindingEnabled, otherwise positionChanged will be called and
 		// then the InfoElement position will be set incorrectly
 		connect(point, &CustomPoint::positionChanged, this, &InfoElement::pointPositionChanged);
-		connect(point, &CustomPoint::moveBegin, this, &InfoElement::moveElementBegin);
-		connect(point, &CustomPoint::moveEnd, this, &InfoElement::moveElementEnd);
+		connect(point, &CustomPoint::visibleChanged, this, &InfoElement::pointVisibleChanged);
 		return;
 	}
 
@@ -646,8 +679,6 @@ void InfoElement::childAdded(const AbstractAspect* child) {
 		connect(m_title, &TextLabel::visibleChanged, this, &InfoElement::labelVisibleChanged);
 		connect(m_title, &TextLabel::textWrapperChanged, this, &InfoElement::labelTextWrapperChanged);
 		connect(m_title, &TextLabel::borderShapeChanged, this, &InfoElement::labelBorderShapeChanged);
-		connect(m_title, &TextLabel::moveBegin, this, &InfoElement::moveElementBegin);
-		connect(m_title, &TextLabel::moveEnd, this, &InfoElement::moveElementEnd);
 		connect(m_title, &TextLabel::rotationAngleChanged, this, &InfoElement::retransform);
 
 		auto* l = const_cast<TextLabel*>(m_titleChild);
@@ -661,10 +692,14 @@ void InfoElement::childAdded(const AbstractAspect* child) {
  * \param new_x
  * \return
  */
-int InfoElement::currentIndex(double x, double* found_x) {
+int InfoElement::currentIndex(double x, double* found_x) const {
+	if (!isValid())
+		return -1;
 	for (auto& markerpoint : markerpoints) {
 		if (markerpoint.curve->name() == connectionLineCurveName()) {
-			int index = markerpoint.curve->xColumn()->indexForValue(x);
+			if (!markerpoint.curve->xColumn())
+				return -1;
+			const int index = markerpoint.curve->xColumn()->indexForValue(x);
 
 			if (found_x && index >= 0) {
 				auto mode = markerpoint.curve->xColumn()->columnMode();
@@ -694,18 +729,28 @@ int InfoElement::currentIndex(double x, double* found_x) {
 double InfoElement::setMarkerpointPosition(double x) {
 	// TODO: can be optimized when it will be checked if all markerpoints have the same xColumn, then the index m_index is the same
 	Q_D(InfoElement);
+	updateValid();
 	double x_new;
 	double x_new_first = 0;
 	for (int i = 0; i < markerpoints.length(); i++) {
+		auto* point = markerpoints[i].customPoint;
 		bool valueFound;
 		double y = markerpoints[i].curve->y(x, x_new, valueFound);
+		m_suppressVisibleChange = true;
+		if (y == std::nan("0")) {
+			point->setVisible(false);
+			m_title->setVisible(false);
+		} else {
+			point->setVisible(markerpoints[i].visible);
+			m_title->setVisible(true);
+		}
+		m_suppressVisibleChange = false;
 		d->positionLogical = x_new;
 		if (i == 0)
 			x_new_first = x_new;
 
 		if (valueFound) {
 			m_suppressChildPositionChanged = true;
-			auto* point = markerpoints[i].customPoint;
 			point->graphicsItem()->setFlag(QGraphicsItem::ItemSendsGeometryChanges, false);
 			point->setUndoAware(false);
 			point->setPositionLogical(QPointF(x_new, y));
@@ -738,10 +783,6 @@ void InfoElement::setParentGraphicsItem(QGraphicsItem* item) {
 	d->updatePosition();
 }
 
-QGraphicsItem* InfoElement::graphicsItem() const {
-	return d_ptr;
-}
-
 void InfoElement::retransform() {
 	Q_D(InfoElement);
 	d->retransform();
@@ -750,9 +791,26 @@ void InfoElement::retransform() {
 void InfoElement::handleResize(double /*horizontalRatio*/, double /*verticalRatio*/, bool /*pageResize*/) {
 }
 
-//##############################################################################
-//######  Getter and setter methods ############################################
-//##############################################################################
+void InfoElement::handleAspectUpdated(const QString& path, const AbstractAspect* aspect) {
+	const auto* curve = dynamic_cast<const XYCurve*>(aspect);
+	if (!curve)
+		return;
+
+	// The curve name changed and now it matches the correct path
+	// Add the curve again
+	for (auto& p : markerpoints) {
+		if (!p.curve && p.curvePath.compare(path) == 0) {
+			p.curve = curve;
+			updateValid();
+			retransform();
+			break;
+		}
+	}
+}
+
+// ##############################################################################
+// ######  Getter and setter methods ############################################
+// ##############################################################################
 
 /* ============================ getter methods ================= */
 BASIC_SHARED_D_READER_IMPL(InfoElement, double, positionLogical, positionLogical)
@@ -813,13 +871,50 @@ void InfoElement::setVisible(bool on) {
 		exec(new InfoElementSetVisibleCmd(d, on, on ? ki18n("%1: set visible") : ki18n("%1: set invisible")));
 }
 
-//##############################################################################
-//######  SLOTs for changes triggered via QActions in the context menu  ########
-//##############################################################################
+bool InfoElement::isValid() const {
+	Q_D(const InfoElement);
+	return d->valid;
+}
 
-//##############################################################################
-//####################### Private implementation ###############################
-//##############################################################################
+void InfoElement::updateValid() {
+	Q_D(InfoElement);
+	bool valid = false;
+	for (const auto& mp : markerpoints) {
+		if (mp.curve && mp.curve->xColumn() && mp.curve->yColumn())
+			valid = true; // at least one valid curve
+	}
+
+	d->valid = valid;
+
+	Lock lock(m_suppressVisibleChange);
+	m_title->setUndoAware(false);
+	m_title->setVisible(valid);
+	m_title->setUndoAware(true);
+
+	if (valid) {
+		for (auto& mp : markerpoints) {
+			if (mp.curve && mp.curve->xColumn() && mp.curve->yColumn()) {
+				mp.customPoint->setUndoAware(false);
+				mp.customPoint->setVisible(mp.visible);
+				mp.customPoint->setUndoAware(true);
+			}
+		}
+	} else {
+		for (auto& mp : markerpoints) {
+			mp.customPoint->setUndoAware(false);
+			mp.customPoint->setVisible(false);
+			mp.customPoint->setUndoAware(true);
+		}
+	}
+}
+
+// ##############################################################################
+// ######  SLOTs for changes triggered via QActions in the context menu  ########
+// ##############################################################################
+
+// ##############################################################################
+// ####################### Private implementation ###############################
+// ##############################################################################
 
 InfoElementPrivate::InfoElementPrivate(InfoElement* owner)
 	: WorksheetElementPrivate(owner)
@@ -850,16 +945,21 @@ void InfoElementPrivate::init() {
  */
 void InfoElementPrivate::retransform() {
 	DEBUG(Q_FUNC_INFO)
-	if (!q->m_title || q->markerpoints.isEmpty() || q->isLoading() || !parentItem())
+	q->updateValid();
+	if (!q->m_title || q->markerpoints.isEmpty() || q->isLoading() || !parentItem() || !valid) {
+		update(m_boundingRectangle);
 		return;
+	}
 
-	q->m_suppressChildPositionChanged = true;
+	Lock lock(q->m_suppressChildPositionChanged);
+	xposLine = QLineF();
+	m_connectionLine = QLineF();
 
 	// new bounding rectangle
 	const QRectF& rect = parentItem()->mapRectFromScene(q->plot()->rect());
-	boundingRectangle = mapFromParent(rect).boundingRect();
+	const QRectF& newBoundingRect = mapFromParent(rect).boundingRect();
 	QDEBUG(Q_FUNC_INFO << ", rect = " << rect)
-	QDEBUG(Q_FUNC_INFO << ", bounding rect = " << boundingRectangle)
+	QDEBUG(Q_FUNC_INFO << ", bounding rect = " << newBoundingRect)
 
 	// TODO: why do I need to retransform the label and the custompoints?
 	q->m_title->retransform();
@@ -868,16 +968,21 @@ void InfoElementPrivate::retransform() {
 
 	// determine the position to connect the line to
 	QPointF pointPos;
+	bool validPos = false;
 	for (int i = 0; i < q->markerPointsCount(); ++i) {
 		const auto* curve = q->markerpoints.at(i).curve;
-		if (curve && curve->name() == connectionLineCurveName) {
+		if (curve && curve->isVisible() && curve->name() == connectionLineCurveName) {
 			const auto& point = q->markerpoints.at(i).customPoint;
 			const auto* cSystem = q->plot()->coordinateSystem(point->coordinateSystemIndex());
 			pointPos = cSystem->mapLogicalToScene(point->positionLogical(), insidePlot, AbstractCoordinateSystem::MappingFlag::SuppressPageClippingVisible);
+			validPos = true;
 			break;
 		}
 	}
 
+	if (!validPos)
+		return;
+
 	if (!insidePlot)
 		return;
 
@@ -891,20 +996,18 @@ void InfoElementPrivate::retransform() {
 	// connection line
 	const QPointF m_titlePosItemCoords = mapFromParent(m_titlePos); // calculate item coords from scene coords
 	const QPointF pointPosItemCoords = mapFromParent(mapPlotAreaToParent(pointPos)); // calculate item coords from scene coords
-	if (boundingRectangle.contains(m_titlePosItemCoords) && boundingRectangle.contains(pointPosItemCoords))
+	if (newBoundingRect.contains(m_titlePosItemCoords) && newBoundingRect.contains(pointPosItemCoords))
 		m_connectionLine = QLineF(m_titlePosItemCoords.x(), m_titlePosItemCoords.y(), pointPosItemCoords.x(), pointPosItemCoords.y());
 	else
 		m_connectionLine = QLineF();
 	QDEBUG(Q_FUNC_INFO << ", connection line = " << m_connectionLine)
 
 	// vertical line
-	const QRectF& dataRect = mapFromParent(q->m_plot->dataRect()).boundingRect();
+	const QRectF& dataRect = mapFromParent(m_plot->dataRect()).boundingRect();
 	xposLine = QLineF(pointPosItemCoords.x(), dataRect.bottom(), pointPosItemCoords.x(), dataRect.top());
 	QDEBUG(Q_FUNC_INFO << ", vertical line " << xposLine)
 
-	update(boundingRectangle);
-
-	q->m_suppressChildPositionChanged = false;
+	recalcShapeAndBoundingRect(newBoundingRect);
 }
 
 void InfoElementPrivate::updatePosition() {
@@ -915,14 +1018,14 @@ void InfoElementPrivate::updatePosition() {
  * Repainting to update xposLine
  */
 void InfoElementPrivate::updateVerticalLine() {
-	update(boundingRectangle);
+	recalcShapeAndBoundingRect();
 }
 
 /*!
  * Repainting to updateConnectionLine
  */
 void InfoElementPrivate::updateConnectionLine() {
-	update(boundingRect());
+	recalcShapeAndBoundingRect();
 }
 
 bool InfoElementPrivate::changeVisibility(bool on) {
@@ -939,27 +1042,22 @@ bool InfoElementPrivate::changeVisibility(bool on) {
 	return oldValue;
 }
 
-// reimplemented from QGraphicsItem
-QRectF InfoElementPrivate::boundingRect() const {
-	return boundingRectangle;
-}
-
 void InfoElementPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) {
-	if (!insidePlot)
+	if (!insidePlot || !valid)
 		return;
 
 	if (q->markerpoints.isEmpty())
 		return;
 
 	// do not draw connection line when the label is not visible
-	if (connectionLine->style() != Qt::NoPen && q->m_title->isVisible()) {
+	if (connectionLine->style() != Qt::NoPen && q->m_title->isVisible() && !m_connectionLine.isNull()) {
 		painter->setOpacity(connectionLine->opacity());
 		painter->setPen(connectionLine->pen());
 		painter->drawLine(m_connectionLine);
 	}
 
 	// draw vertical line, which connects all points together
-	if (verticalLine->style() != Qt::NoPen) {
+	if (verticalLine->style() != Qt::NoPen && !xposLine.isNull()) {
 		painter->setOpacity(verticalLine->opacity());
 		painter->setPen(verticalLine->pen());
 		painter->drawLine(xposLine);
@@ -1022,16 +1120,16 @@ void InfoElementPrivate::mousePressEvent(QGraphicsSceneMouseEvent* event) {
 }
 
 void InfoElementPrivate::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
-	QPointF eventPos = mapToParent(event->pos());
+	const auto eventPos = mapToParent(event->pos());
 	// 	DEBUG("EventPos: " << eventPos.x() << " Y: " << eventPos.y());
-	QPointF delta = eventPos - oldMousePos;
+	const auto delta = eventPos - oldMousePos;
 	if (delta == QPointF(0, 0))
 		return;
 
 	if (!q->cSystem->isValid())
 		return;
-	QPointF eventLogicPos = q->cSystem->mapSceneToLogical(eventPos, AbstractCoordinateSystem::MappingFlag::SuppressPageClipping);
-	QPointF delta_logic = eventLogicPos - q->cSystem->mapSceneToLogical(oldMousePos);
+	const auto eventLogicPos = q->cSystem->mapSceneToLogical(eventPos, AbstractCoordinateSystem::MappingFlag::SuppressPageClipping);
+	const auto delta_logic = eventLogicPos - q->cSystem->mapSceneToLogical(oldMousePos);
 
 	if (!q->m_title)
 		return;
@@ -1052,6 +1150,8 @@ void InfoElementPrivate::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 	}
 	x += delta_logic.x();
 	auto xColumn = q->markerpoints[activeIndex].curve->xColumn();
+	if (!xColumn)
+		return;
 	int xindex = xColumn->indexForValue(x);
 	double x_new = NAN;
 	if (xColumn->isNumeric())
@@ -1076,7 +1176,6 @@ void InfoElementPrivate::keyPressEvent(QKeyEvent* event) {
 		return QGraphicsItem::keyPressEvent(event);
 	}
 
-	TextLabel::TextWrapper text;
 	if (event->key() == Qt::Key_Right || event->key() == Qt::Key_Left) {
 		int index;
 		if (event->key() == Qt::Key_Right)
@@ -1090,6 +1189,8 @@ void InfoElementPrivate::keyPressEvent(QKeyEvent* event) {
 		for (int i = 0; i < q->markerPointsCount(); i++) {
 			const auto* curve = q->markerpoints[i].curve;
 			if (curve->name().compare(connectionLineCurveName) == 0) {
+				if (!curve->xColumn())
+					return;
 				auto rowCount = curve->xColumn()->rowCount();
 				m_index += index;
 				if (m_index > rowCount - 1)
@@ -1106,9 +1207,55 @@ void InfoElementPrivate::keyPressEvent(QKeyEvent* event) {
 	}
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+bool InfoElementPrivate::activate(QPointF mouseScenePos, double /*maxDist*/) {
+	if (!isVisible())
+		return false;
+
+	return m_shape.contains(mouseScenePos);
+}
+
+void InfoElementPrivate::contextMenuEvent(QGraphicsSceneContextMenuEvent* event) {
+	if (activate(event->pos())) {
+		q->createContextMenu()->exec(event->screenPos());
+		return;
+	}
+	QGraphicsItem::contextMenuEvent(event);
+}
+
+void InfoElementPrivate::recalcShape() {
+	m_shape = QPainterPath();
+
+	if (verticalLine->style() != Qt::PenStyle::NoPen) {
+		QPainterPath path;
+		path.moveTo(xposLine.p1());
+		path.lineTo(xposLine.p2());
+		m_shape.addPath(WorksheetElement::shapeFromPath(path, verticalLine->pen()));
+	}
+
+	if (connectionLine->style() != Qt::PenStyle::NoPen) {
+		QPainterPath path;
+		path.moveTo(m_connectionLine.p1());
+		path.lineTo(m_connectionLine.p2());
+		m_shape.addPath(WorksheetElement::shapeFromPath(path, connectionLine->pen()));
+	}
+}
+
+void InfoElementPrivate::recalcShapeAndBoundingRect(const QRectF& rect) {
+	prepareGeometryChange();
+	m_boundingRectangle = rect;
+	recalcShape();
+	update(m_boundingRectangle);
+}
+
+void InfoElementPrivate::recalcShapeAndBoundingRect() {
+	prepareGeometryChange();
+	recalcShape();
+	update(m_boundingRectangle);
+}
+
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 void InfoElement::save(QXmlStreamWriter* writer) const {
 	Q_D(const InfoElement);
 
@@ -1139,6 +1286,7 @@ void InfoElement::save(QXmlStreamWriter* writer) const {
 		for (const auto& custompoint : markerpoints) {
 			writer->writeStartElement(QStringLiteral("point"));
 			writer->writeAttribute(QLatin1String("curvepath"), custompoint.curve->path());
+			writer->writeAttribute(QLatin1String("visible"), QString::number(custompoint.visible));
 			custompoint.customPoint->save(writer);
 			writer->writeEndElement(); // close "point"
 		}
@@ -1155,9 +1303,7 @@ bool InfoElement::load(XmlStreamReader* reader, bool preview) {
 	Q_D(InfoElement);
 
 	QXmlStreamAttributes attribs;
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QString str;
-	QString curvePath;
 
 	while (!reader->atEnd()) {
 		reader->readNext();
@@ -1177,7 +1323,7 @@ bool InfoElement::load(XmlStreamReader* reader, bool preview) {
 
 			str = attribs.value(QStringLiteral("visible")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("x")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("x"));
 			else
 				setVisible(str.toInt());
 
@@ -1191,42 +1337,65 @@ bool InfoElement::load(XmlStreamReader* reader, bool preview) {
 			d->connectionLine->load(reader, preview);
 		} else if (reader->name() == QLatin1String("textLabel")) {
 			if (!m_title) {
-				m_title = new TextLabel(i18n("Label"), m_plot);
+				m_title = new TextLabel(i18n("Label"), d->m_plot);
 				m_title->setIsLoading(true);
 				this->addChild(m_title);
 			}
 			if (!m_title->load(reader, preview))
 				return false;
-		} else if (reader->name() == QLatin1String("customPoint")) {
-			if (curvePath.isEmpty()) // safety check in case the xml is broken
-				continue;
-
-			auto* point = new CustomPoint(m_plot, QString());
-			point->setIsLoading(true);
-			if (!point->load(reader, preview)) {
-				delete point;
+		} else if (reader->name() == QLatin1String("points")) {
+			loadPoints(reader, preview);
+		} else { // unknown element
+			reader->raiseUnknownElementWarning();
+			if (!reader->skipToEndElement())
 				return false;
-			}
-			this->addChild(point);
-			addCurvePath(curvePath, point);
-			curvePath.clear();
-		} else if (reader->name() == QLatin1String("point")) {
-			attribs = reader->attributes();
-			curvePath = attribs.value(QStringLiteral("curvepath")).toString();
 		}
 	}
 
 	return true;
 }
 
-//##############################################################################
-//#########################  Theme management ##################################
-//##############################################################################
+void InfoElement::loadPoints(XmlStreamReader* reader, bool preview) {
+	reader->readNextStartElement();
+	if (!reader->isStartElement())
+		return;
+
+	while (!(reader->isEndElement() && reader->name() == QStringLiteral("points"))) {
+		Q_D(const InfoElement);
+		if (!reader->isStartElement()) {
+			reader->readNext();
+			continue;
+		}
+		if (reader->name() != QLatin1String("point"))
+			break;
+		const auto& attribs = reader->attributes();
+		const QString curvePath = attribs.value(QStringLiteral("curvepath")).toString();
+		const bool visible = attribs.value(QStringLiteral("visible")).toInt();
+
+		reader->readNextStartElement();
+		if (reader->name() != CustomPoint::xmlName())
+			break;
+
+		auto* point = new CustomPoint(d->m_plot, QString());
+		point->setIsLoading(true);
+		if (!point->load(reader, preview)) {
+			delete point;
+			return;
+		}
+		point->setVisible(visible);
+		this->addChild(point);
+		addCurvePath(curvePath, point);
+	}
+}
+
+// ##############################################################################
+// #########################  Theme management ##################################
+// ##############################################################################
 void InfoElement::loadThemeConfig(const KConfig& config) {
 	// use the color for the axis line from the theme also for info element's lines
-	const KConfigGroup& group = config.group("Axis");
+	const KConfigGroup& group = config.group(QStringLiteral("Axis"));
 
-	const QColor& themeColor = group.readEntry("LineColor", QColor(Qt::black));
+	const QColor& themeColor = group.readEntry(QStringLiteral("LineColor"), QColor(Qt::black));
 	Q_D(InfoElement);
 	d->verticalLine->loadThemeConfig(group, themeColor);
 	d->connectionLine->loadThemeConfig(group, themeColor);
