@@ -30,22 +30,17 @@ class InfoElement : public WorksheetElement {
 
 public:
 	InfoElement(const QString& name, CartesianPlot*);
-	InfoElement(const QString& name, CartesianPlot*, const XYCurve*, double pos);
-	void setParentGraphicsItem(QGraphicsItem* item);
+	InfoElement(const QString& name, CartesianPlot*, const XYCurve*, double logicalPos);
+	virtual void setParentGraphicsItem(QGraphicsItem* item) override;
 	~InfoElement();
 
 	struct MarkerPoints_T {
 		MarkerPoints_T() = default;
-		MarkerPoints_T(CustomPoint* custompoint, QString customPointPath, const XYCurve* curve, QString curvePath)
-			: customPoint(custompoint)
-			, customPointPath(customPointPath)
-			, curve(curve)
-			, curvePath(curvePath) {
-		}
+		MarkerPoints_T(CustomPoint* custompoint, const XYCurve* curve, QString curvePath);
 		CustomPoint* customPoint{nullptr};
-		QString customPointPath;
 		const XYCurve* curve{nullptr};
 		QString curvePath;
+		bool visible{true};
 	};
 
 	void save(QXmlStreamWriter*) const override;
@@ -54,22 +49,21 @@ public:
 
 	TextLabel* title();
 	void addCurve(const XYCurve*, CustomPoint* = nullptr);
-	void addCurvePath(QString& curvePath, CustomPoint* = nullptr);
+	void addCurvePath(const QString& curvePath, CustomPoint* = nullptr);
 	bool assignCurve(const QVector<XYCurve*>&);
 	void removeCurve(const XYCurve*);
+	void curveDeleted(const AbstractAspect*);
 	void setZValue(qreal) override;
-	int markerPointsCount();
-	MarkerPoints_T markerPointAt(int index);
-	int gluePointsCount();
-	TextLabel::GluePoint gluePoint(int index);
+	int markerPointsCount() const;
+	MarkerPoints_T markerPointAt(int index) const;
+	int gluePointsCount() const;
+	TextLabel::GluePoint gluePoint(int index) const;
 	TextLabel::TextWrapper createTextLabelText();
 	QMenu* createContextMenu() override;
 
 	bool isTextLabel() const;
 	double setMarkerpointPosition(double x);
-	int currentIndex(double new_x, double* found_x = nullptr);
-
-	QGraphicsItem* graphicsItem() const override;
+	int currentIndex(double new_x, double* found_x = nullptr) const;
 
 	void retransform() override;
 	void handleResize(double horizontalRatio, double verticalRatio, bool pageResize) override;
@@ -79,6 +73,7 @@ public:
 	CLASS_D_ACCESSOR_DECL(QString, connectionLineCurveName, ConnectionLineCurveName)
 	Line* verticalLine() const;
 	Line* connectionLine() const;
+	bool isValid() const;
 
 	virtual void setVisible(bool on) override;
 
@@ -92,34 +87,30 @@ public Q_SLOTS:
 	void childAdded(const AbstractAspect*);
 	void labelBorderShapeChanged();
 	void labelTextWrapperChanged(TextLabel::TextWrapper);
-	void moveElementBegin();
-	void moveElementEnd();
 	void curveVisibilityChanged();
 	void curveDataChanged();
 	void curveCoordinateSystemIndexChanged(int);
+	void pointVisibleChanged(bool visible);
 
 private:
 	Q_DECLARE_PRIVATE(InfoElement)
 	TextLabel* m_title{nullptr};
+	bool m_suppressVisibleChange{false};
 	QVector<struct MarkerPoints_T> markerpoints;
 	bool m_menusInitialized{false};
 	bool m_suppressChildRemoved{false};
 	bool m_suppressChildPositionChanged{false};
 	bool m_setTextLabelText{false};
-	/*!
-	 * This variable is set when a curve is moved in the order, because there
-	 * the curve is removed and readded and we would like to ignore this remove and
-	 * add. Because of single thread it makes no problems.
-	 */
-	bool m_curveGetsMoved{false};
-
-	// Actions
-	QAction* visibilityAction;
 
 	void init();
 	void initActions();
 	void initMenus();
 	void initCurveConnections(const XYCurve*);
+	void initCustomPointConnections(const CustomPoint*);
+	void updateValid();
+	void setConnectionLineNextValidCurve();
+	virtual void handleAspectUpdated(const QString& path, const AbstractAspect*) override;
+	void loadPoints(XmlStreamReader* reader, bool preview);
 
 Q_SIGNALS:
 	void gluePointIndexChanged(const int);
