@@ -14,8 +14,10 @@
 #include "TextLabelPrivate.h"
 #include "Worksheet.h"
 #include "backend/core/Project.h"
+#include "backend/core/Settings.h"
 #include "backend/lib/XmlStreamReader.h"
 #include "backend/lib/commandtemplates.h"
+#include "backend/lib/macros.h"
 #include "backend/worksheet/plots/PlotArea.h"
 #include "backend/worksheet/plots/cartesian/CartesianCoordinateSystem.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
@@ -23,7 +25,6 @@
 
 #include <QApplication>
 #include <QBuffer>
-#include <QDesktopWidget>
 #include <QGraphicsScene>
 #include <QGraphicsSceneMouseEvent>
 #include <QKeyEvent>
@@ -49,6 +50,17 @@ class ScaledTextItem : public QGraphicsTextItem {
 public:
 	explicit ScaledTextItem(QGraphicsItem* parent = nullptr)
 		: QGraphicsTextItem(parent) {
+		auto textOptions = document()->defaultTextOption();
+		textOptions.setWrapMode(QTextOption::NoWrap);
+		document()->setDefaultTextOption(textOptions);
+	}
+
+	QRectF scaledBoundingRect() const {
+		auto rect = QGraphicsTextItem::boundingRect();
+		rect.setHeight(rect.height() * scale());
+		rect.setWidth(rect.width() * scale());
+		// rect.moveTopLeft(QPointF(-rect.width() * scale() / 2, -rect.height() * scale() / 2));
+		return rect;
 	}
 
 protected:
@@ -56,15 +68,18 @@ protected:
 #if DEBUG_TEXTLABEL_BOUNDING_RECT
 		painter->setPen(QColor(Qt::GlobalColor::green));
 		painter->drawRect(boundingRect());
-#endif
 
-#if DEBUG_TEXTLABEL_BOUNDING_RECT
 		painter->setPen(QColor(Qt::GlobalColor::black));
 		painter->drawRect(QRectF(-5, -5, 10, 10));
 #endif
 		QGraphicsTextItem::paint(painter, option, widget);
 	}
 
+private:
+	QRectF boundingRect() const override {
+		return QGraphicsTextItem::boundingRect();
+	}
+
 private:
 };
 
@@ -93,8 +108,9 @@ TextLabel::TextLabel(const QString& name, TextLabelPrivate* dd, Type type)
 TextLabel::TextLabel(const QString& name, CartesianPlot* plot, Type type)
 	: WorksheetElement(name, new TextLabelPrivate(this), AspectType::TextLabel)
 	, m_type(type) {
-	m_plot = plot;
-	cSystem = dynamic_cast<const CartesianCoordinateSystem*>(m_plot->coordinateSystem(m_cSystemIndex));
+	Q_D(TextLabel);
+	d->m_plot = plot;
+	cSystem = dynamic_cast<const CartesianCoordinateSystem*>(d->m_plot->coordinateSystem(m_cSystemIndex));
 	init();
 }
 
@@ -135,42 +151,43 @@ void TextLabel::init() {
 		d->position.verticalPosition = WorksheetElement::VerticalPosition::Center;
 	}
 
-	KConfigGroup conf(KSharedConfig::openConfig(), QLatin1String("Settings_Worksheet"));
-	const auto& engine = conf.readEntry(QLatin1String("LaTeXEngine"), "");
+	KConfigGroup conf = Settings::group(QStringLiteral("Settings_Worksheet"));
+	const auto& engine = conf.readEntry(QStringLiteral("LaTeXEngine"), "");
 	if (engine == QLatin1String("lualatex"))
-		d->teXFont.setFamily(QLatin1String("Latin Modern Roman"));
+		d->teXFont.setFamily(QStringLiteral("Latin Modern Roman"));
 
 	// read settings from config if group exists
 	if (group.isValid()) {
 		// properties common to all types
-		d->textWrapper.mode = static_cast<TextLabel::Mode>(group.readEntry("Mode", static_cast<int>(d->textWrapper.mode)));
-		d->teXFont.setFamily(group.readEntry("TeXFontFamily", d->teXFont.family()));
-		d->teXFont.setPointSize(group.readEntry("TeXFontSize", d->teXFont.pointSize()));
-		d->fontColor = group.readEntry("TeXFontColor", d->fontColor);
-		d->backgroundColor = group.readEntry("TeXBackgroundColor", d->backgroundColor);
-		d->setRotation(group.readEntry("Rotation", d->rotation()));
+		d->textWrapper.mode = static_cast<TextLabel::Mode>(group.readEntry(QStringLiteral("Mode"), static_cast<int>(d->textWrapper.mode)));
+		d->teXFont.setFamily(group.readEntry(QStringLiteral("TeXFontFamily"), d->teXFont.family()));
+		d->teXFont.setPointSize(group.readEntry(QStringLiteral("TeXFontSize"), d->teXFont.pointSize()));
+		d->fontColor = group.readEntry(QStringLiteral("FontColor"), d->fontColor);
+		d->backgroundColor = group.readEntry(QStringLiteral("BackgroundColor"), d->backgroundColor);
+		d->setRotation(group.readEntry(QStringLiteral("Rotation"), d->rotation()));
 
 		// border
-		d->borderShape = (TextLabel::BorderShape)group.readEntry("BorderShape", (int)d->borderShape);
-		d->borderPen = QPen(group.readEntry("BorderColor", d->borderPen.color()),
-							group.readEntry("BorderWidth", d->borderPen.width()),
-							(Qt::PenStyle)group.readEntry("BorderStyle", (int)(d->borderPen.style())));
-		d->borderOpacity = group.readEntry("BorderOpacity", d->borderOpacity);
+		d->borderShape = (TextLabel::BorderShape)group.readEntry(QStringLiteral("BorderShape"), (int)d->borderShape);
+		d->borderPen = QPen(group.readEntry(QStringLiteral("BorderColor"), d->borderPen.color()),
+							group.readEntry(QStringLiteral("BorderWidth"), d->borderPen.width()),
+							(Qt::PenStyle)group.readEntry(QStringLiteral("BorderStyle"), (int)(d->borderPen.style())));
+		d->borderOpacity = group.readEntry(QStringLiteral("BorderOpacity"), d->borderOpacity);
 
 		// position and alignment relevant properties
-		d->position.point.setX(group.readEntry("PositionXValue", 0.));
-		d->position.point.setY(group.readEntry("PositionYValue", 0.));
-		d->position.horizontalPosition = (HorizontalPosition)group.readEntry("PositionX", (int)d->position.horizontalPosition);
-		d->position.verticalPosition = (VerticalPosition)group.readEntry("PositionY", (int)d->position.verticalPosition);
-		d->horizontalAlignment = (WorksheetElement::HorizontalAlignment)group.readEntry("HorizontalAlignment", static_cast<int>(d->horizontalAlignment));
-		d->verticalAlignment = (WorksheetElement::VerticalAlignment)group.readEntry("VerticalAlignment", static_cast<int>(d->verticalAlignment));
+		d->position.point.setX(group.readEntry(QStringLiteral("PositionXValue"), 0.));
+		d->position.point.setY(group.readEntry(QStringLiteral("PositionYValue"), 0.));
+		d->position.horizontalPosition = (HorizontalPosition)group.readEntry(QStringLiteral("PositionX"), (int)d->position.horizontalPosition);
+		d->position.verticalPosition = (VerticalPosition)group.readEntry(QStringLiteral("PositionY"), (int)d->position.verticalPosition);
+		d->horizontalAlignment =
+			(WorksheetElement::HorizontalAlignment)group.readEntry(QStringLiteral("HorizontalAlignment"), static_cast<int>(d->horizontalAlignment));
+		d->verticalAlignment =
+			(WorksheetElement::VerticalAlignment)group.readEntry(QStringLiteral("VerticalAlignment"), static_cast<int>(d->verticalAlignment));
 		if (cSystem && cSystem->isValid())
 			d->positionLogical = cSystem->mapSceneToLogical(d->position.point, AbstractCoordinateSystem::MappingFlag::SuppressPageClipping);
 	}
 	d->updatePosition();
 
-	DEBUG(Q_FUNC_INFO << ", default/run time image resolution: " << d->teXImageResolution << '/' << QApplication::desktop()->physicalDpiX());
-
+	DEBUG(Q_FUNC_INFO << ", default/run time image resolution: " << d->teXImageResolution << '/' << QApplication::primaryScreen()->physicalDotsPerInchX());
 	connect(&d->teXImageFutureWatcher, &QFutureWatcher<QByteArray>::finished, this, &TextLabel::updateTeXImage);
 }
 
@@ -178,20 +195,6 @@ void TextLabel::init() {
 // and is deleted during the cleanup in QGraphicsScene
 TextLabel::~TextLabel() = default;
 
-QGraphicsItem* TextLabel::graphicsItem() const {
-	return d_ptr;
-}
-
-/*!
- * \brief TextLabel::setParentGraphicsItem
- * Sets the parent graphicsitem, needed for binding to coord
- * \param item parent graphicsitem
- */
-void TextLabel::setParentGraphicsItem(QGraphicsItem* item) {
-	Q_D(TextLabel);
-	d->setParentItem(item);
-}
-
 void TextLabel::setZoomFactor(double factor) {
 	Q_D(TextLabel);
 	d->setZoomFactor(factor);
@@ -233,35 +236,18 @@ void TextLabel::handleResize(double horizontalRatio, double verticalRatio, bool
 QIcon TextLabel::icon() const {
 	switch (text().mode) {
 	case Mode::Markdown:
-		return QIcon::fromTheme(QLatin1String("text-x-markdown"));
+		return QIcon::fromTheme(QStringLiteral("text-x-markdown"));
 		break;
 	case Mode::LaTeX:
-		return QIcon::fromTheme(QLatin1String("text-x-tex"));
+		return QIcon::fromTheme(QStringLiteral("text-x-tex"));
 		break;
 	case Mode::Text:
 	default:
-		return QIcon::fromTheme(QLatin1String("draw-text"));
+		return QIcon::fromTheme(QStringLiteral("draw-text"));
 		// return QIcon::fromTheme(QLatin1String("text-x-plain"));
 	}
 }
 
-QMenu* TextLabel::createContextMenu() {
-	QMenu* menu = WorksheetElement::createContextMenu();
-	QAction* firstAction = menu->actions().at(1); // skip the first action because of the "title-action"
-
-	if (!visibilityAction) {
-		visibilityAction = new QAction(i18n("Visible"), this);
-		visibilityAction->setCheckable(true);
-		connect(visibilityAction, &QAction::triggered, this, &TextLabel::changeVisibility);
-	}
-
-	visibilityAction->setChecked(isVisible());
-	menu->insertAction(firstAction, visibilityAction);
-	menu->insertSeparator(firstAction);
-
-	return menu;
-}
-
 /* ============================ getter methods ================= */
 BASIC_SHARED_D_READER_IMPL(TextLabel, TextLabel::TextWrapper, text, textWrapper)
 BASIC_SHARED_D_READER_IMPL(TextLabel, QColor, fontColor, fontColor)
@@ -272,23 +258,25 @@ BASIC_SHARED_D_READER_IMPL(TextLabel, QPen, borderPen, borderPen)
 BASIC_SHARED_D_READER_IMPL(TextLabel, qreal, borderOpacity, borderOpacity)
 
 /* ============================ setter methods and undo commands ================= */
+STD_SETTER_CMD_IMPL_F_S(TextLabel, SetTeXBackgroundColor, QColor, backgroundColor, updateText)
+void TextLabel::setBackgroundColor(const QColor color) {
+	QDEBUG(Q_FUNC_INFO << ", color = " << color)
+	Q_D(TextLabel);
+	if (color != d->backgroundColor)
+		exec(new TextLabelSetTeXBackgroundColorCmd(d, color, ki18n("%1: set background color")));
+}
+
 STD_SETTER_CMD_IMPL_F_S(TextLabel, SetText, TextLabel::TextWrapper, textWrapper, updateText)
 void TextLabel::setText(const TextWrapper& textWrapper) {
 	Q_D(TextLabel);
 	// DEBUG("********************\n" << Q_FUNC_INFO << ", old/new mode = " << (int)d->textWrapper.mode << " " << (int)textWrapper.mode)
 	// DEBUG("\nOLD TEXT = " << STDSTRING(d->textWrapper.text) << '\n')
 	// DEBUG("NEW TEXT = " << STDSTRING(textWrapper.text) << '\n')
-
 	// QDEBUG("COLORS: color =" << d->fontColor << ", background color =" << d->backgroundColor)
 
-	if ((textWrapper.text != d->textWrapper.text) || (textWrapper.mode != d->textWrapper.mode)
-		|| ((d->textWrapper.allowPlaceholder || textWrapper.allowPlaceholder) && (textWrapper.textPlaceholder != d->textWrapper.textPlaceholder))
-		|| textWrapper.allowPlaceholder != d->textWrapper.allowPlaceholder) {
+	if (d->textWrapper != textWrapper) {
 		bool oldEmpty = d->textWrapper.text.isEmpty();
 		if (textWrapper.mode == TextLabel::Mode::Text && !textWrapper.text.isEmpty()) {
-			// DEBUG("SET TEXTWRAPPER")
-			TextWrapper tw = textWrapper;
-
 			QTextEdit pte(d->textWrapper.text); // te with previous text
 			// restore formatting when text changes or switching back to text mode
 			if (d->textWrapper.mode != TextLabel::Mode::Text || oldEmpty || pte.toPlainText().isEmpty()) {
@@ -300,13 +288,39 @@ void TextLabel::setText(const TextWrapper& textWrapper) {
 				te.setTextColor(d->fontColor);
 				te.setTextBackgroundColor(d->backgroundColor);
 
+				TextWrapper tw = textWrapper;
 				tw.text = te.toHtml();
 				// DEBUG("\nTW TEXT = " << STDSTRING(tw.text) << std::endl)
-			}
+				exec(new TextLabelSetTextCmd(d, tw, ki18n("%1: set label text")));
+			} else { // the existing text is being modified
+				QUndoCommand* parent = nullptr;
+				TextLabelSetTextCmd* command = nullptr;
+				QTextEdit te;
+				te.setHtml(textWrapper.text);
+				te.selectAll();
+				if (textWrapper.text.indexOf(QStringLiteral("background-color:#")) != -1) {
+					const auto& bgColor = te.textBackgroundColor();
+					if (bgColor != d->backgroundColor) {
+						parent = new QUndoCommand(ki18n("%1: set label text").subs(name()).toString());
+						new TextLabelSetTeXBackgroundColorCmd(d, bgColor, ki18n("%1: set background color"), parent);
+					}
+					command = new TextLabelSetTextCmd(d, textWrapper, ki18n("%1: set label text"), parent);
+				} else {
+					// no color available yet, plain text is being provided -> set the color from member variable
+					te.setTextBackgroundColor(d->backgroundColor);
+					TextWrapper tw = textWrapper;
+					tw.text = te.toHtml();
+					command = new TextLabelSetTextCmd(d, tw, ki18n("%1: set label text"), parent);
+				}
 
-			exec(new TextLabelSetTextCmd(d, tw, ki18n("%1: set label text")));
+				if (!parent)
+					exec(command);
+				else
+					exec(parent);
+			}
 		} else
 			exec(new TextLabelSetTextCmd(d, textWrapper, ki18n("%1: set label text")));
+
 		// If previously the text was empty, the bounding rect is zero
 		// therefore the alignment did not work properly.
 		// If text is added, the bounding rectangle is updated
@@ -339,14 +353,6 @@ void TextLabel::setFontColor(const QColor color) {
 		exec(new TextLabelSetTeXFontColorCmd(d, color, ki18n("%1: set font color")));
 }
 
-STD_SETTER_CMD_IMPL_F_S(TextLabel, SetTeXBackgroundColor, QColor, backgroundColor, updateText)
-void TextLabel::setBackgroundColor(const QColor color) {
-	QDEBUG(Q_FUNC_INFO << ", color = " << color)
-	Q_D(TextLabel);
-	if (color != d->backgroundColor)
-		exec(new TextLabelSetTeXBackgroundColorCmd(d, color, ki18n("%1: set background color")));
-}
-
 // Border
 STD_SETTER_CMD_IMPL_F_S(TextLabel, SetBorderShape, TextLabel::BorderShape, borderShape, updateBorder)
 void TextLabel::setBorderShape(TextLabel::BorderShape shape) {
@@ -388,7 +394,7 @@ TextLabel::GluePoint TextLabel::gluePointAt(int index) {
 
 int TextLabel::gluePointCount() {
 	Q_D(const TextLabel);
-	return d->m_gluePointsTransformed.length();
+	return d->m_gluePoints.length();
 }
 
 void TextLabel::updateTeXImage() {
@@ -396,9 +402,9 @@ void TextLabel::updateTeXImage() {
 	d->updateTeXImage();
 }
 
-//##############################################################################
-//####################### Private implementation ###############################
-//##############################################################################
+// ##############################################################################
+// ####################### Private implementation ###############################
+// ##############################################################################
 TextLabelPrivate::TextLabelPrivate(TextLabel* owner)
 	: WorksheetElementPrivate(owner)
 	, q(owner) {
@@ -439,8 +445,8 @@ QRectF TextLabelPrivate::size() {
 		// if (textWrapper.mode == TextLabel::Mode::Markdown && textWrapper.text.contains(QLatin1Char('\n')))
 		//	yScale = 0.95;
 		// see updateBoundingRect()
-		w = m_textItem->boundingRect().width() * m_textItem->scale();
-		h = m_textItem->boundingRect().height() * m_textItem->scale();
+		w = m_textItem->scaledBoundingRect().width();
+		h = m_textItem->scaledBoundingRect().height();
 	}
 	qreal x = position.point.x();
 	qreal y = position.point.y();
@@ -454,18 +460,18 @@ QRectF TextLabelPrivate::size() {
  * \return Nearest point to @param point
  */
 QPointF TextLabelPrivate::findNearestGluePoint(QPointF scenePoint) {
-	if (m_gluePointsTransformed.isEmpty())
-		return boundingRectangle.center();
+	if (m_gluePoints.isEmpty())
+		return boundingRect().center();
 
-	if (m_gluePointsTransformed.length() == 1)
-		return mapParentToPlotArea(mapToParent(m_gluePointsTransformed.at(0).point));
+	if (m_gluePoints.length() == 1)
+		return mapParentToPlotArea(mapToParent(m_gluePoints.at(0).point));
 
-	QPointF point = mapParentToPlotArea(mapToParent(m_gluePointsTransformed.at(0).point));
+	QPointF point = mapParentToPlotArea(mapToParent(m_gluePoints.at(0).point));
 	QPointF nearestPoint = point;
 	double distance2 = pow(point.x() - scenePoint.x(), 2) + pow(point.y() - scenePoint.y(), 2);
 	// assumption, more than one point available
-	for (int i = 1; i < m_gluePointsTransformed.length(); i++) {
-		point = mapParentToPlotArea(mapToParent(m_gluePointsTransformed.at(i).point));
+	for (int i = 1; i < m_gluePoints.length(); i++) {
+		point = mapParentToPlotArea(mapToParent(m_gluePoints.at(i).point));
 		double distance2_temp = pow(point.x() - scenePoint.x(), 2) + pow(point.y() - scenePoint.y(), 2);
 		if (distance2_temp < distance2) {
 			nearestPoint = point;
@@ -486,15 +492,15 @@ TextLabel::GluePoint TextLabelPrivate::gluePointAt(int index) {
 	QPointF pos;
 	QString name;
 
-	if (m_gluePointsTransformed.isEmpty() || index > m_gluePointsTransformed.length()) {
-		pos = boundingRectangle.center();
+	if (m_gluePoints.isEmpty() || index > m_gluePoints.length()) {
+		pos = boundingRect().center();
 		name = QLatin1String("center");
 	} else if (index < 0) {
-		pos = m_gluePointsTransformed.at(0).point;
-		name = m_gluePointsTransformed.at(0).name;
+		pos = m_gluePoints.at(0).point;
+		name = m_gluePoints.at(0).name;
 	} else {
-		pos = m_gluePointsTransformed.at(index).point;
-		name = m_gluePointsTransformed.at(index).name;
+		pos = m_gluePoints.at(index).point;
+		name = m_gluePoints.at(index).name;
 	}
 
 	return {mapParentToPlotArea(mapToParent(pos)), name};
@@ -511,8 +517,6 @@ void TextLabelPrivate::retransform() {
 
 	updatePosition(); // needed, because CartesianPlot calls retransform if some operations are done
 	updateBorder();
-
-	Q_EMIT q->changed();
 }
 
 void TextLabelPrivate::setZoomFactor(double factor) {
@@ -544,7 +548,7 @@ void TextLabelPrivate::updatePosition() {
 		// the position in scene coordinates was changed, calculate the position in logical coordinates
 		if (q->cSystem) {
 			if (!coordinateBindingEnabled) {
-				QPointF pos = q->align(p, boundingRectangle, horizontalAlignment, verticalAlignment, false);
+				QPointF pos = q->align(p, m_boundingRectangle, horizontalAlignment, verticalAlignment, false);
 				positionLogical = q->cSystem->mapSceneToLogical(pos, AbstractCoordinateSystem::MappingFlag::SuppressPageClipping);
 			}
 			Q_EMIT q->positionLogicalChanged(positionLogical);
@@ -570,12 +574,14 @@ void TextLabelPrivate::updateText() {
 			QTextEdit te(textWrapper.text);
 			te.selectAll();
 			te.setTextColor(fontColor);
+
 			// don't set background color (not used if not inside text)
 			textWrapper.text = te.toHtml();
 		}
 
 		m_textItem->show();
 		m_textItem->setHtml(textWrapper.text);
+		m_textItem->adjustSize(); // has to be called so the text alignment is properly updated
 
 		// the size of the label was most probably changed,
 		// recalculate the bounding box of the label
@@ -654,7 +660,7 @@ void TextLabelPrivate::updateBoundingRect() {
 		h = teXImage.height() * teXImageScaleFactor / zoomFactor;
 	} else {
 		// size is in points, convert to scene units
-		// QDEBUG(" BOUNDING RECT = " << m_textItem->boundingRect())
+		// QDEBUG(" BOUNDING RECT = " << m_textItem->scaledBoundingRect())
 		//  TODO: the shift and scaling is just a workaround to avoid the big bounding box
 		//  s.a. TextLabelPrivate::size()
 
@@ -662,17 +668,18 @@ void TextLabelPrivate::updateBoundingRect() {
 		//  better scaling for multiline Markdown
 		// if (textWrapper.mode == TextLabel::Mode::Markdown && textWrapper.text.contains(QLatin1Char('\n')))
 		//	yScale = 0.95;
-		w = m_textItem->boundingRect().width() * m_textItem->scale(); // - xShift;
-		h = m_textItem->boundingRect().height() * m_textItem->scale(); // * yScale;
-		m_textItem->setPos(QPointF(-w / 2, -h / 2));
+		w = m_textItem->scaledBoundingRect().width(); // - xShift;
+		h = m_textItem->scaledBoundingRect().height(); // * yScale;
+		m_textItem->setPos(-w / 2, -h / 2);
 	}
 
 	// DEBUG(Q_FUNC_INFO << ", scale factor = " << scaleFactor << ", w/h = " << w << " / " << h)
-	boundingRectangle.setX(-w / 2);
-	boundingRectangle.setY(-h / 2);
-	boundingRectangle.setWidth(w);
-	boundingRectangle.setHeight(h);
+	m_boundingRectangle.setX(-w / 2);
+	m_boundingRectangle.setY(-h / 2);
+	m_boundingRectangle.setWidth(w);
+	m_boundingRectangle.setHeight(h);
 
+	updatePosition();
 	updateBorder();
 }
 
@@ -696,34 +703,31 @@ void TextLabelPrivate::updateBorder() {
 	using GluePoint = TextLabel::GluePoint;
 
 	borderShapePath = QPainterPath();
+	const auto& br = m_boundingRectangle;
 	switch (borderShape) {
 	case (TextLabel::BorderShape::NoBorder): {
 		m_gluePoints.clear();
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width() / 2, boundingRectangle.y()), QStringLiteral("top")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width(), boundingRectangle.y() + boundingRectangle.height() / 2),
-									  QStringLiteral("right")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width() / 2, boundingRectangle.y() + boundingRectangle.height()),
-									  QStringLiteral("bottom")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x(), boundingRectangle.y() + boundingRectangle.height() / 2), QStringLiteral("left")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width() / 2, br.y()), QStringLiteral("top")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width(), br.y() + br.height() / 2), QStringLiteral("right")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width() / 2, br.y() + br.height()), QStringLiteral("bottom")));
+		m_gluePoints.append(GluePoint(QPointF(br.x(), br.y() + br.height() / 2), QStringLiteral("left")));
 		break;
 	}
 	case (TextLabel::BorderShape::Rect): {
-		borderShapePath.addRect(boundingRectangle);
+		borderShapePath.addRect(br);
 
 		m_gluePoints.clear();
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width() / 2, boundingRectangle.y()), QStringLiteral("top")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width(), boundingRectangle.y() + boundingRectangle.height() / 2),
-									  QStringLiteral("right")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x() + boundingRectangle.width() / 2, boundingRectangle.y() + boundingRectangle.height()),
-									  QStringLiteral("bottom")));
-		m_gluePoints.append(GluePoint(QPointF(boundingRectangle.x(), boundingRectangle.y() + boundingRectangle.height() / 2), QStringLiteral("left")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width() / 2, br.y()), QStringLiteral("top")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width(), br.y() + br.height() / 2), QStringLiteral("right")));
+		m_gluePoints.append(GluePoint(QPointF(br.x() + br.width() / 2, br.y() + br.height()), QStringLiteral("bottom")));
+		m_gluePoints.append(GluePoint(QPointF(br.x(), br.y() + br.height() / 2), QStringLiteral("left")));
 		break;
 	}
 	case (TextLabel::BorderShape::Ellipse): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		const QRectF ellipseRect(xs - 0.1 * w, ys - 0.1 * h, 1.2 * w, 1.2 * h);
 		borderShapePath.addEllipse(ellipseRect);
 
@@ -735,10 +739,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::RoundSideRect): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs, ys);
 		borderShapePath.lineTo(xs + w, ys);
 		borderShapePath.quadTo(xs + w + h / 2, ys + h / 2, xs + w, ys + h);
@@ -753,10 +757,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::RoundCornerRect): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs + h * 0.2, ys);
 		borderShapePath.lineTo(xs + w - h * 0.2, ys);
 		borderShapePath.quadTo(xs + w, ys, xs + w, ys + h * 0.2);
@@ -775,10 +779,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::InwardsRoundCornerRect): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs, ys - 0.3 * h);
 		borderShapePath.lineTo(xs + w, ys - 0.3 * h);
 		borderShapePath.quadTo(xs + w, ys, xs + w + 0.3 * h, ys);
@@ -797,10 +801,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::DentedBorderRect): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs - 0.2 * h, ys - 0.2 * h);
 		borderShapePath.quadTo(xs + w / 2, ys, xs + w + 0.2 * h, ys - 0.2 * h);
 		borderShapePath.quadTo(xs + w, ys + h / 2, xs + w + 0.2 * h, ys + h + 0.2 * h);
@@ -815,10 +819,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::Cuboid): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs, ys);
 		borderShapePath.lineTo(xs + w, ys);
 		borderShapePath.lineTo(xs + w, ys + h);
@@ -839,10 +843,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::UpPointingRectangle): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs + h * 0.2, ys);
 		borderShapePath.lineTo(xs + w / 2 - 0.2 * h, ys);
 		borderShapePath.lineTo(xs + w / 2, ys - 0.2 * h);
@@ -864,10 +868,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::DownPointingRectangle): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs + h * 0.2, ys);
 		borderShapePath.lineTo(xs + w - h * 0.2, ys);
 		borderShapePath.quadTo(xs + w, ys, xs + w, ys + h * 0.2);
@@ -889,10 +893,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::LeftPointingRectangle): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs + h * 0.2, ys);
 		borderShapePath.lineTo(xs + w - h * 0.2, ys);
 		borderShapePath.quadTo(xs + w, ys, xs + w, ys + h * 0.2);
@@ -914,10 +918,10 @@ void TextLabelPrivate::updateBorder() {
 		break;
 	}
 	case (TextLabel::BorderShape::RightPointingRectangle): {
-		const double xs = boundingRectangle.x();
-		const double ys = boundingRectangle.y();
-		const double w = boundingRectangle.width();
-		const double h = boundingRectangle.height();
+		const double xs = br.x();
+		const double ys = br.y();
+		const double w = br.width();
+		const double h = br.height();
 		borderShapePath.moveTo(xs + h * 0.2, ys);
 		borderShapePath.lineTo(xs + w - h * 0.2, ys);
 		borderShapePath.quadTo(xs + w, ys, xs + w, ys + h * 0.2);
@@ -943,13 +947,6 @@ void TextLabelPrivate::updateBorder() {
 	recalcShapeAndBoundingRect();
 }
 
-/*!
-	Returns the outer bounds of the item as a rectangle.
- */
-QRectF TextLabelPrivate::boundingRect() const {
-	return transformedBoundingRectangle;
-}
-
 /*!
 	Returns the shape of this item as a QPainterPath in local coordinates.
 */
@@ -963,22 +960,12 @@ QPainterPath TextLabelPrivate::shape() const {
 void TextLabelPrivate::recalcShapeAndBoundingRect() {
 	prepareGeometryChange();
 
-	QMatrix matrix;
 	labelShape = QPainterPath();
 	if (borderShape != TextLabel::BorderShape::NoBorder) {
 		labelShape.addPath(WorksheetElement::shapeFromPath(borderShapePath, borderPen));
-		transformedBoundingRectangle = matrix.mapRect(labelShape.boundingRect());
-	} else {
-		labelShape.addRect(boundingRectangle);
-		transformedBoundingRectangle = matrix.mapRect(boundingRectangle);
-	}
-
-	labelShape = matrix.map(labelShape);
-
-	// rotate gluePoints
-	m_gluePointsTransformed.clear();
-	for (auto& gPoint : m_gluePoints)
-		m_gluePointsTransformed.append(TextLabel::GluePoint(matrix.map(gPoint.point), gPoint.name));
+		m_boundingRectangle = labelShape.boundingRect();
+	} else
+		labelShape.addRect(m_boundingRectangle);
 
 	Q_EMIT q->changed();
 }
@@ -992,8 +979,8 @@ void TextLabelPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem*
 	switch (textWrapper.mode) {
 	case TextLabel::Mode::LaTeX: {
 		painter->setRenderHint(QPainter::SmoothPixmapTransform);
-		if (boundingRectangle.width() != 0.0 && boundingRectangle.height() != 0.0)
-			painter->drawImage(boundingRectangle, teXImage);
+		if (m_boundingRectangle.width() != 0.0 && m_boundingRectangle.height() != 0.0)
+			painter->drawImage(m_boundingRectangle, teXImage);
 		break;
 	}
 	case TextLabel::Mode::Text:
@@ -1004,6 +991,11 @@ void TextLabelPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem*
 	}
 	painter->restore();
 
+	// Fill the complete path with the background color, for text mode only since for latex and markdown
+	// a rectangular image is drawn which is not cropped to the boundaries of the selected shape (eclipse, etc.)
+	if (textWrapper.mode == TextLabel::Mode::Text)
+		painter->fillPath(labelShape, QBrush(backgroundColor));
+
 	// draw the border
 	if (borderShape != TextLabel::BorderShape::NoBorder) {
 		painter->save();
@@ -1021,31 +1013,31 @@ void TextLabelPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem*
 	const bool hovered = (m_hovered && !selected);
 	if ((hovered || selected) && !q->isPrinting()) {
 		static double penWidth = 2.;
-		const QRectF& br = boundingRect();
-		const qreal width = br.width();
-		const qreal height = br.height();
-		const QRectF rect = QRectF(-width / 2 + penWidth / 2, -height / 2 + penWidth / 2, width - penWidth, height - penWidth);
-
 		if (hovered)
 			painter->setPen(QPen(QApplication::palette().color(QPalette::Shadow), penWidth));
 		else
 			painter->setPen(QPen(QApplication::palette().color(QPalette::Highlight), penWidth));
 
-		painter->drawRect(rect);
+		painter->drawPath(labelShape);
 	}
 
 #if DEBUG_TEXTLABEL_BOUNDING_RECT
 	painter->setPen(QColor(Qt::GlobalColor::red));
 	painter->drawRect(boundingRect());
+
+	painter->setPen(QColor(Qt::GlobalColor::darkGreen));
+	painter->drawRect(m_boundingRectangle.marginsAdded(QMarginsF(3, 3, 3, 3)));
+
+	painter->setPen(QColor(Qt::GlobalColor::blue));
+	painter->drawEllipse(QRectF(-5, -5, 10, 10));
 #endif
 
 #if DEBUG_TEXTLABEL_GLUEPOINTS
 	// just for debugging
 	painter->setPen(QColor(Qt::GlobalColor::red));
 	QRectF gluePointRect(0, 0, 10, 10);
-	for (int i = 0; i < m_gluePointsTransformed.length(); i++) {
-		gluePointRect.moveTo(m_gluePointsTransformed[i].point.x() - gluePointRect.width() / 2,
-							 m_gluePointsTransformed[i].point.y() - gluePointRect.height() / 2);
+	for (int i = 0; i < m_gluePoints.length(); i++) {
+		gluePointRect.moveTo(m_gluePoints[i].point.x() - gluePointRect.width() / 2, m_gluePoints[i].point.y() - gluePointRect.height() / 2);
 		painter->fillRect(gluePointRect, QColor(Qt::GlobalColor::red));
 	}
 #endif
@@ -1058,33 +1050,9 @@ void TextLabelPrivate::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 	return QGraphicsItem::mouseMoveEvent(event);
 }*/
 
-void TextLabelPrivate::contextMenuEvent(QGraphicsSceneContextMenuEvent* event) {
-	// don't show any context menu if the label is hidden which is the case
-	// for example for axis and plot title labels. For such objects the context menu
-	// of their parents, i.e. of axis and plot, is used.
-	if (!q->hidden())
-		q->createContextMenu()->exec(event->screenPos());
-}
-
-void TextLabelPrivate::hoverEnterEvent(QGraphicsSceneHoverEvent*) {
-	if (!isSelected()) {
-		m_hovered = true;
-		Q_EMIT q->hovered();
-		update();
-	}
-}
-
-void TextLabelPrivate::hoverLeaveEvent(QGraphicsSceneHoverEvent*) {
-	if (m_hovered) {
-		m_hovered = false;
-		Q_EMIT q->unhovered();
-		update();
-	}
-}
-
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 //! Save as XML
 void TextLabel::save(QXmlStreamWriter* writer) const {
 	Q_D(const TextLabel);
@@ -1137,7 +1105,6 @@ bool TextLabel::load(XmlStreamReader* reader, bool preview) {
 		return false;
 
 	Q_D(TextLabel);
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -1185,7 +1152,7 @@ bool TextLabel::load(XmlStreamReader* reader, bool preview) {
 			d->teXPdfData = QByteArray::fromBase64(content.toLatin1());
 			d->teXImage = GuiTools::imageFromPDFData(d->teXPdfData);
 		} else { // unknown element
-			reader->raiseWarning(i18n("unknown element '%1'", reader->name().toString()));
+			reader->raiseUnknownElementWarning();
 			if (!reader->skipToEndElement())
 				return false;
 		}
@@ -1194,6 +1161,19 @@ bool TextLabel::load(XmlStreamReader* reader, bool preview) {
 	if (preview)
 		return true;
 
+	// starting with XML version 10, the background color used in the html text is extracted
+	// to fill the complete shape of the label and is also saved. For older projects we need
+	// to extract it from hmtl here if available.
+	if (d->textWrapper.mode == TextLabel::Mode::Text && Project::xmlVersion() < 10) {
+		if (d->textWrapper.text.indexOf(QStringLiteral("background-color:#")) != -1) {
+			QTextEdit te;
+			te.setHtml(d->textWrapper.text);
+			te.selectAll();
+			d->backgroundColor = te.textBackgroundColor();
+		} else
+			d->backgroundColor.setAlpha(0);
+	}
+
 	// in case we use latex and the image was stored (older versions of LabPlot didn't save the image)and loaded,
 	// we just need to call updateBoundingRect() to calculate the new rect.
 	// otherwise, we set the static text and call updateBoundingRect() in updateText()
@@ -1208,21 +1188,19 @@ bool TextLabel::load(XmlStreamReader* reader, bool preview) {
 	return true;
 }
 
-//##############################################################################
-//#########################  Theme management ##################################
-//##############################################################################
+// ##############################################################################
+// #########################  Theme management ##################################
+// ##############################################################################
 void TextLabel::loadThemeConfig(const KConfig& config) {
 	DEBUG(Q_FUNC_INFO << ", label = " << STDSTRING(name()))
 	Q_D(TextLabel);
 
-	KConfigGroup group = config.group("Label");
+	KConfigGroup group = config.group(QStringLiteral("Label"));
 	// TODO: dark mode support?
-	d->fontColor = group.readEntry("FontColor", QColor(Qt::black)); // used when it's latex text
-	d->backgroundColor = group.readEntry("BackgroundColor", QColor(Qt::white)); // used when it's latex text
-
+	d->fontColor = group.readEntry(QStringLiteral("FontColor"), QColor(Qt::black)); // used when it's latex text
+	d->backgroundColor = group.readEntry(QStringLiteral("BackgroundColor"), QColor(Qt::transparent)); // used when it's latex text
 	if (d->textWrapper.mode == TextLabel::Mode::Text && !d->textWrapper.text.isEmpty()) {
-		// TODO: Replace QTextEdit by QTextDocument, because this does not contain the graphical stuff.
-		// To set the color in a html text, a QTextEdit must be used
+		// To set the color in a html text, a QTextEdit must be used, QTextDocument is not enough
 		QTextEdit te;
 		te.setHtml(d->textWrapper.text);
 		te.selectAll();
@@ -1250,17 +1228,17 @@ void TextLabel::loadThemeConfig(const KConfig& config) {
 	backgroundColorChanged(d->backgroundColor);
 	fontColorChanged(d->fontColor);
 
-	group = config.group("CartesianPlot");
+	group = config.group(QStringLiteral("CartesianPlot"));
 	QPen pen = this->borderPen();
-	pen.setColor(group.readEntry("BorderColor", pen.color()));
-	pen.setStyle((Qt::PenStyle)(group.readEntry("BorderStyle", (int)pen.style())));
-	pen.setWidthF(group.readEntry("BorderWidth", pen.widthF()));
+	pen.setColor(group.readEntry(QStringLiteral("BorderColor"), pen.color()));
+	pen.setStyle((Qt::PenStyle)(group.readEntry(QStringLiteral("BorderStyle"), (int)pen.style())));
+	pen.setWidthF(group.readEntry(QStringLiteral("BorderWidth"), pen.widthF()));
 	this->setBorderPen(pen);
-	this->setBorderOpacity(group.readEntry("BorderOpacity", this->borderOpacity()));
+	this->setBorderOpacity(group.readEntry(QStringLiteral("BorderOpacity"), this->borderOpacity()));
 }
 
 void TextLabel::saveThemeConfig(const KConfig& config) {
-	KConfigGroup group = config.group("Label");
+	KConfigGroup group = config.group(QStringLiteral("Label"));
 	// TODO
 	// 	group.writeEntry("TeXFontColor", (QColor) this->fontColor());
 }
