@@ -31,24 +31,45 @@ public:
 	bool positionInvalid{false};
 	bool coordinateBindingEnabled{false};
 	QPointF positionLogical;
-	QRectF boundingRectangle; // bounding rectangle of the text
 	bool suppressItemChangeEvent{false};
 	bool suppressRetransform{false};
+	bool suppressRecalc{false};
 	WorksheetElement* const q{nullptr};
 	bool insidePlot{true}; // point inside the plot (visible) or not
+	bool lock{false};
+	// parent plot if available
+	CartesianPlot* m_plot{nullptr};
 
 	bool swapVisible(bool on);
 	QString name() const;
 	virtual void retransform() = 0;
 	virtual void recalcShapeAndBoundingRect() = 0;
 	void updatePosition();
-	QRectF boundingRect() const override;
+	virtual QRectF boundingRect() const override;
+	virtual QPainterPath shape() const override;
 	void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget* widget = nullptr) override;
 	virtual void keyPressEvent(QKeyEvent*) override;
+	virtual void mousePressEvent(QGraphicsSceneMouseEvent*) override;
+	virtual void mouseMoveEvent(QGraphicsSceneMouseEvent*) override;
 	virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent*) override;
 	virtual QVariant itemChange(GraphicsItemChange, const QVariant& value) override;
-	QPointF mapParentToPlotArea(QPointF);
-	QPointF mapPlotAreaToParent(QPointF);
+	virtual bool sceneEvent(QEvent* event) override;
+	QPointF mapParentToPlotArea(QPointF) const;
+	QPointF mapPlotAreaToParent(QPointF) const;
+	void setHover(bool);
+	bool isHovered() const;
+
+private:
+	virtual void hoverEnterEvent(QGraphicsSceneHoverEvent*) override;
+	virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent*) override;
+	virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent*) override;
+
+protected:
+	bool m_hovered{false};
+	bool m_leftButtonPressed{false};
+	bool m_moveStarted{false};
+	QRectF m_boundingRectangle; // bounding rectangle of the element
+	QPainterPath m_shape;
 };
 
 #endif
