@@ -12,6 +12,7 @@
 #include "BoxPlotPrivate.h"
 #include "backend/core/AbstractColumn.h"
 #include "backend/core/Folder.h"
+#include "backend/core/Settings.h"
 #include "backend/core/column/Column.h"
 #include "backend/lib/XmlStreamReader.h"
 #include "backend/lib/commandtemplates.h"
@@ -23,6 +24,7 @@
 #include "backend/worksheet/plots/cartesian/CartesianCoordinateSystem.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
 #include "backend/worksheet/plots/cartesian/Symbol.h"
+#include "kdefrontend/GuiTools.h"
 #include "tools/ImageTools.h"
 
 #include <QActionGroup>
@@ -40,9 +42,9 @@
  * \brief Box Plot
  */
 
-BoxPlot::BoxPlot(const QString& name)
+BoxPlot::BoxPlot(const QString& name, bool loading)
 	: Plot(name, new BoxPlotPrivate(this), AspectType::BoxPlot) {
-	init();
+	init(loading);
 }
 
 BoxPlot::BoxPlot(const QString& name, BoxPlotPrivate* dd)
@@ -54,20 +56,11 @@ BoxPlot::BoxPlot(const QString& name, BoxPlotPrivate* dd)
 // and is deleted during the cleanup in QGraphicsScene
 BoxPlot::~BoxPlot() = default;
 
-void BoxPlot::init() {
+void BoxPlot::init(bool loading) {
 	Q_D(BoxPlot);
 
 	KConfig config;
-	KConfigGroup group = config.group("BoxPlot");
-
-	// general
-	d->ordering = (BoxPlot::Ordering)group.readEntry("Ordering", (int)BoxPlot::Ordering::None);
-	d->whiskersType = (BoxPlot::WhiskersType)group.readEntry("WhiskersType", (int)BoxPlot::WhiskersType::IQR);
-	d->whiskersRangeParameter = group.readEntry("WhiskersIQRParameter", 1.5);
-	d->orientation = (BoxPlot::Orientation)group.readEntry("Orientation", (int)BoxPlot::Orientation::Vertical);
-	d->variableWidth = group.readEntry("VariableWidth", false);
-	d->widthFactor = group.readEntry("WidthFactor", 1.0);
-	d->notchesEnabled = group.readEntry("NotchesEnabled", false);
+	KConfigGroup group = config.group(QStringLiteral("BoxPlot"));
 
 	// box
 	d->addBackground(group);
@@ -78,7 +71,6 @@ void BoxPlot::init() {
 	d->symbolMean = new Symbol(QStringLiteral("symbolMean"));
 	addChild(d->symbolMean);
 	d->symbolMean->setHidden(true);
-	d->symbolMean->init(group);
 	d->symbolMean->setStyle(Symbol::Style::Square);
 	connect(d->symbolMean, &Symbol::updateRequested, [=] {
 		d->recalcShapeAndBoundingRect();
@@ -90,7 +82,6 @@ void BoxPlot::init() {
 	d->symbolMedian = new Symbol(QStringLiteral("symbolMedian"));
 	addChild(d->symbolMedian);
 	d->symbolMedian->setHidden(true);
-	d->symbolMedian->init(group);
 	d->symbolMedian->setStyle(Symbol::Style::NoSymbols);
 	connect(d->symbolMedian, &Symbol::updateRequested, [=] {
 		d->recalcShapeAndBoundingRect();
@@ -102,7 +93,6 @@ void BoxPlot::init() {
 	d->symbolOutlier = new Symbol(QStringLiteral("symbolOutlier"));
 	addChild(d->symbolOutlier);
 	d->symbolOutlier->setHidden(true);
-	d->symbolOutlier->init(group);
 	connect(d->symbolOutlier, &Symbol::updateRequested, [=] {
 		d->recalcShapeAndBoundingRect();
 	});
@@ -113,7 +103,6 @@ void BoxPlot::init() {
 	d->symbolFarOut = new Symbol(QStringLiteral("symbolFarOut"));
 	addChild(d->symbolFarOut);
 	d->symbolFarOut->setHidden(true);
-	d->symbolFarOut->init(group);
 	d->symbolFarOut->setStyle(Symbol::Style::Plus);
 	connect(d->symbolFarOut, &Symbol::updateRequested, [=] {
 		d->recalcShapeAndBoundingRect();
@@ -125,7 +114,6 @@ void BoxPlot::init() {
 	d->symbolData = new Symbol(QStringLiteral("symbolData"));
 	addChild(d->symbolData);
 	d->symbolData->setHidden(true);
-	d->symbolData->init(group);
 	d->symbolData->setStyle(Symbol::Style::NoSymbols);
 	d->symbolData->setOpacity(0.5);
 	connect(d->symbolData, &Symbol::updateRequested, [=] {
@@ -138,7 +126,6 @@ void BoxPlot::init() {
 	d->symbolWhiskerEnd = new Symbol(QStringLiteral("symbolWhiskerEnd"));
 	addChild(d->symbolWhiskerEnd);
 	d->symbolWhiskerEnd->setHidden(true);
-	d->symbolWhiskerEnd->init(group);
 	d->symbolWhiskerEnd->setStyle(Symbol::Style::NoSymbols);
 	connect(d->symbolWhiskerEnd, &Symbol::updateRequested, [=] {
 		d->recalcShapeAndBoundingRect();
@@ -147,15 +134,13 @@ void BoxPlot::init() {
 		d->updatePixmap();
 	});
 
-	d->jitteringEnabled = group.readEntry("JitteringEnabled", true);
-
 	// whiskers
 	d->whiskersLine = new Line(QString());
-	d->whiskersLine->setPrefix(QLatin1String("Whiskers"));
+	d->whiskersLine->setPrefix(QStringLiteral("Whiskers"));
 	d->whiskersLine->setCreateXmlElement(false); // whiskers element is created in BoxPlot::save()
 	d->whiskersLine->setHidden(true);
 	addChild(d->whiskersLine);
-	d->whiskersLine->init(group);
+
 	connect(d->whiskersLine, &Line::updatePixmapRequested, [=] {
 		d->updatePixmap();
 	});
@@ -163,13 +148,11 @@ void BoxPlot::init() {
 		d->recalcShapeAndBoundingRect();
 	});
 
-	d->whiskersCapSize = group.readEntry("WhiskersCapSize", Worksheet::convertToSceneUnits(5.0, Worksheet::Unit::Point));
 	d->whiskersCapLine = new Line(QString());
-	d->whiskersCapLine->setPrefix(QLatin1String("WhiskersCap"));
+	d->whiskersCapLine->setPrefix(QStringLiteral("WhiskersCap"));
 	d->whiskersCapLine->setCreateXmlElement(false); // whiskers cap element is created in BoxPlot::save()
 	d->whiskersCapLine->setHidden(true);
 	addChild(d->whiskersCapLine);
-	d->whiskersCapLine->init(group);
 	connect(d->whiskersCapLine, &Line::updatePixmapRequested, [=] {
 		d->updatePixmap();
 	});
@@ -177,18 +160,44 @@ void BoxPlot::init() {
 		d->recalcShapeAndBoundingRect();
 	});
 
+	if (loading)
+		return;
+
+	// init the properties
+	// general
+	d->ordering = (BoxPlot::Ordering)group.readEntry(QStringLiteral("Ordering"), (int)BoxPlot::Ordering::None);
+	d->whiskersType = (BoxPlot::WhiskersType)group.readEntry(QStringLiteral("WhiskersType"), (int)BoxPlot::WhiskersType::IQR);
+	d->whiskersRangeParameter = group.readEntry(QStringLiteral("WhiskersIQRParameter"), 1.5);
+	d->orientation = (BoxPlot::Orientation)group.readEntry(QStringLiteral("Orientation"), (int)BoxPlot::Orientation::Vertical);
+	d->variableWidth = group.readEntry(QStringLiteral("VariableWidth"), false);
+	d->widthFactor = group.readEntry(QStringLiteral("WidthFactor"), 1.0);
+	d->notchesEnabled = group.readEntry(QStringLiteral("NotchesEnabled"), false);
+
+	// symbols
+	d->symbolMean->init(group);
+	d->symbolMedian->init(group);
+	d->symbolOutlier->init(group);
+	d->symbolFarOut->init(group);
+	d->symbolData->init(group);
+	d->symbolWhiskerEnd->init(group);
+	d->jitteringEnabled = group.readEntry(QStringLiteral("JitteringEnabled"), true);
+
+	// whiskers
+	d->whiskersCapSize = group.readEntry(QStringLiteral("WhiskersCapSize"), Worksheet::convertToSceneUnits(5.0, Worksheet::Unit::Point));
+	d->whiskersLine->init(group);
+	d->whiskersCapLine->init(group);
+
 	// marginal plots (rug, BoxPlot, boxplot)
-	d->rugEnabled = group.readEntry("RugEnabled", false);
-	d->rugLength = group.readEntry("RugLength", Worksheet::convertToSceneUnits(5, Worksheet::Unit::Point));
-	d->rugWidth = group.readEntry("RugWidth", 0.0);
-	d->rugOffset = group.readEntry("RugOffset", 0.0);
+	d->rugEnabled = group.readEntry(QStringLiteral("RugEnabled"), false);
+	d->rugLength = group.readEntry(QStringLiteral("RugLength"), Worksheet::convertToSceneUnits(5, Worksheet::Unit::Point));
+	d->rugWidth = group.readEntry(QStringLiteral("RugWidth"), 0.0);
+	d->rugOffset = group.readEntry(QStringLiteral("RugOffset"), 0.0);
 }
 
 /*!
 	Returns an icon to be used in the project explorer.
 */
 QIcon BoxPlot::icon() const {
-	// 	return QIcon::fromTheme(QLatin1String("draw-line"));
 	return BoxPlot::staticIcon();
 }
 
@@ -199,7 +208,7 @@ QIcon BoxPlot::staticIcon() {
 	QPixmap pm(iconSize, iconSize);
 
 	QPen pen(Qt::SolidLine);
-	pen.setColor(DARKMODE ? Qt::white : Qt::black);
+	pen.setColor(GuiTools::isDarkMode() ? Qt::white : Qt::black);
 	pen.setWidthF(0.0);
 
 	pm.fill(Qt::transparent);
@@ -214,19 +223,15 @@ QIcon BoxPlot::staticIcon() {
 }
 
 void BoxPlot::initActions() {
-	visibilityAction = new QAction(QIcon::fromTheme(QStringLiteral("view-visible")), i18n("Visible"), this);
-	visibilityAction->setCheckable(true);
-	connect(visibilityAction, &QAction::triggered, this, &BoxPlot::visibilityChangedSlot);
-
 	// Orientation
 	auto* orientationActionGroup = new QActionGroup(this);
 	orientationActionGroup->setExclusive(true);
 	connect(orientationActionGroup, &QActionGroup::triggered, this, &BoxPlot::orientationChangedSlot);
 
-	orientationHorizontalAction = new QAction(QIcon::fromTheme(QLatin1String("transform-move-horizontal")), i18n("Horizontal"), orientationActionGroup);
+	orientationHorizontalAction = new QAction(QIcon::fromTheme(QStringLiteral("transform-move-horizontal")), i18n("Horizontal"), orientationActionGroup);
 	orientationHorizontalAction->setCheckable(true);
 
-	orientationVerticalAction = new QAction(QIcon::fromTheme(QLatin1String("transform-move-vertical")), i18n("Vertical"), orientationActionGroup);
+	orientationVerticalAction = new QAction(QIcon::fromTheme(QStringLiteral("transform-move-vertical")), i18n("Vertical"), orientationActionGroup);
 	orientationVerticalAction->setCheckable(true);
 }
 
@@ -235,7 +240,7 @@ void BoxPlot::initMenus() {
 
 	// Orientation
 	orientationMenu = new QMenu(i18n("Orientation"));
-	orientationMenu->setIcon(QIcon::fromTheme(QLatin1String("draw-cross")));
+	orientationMenu->setIcon(QIcon::fromTheme(QStringLiteral("draw-cross")));
 	orientationMenu->addAction(orientationHorizontalAction);
 	orientationMenu->addAction(orientationVerticalAction);
 }
@@ -245,11 +250,7 @@ QMenu* BoxPlot::createContextMenu() {
 		initMenus();
 
 	QMenu* menu = WorksheetElement::createContextMenu();
-	QAction* firstAction = menu->actions().at(1); // skip the first action because of the "title-action"
-
-	// Visibility
-	visibilityAction->setChecked(isVisible());
-	menu->insertAction(firstAction, visibilityAction);
+	QAction* visibilityAction = this->visibilityAction();
 
 	// Orientation
 	Q_D(const BoxPlot);
@@ -257,16 +258,12 @@ QMenu* BoxPlot::createContextMenu() {
 		orientationHorizontalAction->setChecked(true);
 	else
 		orientationVerticalAction->setChecked(true);
-	menu->insertMenu(firstAction, orientationMenu);
-	menu->insertSeparator(firstAction);
+	menu->insertMenu(visibilityAction, orientationMenu);
+	menu->insertSeparator(visibilityAction);
 
 	return menu;
 }
 
-QGraphicsItem* BoxPlot::graphicsItem() const {
-	return d_ptr;
-}
-
 void BoxPlot::retransform() {
 	Q_D(BoxPlot);
 	d->retransform();
@@ -280,16 +277,6 @@ void BoxPlot::recalc() {
 void BoxPlot::handleResize(double /*horizontalRatio*/, double /*verticalRatio*/, bool /*pageResize*/) {
 }
 
-bool BoxPlot::activatePlot(QPointF mouseScenePos, double maxDist) {
-	Q_D(BoxPlot);
-	return d->activatePlot(mouseScenePos, maxDist);
-}
-
-void BoxPlot::setHover(bool on) {
-	Q_D(BoxPlot);
-	d->setHover(on);
-}
-
 /*!
  * creates a new spreadsheet having the data with the positions and the values of the bins.
  * the new spreadsheet is added to the current folder.
@@ -448,6 +435,53 @@ double BoxPlot::maximum(const Dimension dim) const {
 	return NAN;
 }
 
+bool BoxPlot::hasData() const {
+	Q_D(const BoxPlot);
+	return !d->dataColumns.isEmpty();
+}
+
+bool BoxPlot::usingColumn(const Column* column) const {
+	Q_D(const BoxPlot);
+
+	for (auto* c : d->dataColumns) {
+		if (c == column)
+			return true;
+	}
+
+	return false;
+}
+
+void BoxPlot::handleAspectUpdated(const QString& aspectPath, const AbstractAspect* aspect) {
+	Q_D(const BoxPlot);
+	const auto column = dynamic_cast<const AbstractColumn*>(aspect);
+	if (!column)
+		return;
+
+	const auto dataColumnPaths = d->dataColumnPaths;
+	auto dataColumns = d->dataColumns;
+	bool changed = false;
+
+	for (int i = 0; i < dataColumnPaths.count(); ++i) {
+		const auto& path = dataColumnPaths.at(i);
+
+		if (path == aspectPath) {
+			dataColumns[i] = column;
+			changed = true;
+		}
+	}
+
+	if (changed) {
+		setUndoAware(false);
+		setDataColumns(dataColumns);
+		setUndoAware(true);
+	}
+}
+
+QColor BoxPlot::color() const {
+	// Q_D(const BoxPlot);
+	return QColor();
+}
+
 /* ============================ setter methods and undo commands ================= */
 
 // General
@@ -462,11 +496,11 @@ void BoxPlot::setDataColumns(const QVector<const AbstractColumn*> columns) {
 				continue;
 
 			connect(column, &AbstractColumn::dataChanged, this, &BoxPlot::dataChanged);
-			connect(column, &AbstractAspect::aspectDescriptionChanged, this, &Plot::updateLegendRequested);
 
 			// update the curve itself on changes
 			connect(column, &AbstractColumn::dataChanged, this, &BoxPlot::recalc);
-			connect(column->parentAspect(), &AbstractAspect::aspectAboutToBeRemoved, this, &BoxPlot::dataColumnAboutToBeRemoved);
+			connect(column, &AbstractAspect::aspectDescriptionChanged, this, &Plot::appearanceChanged);
+			connect(column->parentAspect(), &AbstractAspect::childAspectAboutToBeRemoved, this, &BoxPlot::dataColumnAboutToBeRemoved);
 			// TODO: add disconnect in the undo-function
 		}
 	}
@@ -566,9 +600,9 @@ void BoxPlot::setRugOffset(double offset) {
 		exec(new BoxPlotSetRugOffsetCmd(d, offset, ki18n("%1: change rug offset")));
 }
 
-//##############################################################################
-//#################################  SLOTS  ####################################
-//##############################################################################
+// ##############################################################################
+// #################################  SLOTS  ####################################
+// ##############################################################################
 
 void BoxPlot::dataColumnAboutToBeRemoved(const AbstractAspect* aspect) {
 	Q_D(BoxPlot);
@@ -576,14 +610,16 @@ void BoxPlot::dataColumnAboutToBeRemoved(const AbstractAspect* aspect) {
 		if (aspect == d->dataColumns.at(i)) {
 			d->dataColumns[i] = nullptr;
 			d->retransform();
+			Q_EMIT dataChanged();
+			Q_EMIT changed();
 			break;
 		}
 	}
 }
 
-//##############################################################################
-//######  SLOTs for changes triggered via QActions in the context menu  ########
-//##############################################################################
+// ##############################################################################
+// ######  SLOTs for changes triggered via QActions in the context menu  ########
+// ##############################################################################
 void BoxPlot::orientationChangedSlot(QAction* action) {
 	if (action == orientationHorizontalAction)
 		this->setOrientation(Axis::Orientation::Horizontal);
@@ -591,14 +627,9 @@ void BoxPlot::orientationChangedSlot(QAction* action) {
 		this->setOrientation(Axis::Orientation::Vertical);
 }
 
-void BoxPlot::visibilityChangedSlot() {
-	Q_D(const BoxPlot);
-	this->setVisible(!d->isVisible());
-}
-
-//##############################################################################
-//####################### Private implementation ###############################
-//##############################################################################
+// ##############################################################################
+// ####################### Private implementation ###############################
+// ##############################################################################
 BoxPlotPrivate::BoxPlotPrivate(BoxPlot* owner)
 	: PlotPrivate(owner)
 	, q(owner) {
@@ -606,24 +637,8 @@ BoxPlotPrivate::BoxPlotPrivate(BoxPlot* owner)
 	setAcceptHoverEvents(false);
 }
 
-bool BoxPlotPrivate::activatePlot(QPointF mouseScenePos, double /*maxDist*/) {
-	if (!isVisible())
-		return false;
-
-	return shape().contains(mouseScenePos);
-}
-
-void BoxPlotPrivate::setHover(bool on) {
-	if (on == m_hovered)
-		return; // don't update if state not changed
-
-	m_hovered = on;
-	on ? Q_EMIT q->hovered() : emit q->unhovered();
-	update();
-}
-
 Background* BoxPlotPrivate::addBackground(const KConfigGroup& group) {
-	auto* background = new Background(QString());
+	auto* background = new Background(QStringLiteral("background"));
 	background->setPrefix(QLatin1String("Filling"));
 	background->setEnabledAvailable(true);
 	background->setHidden(true);
@@ -643,7 +658,7 @@ Background* BoxPlotPrivate::addBackground(const KConfigGroup& group) {
 }
 
 Line* BoxPlotPrivate::addBorderLine(const KConfigGroup& group) {
-	auto* line = new Line(QString());
+	auto* line = new Line(QStringLiteral("line"));
 	line->setPrefix(QLatin1String("Border"));
 	line->setHidden(true);
 	q->addChild(line);
@@ -666,7 +681,7 @@ Line* BoxPlotPrivate::addBorderLine(const KConfigGroup& group) {
 }
 
 Line* BoxPlotPrivate::addMedianLine(const KConfigGroup& group) {
-	auto* line = new Line(QString());
+	auto* line = new Line(QStringLiteral("medianLine"));
 	line->setPrefix(QLatin1String("MedianLine"));
 	line->setHidden(true);
 	q->addChild(line);
@@ -749,7 +764,9 @@ void BoxPlotPrivate::adjustPropertiesContainers() {
   triggers the update of lines, drop lines, symbols etc.
 */
 void BoxPlotPrivate::retransform() {
-	if (suppressRetransform || !isVisible() || q->isLoading())
+	const bool suppressed = suppressRetransform || !isVisible() || q->isLoading();
+	Q_EMIT trackRetransformCalled(suppressed);
+	if (suppressed)
 		return;
 
 	PERFTRACE(name() + QLatin1String(Q_FUNC_INFO));
@@ -836,6 +853,11 @@ void BoxPlotPrivate::recalc() {
 		// TODO:
 	}
 
+	const double xMinOld = xMin;
+	const double xMaxOld = xMax;
+	const double yMinOld = yMin;
+	const double yMaxOld = yMax;
+
 	// calculate the new min and max values of the box plot
 	// for the current sizes of the box and of the whiskers
 	if (orientation == BoxPlot::Orientation::Vertical) {
@@ -892,11 +914,15 @@ void BoxPlotPrivate::recalc() {
 	for (int i = 0; i < count; ++i)
 		recalc(i);
 
-	// the size of the boxplot changed because of the actual
-	// data changes or because of new boxplot settings.
-	// Q_EMIT dataChanged() in order to recalculate everything
-	// in the parent plot with the new size/shape of the boxplot
-	Q_EMIT q->dataChanged();
+	// if the size of the plot has changed because of the actual
+	// data changes or because of new plot settings, emit dataChanged()
+	// in order to recalculate the data ranges in the parent plot area
+	// and to retransform all its children.
+	// Just call retransform() to update the plot only if the ranges didn't change.
+	if (xMin != xMinOld || xMax != xMaxOld || yMin != yMinOld || yMax != yMaxOld)
+		Q_EMIT q->dataChanged();
+	else
+		retransform();
 }
 
 QPointF BoxPlotPrivate::setOutlierPoint(double pos, double value) {
@@ -1360,7 +1386,7 @@ void BoxPlotPrivate::updateFillingRect(int index, const QVector<QLineF>& lines)
 		++i;
 	}
 
-	m_fillPolygon[index] = polygon;
+	m_fillPolygon[index] = std::move(polygon);
 }
 
 /*!
@@ -1423,26 +1449,12 @@ void BoxPlotPrivate::mapSymbolsToScene(int index) {
 	m_medianPointVisible[index] = visible;
 }
 
-/*!
-	Returns the outer bounds of the item as a rectangle.
- */
-QRectF BoxPlotPrivate::boundingRect() const {
-	return m_boundingRectangle;
-}
-
-/*!
-	Returns the shape of this item as a QPainterPath in local coordinates.
-*/
-QPainterPath BoxPlotPrivate::shape() const {
-	return m_boxPlotShape;
-}
-
 /*!
   recalculates the outer bounds and the shape of the item.
 */
 void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 	prepareGeometryChange();
-	m_boxPlotShape = QPainterPath();
+	m_shape = QPainterPath();
 
 	for (int i = 0; i < dataColumnsOrdered.size(); ++i) {
 		if (!dataColumnsOrdered.at(i) || static_cast<const Column*>(dataColumnsOrdered.at(i))->statistics().size == 0)
@@ -1453,19 +1465,19 @@ void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 			boxPath.moveTo(line.p1());
 			boxPath.lineTo(line.p2());
 		}
-		m_boxPlotShape.addPath(WorksheetElement::shapeFromPath(boxPath, borderLines.at(i)->pen()));
+		m_shape.addPath(WorksheetElement::shapeFromPath(boxPath, borderLines.at(i)->pen()));
 
-		m_boxPlotShape.addPath(WorksheetElement::shapeFromPath(m_whiskersPath.at(i), whiskersLine->pen()));
-		m_boxPlotShape.addPath(WorksheetElement::shapeFromPath(m_whiskersCapPath.at(i), whiskersCapLine->pen()));
+		m_shape.addPath(WorksheetElement::shapeFromPath(m_whiskersPath.at(i), whiskersLine->pen()));
+		m_shape.addPath(WorksheetElement::shapeFromPath(m_whiskersCapPath.at(i), whiskersCapLine->pen()));
 
-		m_boxPlotShape.addPath(WorksheetElement::shapeFromPath(m_rugPath.at(i), borderLines.at(i)->pen()));
+		m_shape.addPath(WorksheetElement::shapeFromPath(m_rugPath.at(i), borderLines.at(i)->pen()));
 
 		// add symbols outlier, jitter and far out values
 		QPainterPath symbolsPath = QPainterPath();
 
 		// outlier values
 		if (symbolOutlier->style() != Symbol::Style::NoSymbols && !m_outlierPoints.at(i).isEmpty()) {
-			QPainterPath path = Symbol::stylePath(symbolOutlier->style());
+			auto path = WorksheetElement::shapeFromPath(Symbol::stylePath(symbolOutlier->style()), symbolOutlier->pen());
 			QTransform trafo;
 			trafo.scale(symbolOutlier->size(), symbolOutlier->size());
 			path = trafo.map(path);
@@ -1485,7 +1497,7 @@ void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 
 		// jitter values
 		if (symbolData->style() != Symbol::Style::NoSymbols && !m_dataPoints.at(i).isEmpty()) {
-			QPainterPath path = Symbol::stylePath(symbolData->style());
+			auto path = WorksheetElement::shapeFromPath(Symbol::stylePath(symbolData->style()), symbolData->pen());
 			QTransform trafo;
 			trafo.scale(symbolData->size(), symbolData->size());
 			path = trafo.map(path);
@@ -1505,7 +1517,7 @@ void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 
 		// far out values
 		if (symbolFarOut->style() != Symbol::Style::NoSymbols && !m_farOutPoints.at(i).isEmpty()) {
-			QPainterPath path = Symbol::stylePath(symbolFarOut->style());
+			auto path = WorksheetElement::shapeFromPath(Symbol::stylePath(symbolFarOut->style()), symbolFarOut->pen());
 			QTransform trafo;
 			trafo.scale(symbolFarOut->size(), symbolFarOut->size());
 			path = trafo.map(path);
@@ -1525,7 +1537,7 @@ void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 
 		// whisker ends
 		if (symbolWhiskerEnd->style() != Symbol::Style::NoSymbols && !m_whiskerEndPoints.at(i).isEmpty()) {
-			QPainterPath path = Symbol::stylePath(symbolWhiskerEnd->style());
+			auto path = WorksheetElement::shapeFromPath(Symbol::stylePath(symbolWhiskerEnd->style()), symbolWhiskerEnd->pen());
 			QTransform trafo;
 			trafo.scale(symbolWhiskerEnd->size(), symbolWhiskerEnd->size());
 			path = trafo.map(path);
@@ -1543,33 +1555,32 @@ void BoxPlotPrivate::recalcShapeAndBoundingRect() {
 			}
 		}
 
-		m_boxPlotShape.addPath(symbolsPath);
+		m_shape.addPath(symbolsPath);
 	}
 
-	m_boundingRectangle = m_boxPlotShape.boundingRect();
+	m_boundingRectangle = m_shape.boundingRect();
 	updatePixmap();
 }
 
 void BoxPlotPrivate::updatePixmap() {
 	PERFTRACE(name() + QLatin1String(Q_FUNC_INFO));
-	QPixmap pixmap(m_boundingRectangle.width(), m_boundingRectangle.height());
+	m_pixmap = QPixmap(m_boundingRectangle.width(), m_boundingRectangle.height());
 	if (m_boundingRectangle.width() == 0. || m_boundingRectangle.height() == 0.) {
-		m_pixmap = pixmap;
 		m_hoverEffectImageIsDirty = true;
 		m_selectionEffectImageIsDirty = true;
 		return;
 	}
-	pixmap.fill(Qt::transparent);
-	QPainter painter(&pixmap);
+	m_pixmap.fill(Qt::transparent);
+	QPainter painter(&m_pixmap);
 	painter.setRenderHint(QPainter::Antialiasing, true);
 	painter.translate(-m_boundingRectangle.topLeft());
 
 	draw(&painter);
 	painter.end();
 
-	m_pixmap = pixmap;
 	m_hoverEffectImageIsDirty = true;
 	m_selectionEffectImageIsDirty = true;
+	Q_EMIT q->changed();
 	update();
 }
 
@@ -1586,11 +1597,9 @@ void BoxPlotPrivate::draw(QPainter* painter) {
 
 		if (!m_boxRect.at(i).isEmpty()) {
 			// draw the box filling
-			if (backgrounds.at(i)->enabled()) {
-				painter->setOpacity(backgrounds.at(i)->opacity());
-				painter->setPen(Qt::NoPen);
-				drawFilling(painter, i);
-			}
+			const auto* background = backgrounds.at(i);
+			if (background->enabled())
+				background->draw(painter, m_fillPolygon.at(i));
 
 			// draw the border
 			const auto* borderLine = borderLines.at(i);
@@ -1665,98 +1674,6 @@ void BoxPlotPrivate::drawSymbols(QPainter* painter, int index) {
 	symbolWhiskerEnd->draw(painter, m_whiskerEndPoints.at(index));
 }
 
-void BoxPlotPrivate::drawFilling(QPainter* painter, int index) {
-	PERFTRACE(name() + QLatin1String(Q_FUNC_INFO));
-
-	const QPolygonF& polygon = m_fillPolygon.at(index);
-	const QRectF& rect = polygon.boundingRect();
-
-	const auto* background = backgrounds.at(index);
-	if (background->type() == Background::Type::Color) {
-		switch (background->colorStyle()) {
-		case Background::ColorStyle::SingleColor: {
-			painter->setBrush(QBrush(background->firstColor()));
-			break;
-		}
-		case Background::ColorStyle::HorizontalLinearGradient: {
-			QLinearGradient linearGrad(rect.topLeft(), rect.topRight());
-			linearGrad.setColorAt(0, background->firstColor());
-			linearGrad.setColorAt(1, background->secondColor());
-			painter->setBrush(QBrush(linearGrad));
-			break;
-		}
-		case Background::ColorStyle::VerticalLinearGradient: {
-			QLinearGradient linearGrad(rect.topLeft(), rect.bottomLeft());
-			linearGrad.setColorAt(0, background->firstColor());
-			linearGrad.setColorAt(1, background->secondColor());
-			painter->setBrush(QBrush(linearGrad));
-			break;
-		}
-		case Background::ColorStyle::TopLeftDiagonalLinearGradient: {
-			QLinearGradient linearGrad(rect.topLeft(), rect.bottomRight());
-			linearGrad.setColorAt(0, background->firstColor());
-			linearGrad.setColorAt(1, background->secondColor());
-			painter->setBrush(QBrush(linearGrad));
-			break;
-		}
-		case Background::ColorStyle::BottomLeftDiagonalLinearGradient: {
-			QLinearGradient linearGrad(rect.bottomLeft(), rect.topRight());
-			linearGrad.setColorAt(0, background->firstColor());
-			linearGrad.setColorAt(1, background->secondColor());
-			painter->setBrush(QBrush(linearGrad));
-			break;
-		}
-		case Background::ColorStyle::RadialGradient: {
-			QRadialGradient radialGrad(rect.center(), rect.width() / 2);
-			radialGrad.setColorAt(0, background->firstColor());
-			radialGrad.setColorAt(1, background->secondColor());
-			painter->setBrush(QBrush(radialGrad));
-			break;
-		}
-		}
-	} else if (background->type() == Background::Type::Image) {
-		if (!background->fileName().trimmed().isEmpty()) {
-			QPixmap pix(background->fileName());
-			switch (background->imageStyle()) {
-			case Background::ImageStyle::ScaledCropped:
-				pix = pix.scaled(rect.size().toSize(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
-				painter->setBrush(QBrush(pix));
-				painter->setBrushOrigin(pix.size().width() / 2, pix.size().height() / 2);
-				break;
-			case Background::ImageStyle::Scaled:
-				pix = pix.scaled(rect.size().toSize(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-				painter->setBrush(QBrush(pix));
-				painter->setBrushOrigin(pix.size().width() / 2, pix.size().height() / 2);
-				break;
-			case Background::ImageStyle::ScaledAspectRatio:
-				pix = pix.scaled(rect.size().toSize(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
-				painter->setBrush(QBrush(pix));
-				painter->setBrushOrigin(pix.size().width() / 2, pix.size().height() / 2);
-				break;
-			case Background::ImageStyle::Centered: {
-				QPixmap backpix(rect.size().toSize());
-				backpix.fill();
-				QPainter p(&backpix);
-				p.drawPixmap(QPointF(0, 0), pix);
-				p.end();
-				painter->setBrush(QBrush(backpix));
-				painter->setBrushOrigin(-pix.size().width() / 2, -pix.size().height() / 2);
-				break;
-			}
-			case Background::ImageStyle::Tiled:
-				painter->setBrush(QBrush(pix));
-				break;
-			case Background::ImageStyle::CenterTiled:
-				painter->setBrush(QBrush(pix));
-				painter->setBrushOrigin(pix.size().width() / 2, pix.size().height() / 2);
-			}
-		}
-	} else if (background->type() == Background::Type::Pattern)
-		painter->setBrush(QBrush(background->firstColor(), background->brushStyle()));
-
-	painter->drawPolygon(polygon);
-}
-
 void BoxPlotPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget*) {
 	if (!isVisible())
 		return;
@@ -1765,7 +1682,7 @@ void BoxPlotPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*
 	painter->setBrush(Qt::NoBrush);
 	painter->setRenderHint(QPainter::SmoothPixmapTransform, true);
 
-	if (KSharedConfig::openConfig()->group("Settings_Worksheet").readEntry<bool>("DoubleBuffering", true))
+	if (!q->isPrinting() && Settings::group(QStringLiteral("Settings_Worksheet")).readEntry<bool>("DoubleBuffering", true))
 		painter->drawPixmap(m_boundingRectangle.topLeft(), m_pixmap); // draw the cached pixmap (fast)
 	else
 		draw(painter); // draw directly again (slow)
@@ -1803,29 +1720,9 @@ void BoxPlotPrivate::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*
 	}
 }
 
-void BoxPlotPrivate::contextMenuEvent(QGraphicsSceneContextMenuEvent* event) {
-	q->createContextMenu()->exec(event->screenPos());
-}
-
-void BoxPlotPrivate::hoverEnterEvent(QGraphicsSceneHoverEvent*) {
-	if (!isSelected()) {
-		m_hovered = true;
-		Q_EMIT q->hovered();
-		update();
-	}
-}
-
-void BoxPlotPrivate::hoverLeaveEvent(QGraphicsSceneHoverEvent*) {
-	if (m_hovered) {
-		m_hovered = false;
-		Q_EMIT q->unhovered();
-		update();
-	}
-}
-
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 //! Save as XML
 void BoxPlot::save(QXmlStreamWriter* writer) const {
 	Q_D(const BoxPlot);
@@ -1847,6 +1744,7 @@ void BoxPlot::save(QXmlStreamWriter* writer) const {
 	writer->writeAttribute(QStringLiteral("xMax"), QString::number(d->xMax));
 	writer->writeAttribute(QStringLiteral("yMin"), QString::number(d->yMin));
 	writer->writeAttribute(QStringLiteral("yMax"), QString::number(d->yMax));
+	writer->writeAttribute(QStringLiteral("legendVisible"), QString::number(d->legendVisible));
 	writer->writeAttribute(QStringLiteral("visible"), QString::number(d->isVisible()));
 	for (auto* column : d->dataColumns) {
 		writer->writeStartElement(QStringLiteral("column"));
@@ -1903,7 +1801,6 @@ bool BoxPlot::load(XmlStreamReader* reader, bool preview) {
 	if (!readBasicAttributes(reader))
 		return false;
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs;
 	QString str;
 	bool firstBackgroundRead = false;
@@ -1936,10 +1833,11 @@ bool BoxPlot::load(XmlStreamReader* reader, bool preview) {
 			READ_DOUBLE_VALUE("xMax", xMax);
 			READ_DOUBLE_VALUE("yMin", yMin);
 			READ_DOUBLE_VALUE("yMax", yMax);
+			READ_INT_VALUE("legendVisible", legendVisible, bool);
 
 			str = attribs.value(QStringLiteral("visible")).toString();
 			if (str.isEmpty())
-				reader->raiseWarning(attributeWarning.subs(QStringLiteral("visible")).toString());
+				reader->raiseMissingAttributeWarning(QStringLiteral("visible"));
 			else
 				d->setVisible(str.toInt());
 		} else if (reader->name() == QLatin1String("column")) {
@@ -2022,7 +1920,7 @@ bool BoxPlot::load(XmlStreamReader* reader, bool preview) {
 	int diff = d->dataColumns.size() - d->backgrounds.size();
 	if (diff > 0) {
 		KConfig config;
-		KConfigGroup group = config.group(QLatin1String("XYCurve"));
+		KConfigGroup group = config.group(QStringLiteral("XYCurve"));
 
 		const auto* background = d->backgrounds.constFirst();
 		const auto* borderLine = d->borderLines.constFirst();
@@ -2056,39 +1954,37 @@ bool BoxPlot::load(XmlStreamReader* reader, bool preview) {
 	return true;
 }
 
-//##############################################################################
-//#########################  Theme management ##################################
-//##############################################################################
+// ##############################################################################
+// #########################  Theme management ##################################
+// ##############################################################################
 void BoxPlot::loadThemeConfig(const KConfig& config) {
 	KConfigGroup group;
-	if (config.hasGroup(QLatin1String("Theme")))
-		group = config.group("XYCurve"); // when loading from the theme config, use the same properties as for XYCurve
+	if (config.hasGroup(QStringLiteral("Theme")))
+		group = config.group(QStringLiteral("XYCurve")); // when loading from the theme config, use the same properties as for XYCurve
 	else
-		group = config.group("BoxPlot");
+		group = config.group(QStringLiteral("BoxPlot"));
 
 	const auto* plot = static_cast<const CartesianPlot*>(parentAspect());
 	int index = plot->curveChildIndex(this);
 	const QColor themeColor = plot->themeColorPalette(index);
 
 	Q_D(BoxPlot);
-	d->m_suppressRecalc = true;
+	d->suppressRecalc = true;
 
-	// box fillings
-	for (int i = 0; i < d->backgrounds.count(); ++i) {
+	for (int i = 0; i < d->dataColumns.count(); ++i) {
+		const auto& color = plot->themeColorPalette(i);
+
+		// box fillings
 		auto* background = d->backgrounds.at(i);
-		background->loadThemeConfig(group, plot->themeColorPalette(i));
-	}
+		background->loadThemeConfig(group, color);
 
-	// box border lines
-	for (int i = 0; i < d->borderLines.count(); ++i) {
+		// box border lines
 		auto* line = d->borderLines.at(i);
-		line->loadThemeConfig(group, plot->themeColorPalette(i));
-	}
+		line->loadThemeConfig(group, color);
 
-	// median lines
-	for (int i = 0; i < d->medianLines.count(); ++i) {
-		auto* line = d->medianLines.at(i);
-		line->loadThemeConfig(group, plot->themeColorPalette(i));
+		// median lines
+		line = d->medianLines.at(i);
+		line->loadThemeConfig(group, color);
 	}
 
 	// whiskers
@@ -2123,6 +2019,6 @@ void BoxPlot::loadThemeConfig(const KConfig& config) {
 		setWhiskersCapSize(0.0);
 	}
 
-	d->m_suppressRecalc = false;
+	d->suppressRecalc = false;
 	d->recalcShapeAndBoundingRect();
 }
