@@ -13,9 +13,7 @@
 #include "backend/worksheet/plots/cartesian/CartesianCoordinateSystemPrivate.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
 
-extern "C" {
 #include "backend/nsl/nsl_math.h"
-}
 
 using Dimension = CartesianCoordinateSystem::Dimension;
 
@@ -56,9 +54,9 @@ QString CartesianCoordinateSystem::info() const {
 	return i18n("no info available");
 }
 
-//##############################################################################
-//######################### logical to scene mappers ###########################
-//##############################################################################
+// ##############################################################################
+// ######################### logical to scene mappers ###########################
+// ##############################################################################
 Points CartesianCoordinateSystem::mapLogicalToScene(const Points& points, MappingFlags flags) const {
 	// DEBUG(Q_FUNC_INFO << ", (points with flags)")
 	const QRectF pageRect = d->plot->dataRect();
@@ -313,7 +311,7 @@ Lines CartesianCoordinateSystem::mapLogicalToScene(const Lines& lines, MappingFl
 	double yGapBefore;
 	double yGapAfter = NAN;
 
-	DEBUG(Q_FUNC_INFO << ", xScales/yScales size: " << d->xScales.size() << '/' << d->yScales.size())
+	// DEBUG(Q_FUNC_INFO << ", xScales/yScales size: " << d->xScales.size() << '/' << d->yScales.size())
 
 	QVectorIterator<CartesianScale*> xIterator(d->xScales);
 	while (xIterator.hasNext()) {
@@ -450,7 +448,7 @@ Lines CartesianCoordinateSystem::mapLogicalToScene(const Lines& lines, MappingFl
 				QLineF mappedLine(QPointF(x1, y1), QPointF(x2, y2));
 				if (doPageClipping) {
 					if (!AbstractCoordinateSystem::clipLineToRect(&mappedLine, pageRect)) {
-						DEBUG(Q_FUNC_INFO << ", WARNING: OMIT mapped line!")
+						// DEBUG(Q_FUNC_INFO << ", WARNING: OMIT mapped line!")
 						continue;
 					}
 				}
@@ -464,9 +462,9 @@ Lines CartesianCoordinateSystem::mapLogicalToScene(const Lines& lines, MappingFl
 	return result;
 }
 
-//##############################################################################
-//######################### scene to logical mappers ###########################
-//##############################################################################
+// ##############################################################################
+// ######################### scene to logical mappers ###########################
+// ##############################################################################
 Points CartesianCoordinateSystem::mapSceneToLogical(const Points& points, MappingFlags flags) const {
 	QRectF pageRect = d->plot->dataRect();
 	Points result;
@@ -720,9 +718,9 @@ bool CartesianCoordinateSystem::rectContainsPoint(const QRectF& rect, QPointF po
 	return true;
 }
 
-//##############################################################################
-//######################### Private implementation #############################
-//##############################################################################
+// ##############################################################################
+// ######################### Private implementation #############################
+// ##############################################################################
 CartesianCoordinateSystemPrivate::CartesianCoordinateSystemPrivate(CartesianCoordinateSystem* owner)
 	: q(owner) {
 }
