@@ -49,6 +49,15 @@ bool XYAnalysisCurve::resultAvailable() const {
 	return result().available;
 }
 
+bool XYAnalysisCurve::usingColumn(const Column* column) const {
+	Q_D(const XYAnalysisCurve);
+
+	if (d->dataSourceType == DataSourceType::Spreadsheet)
+		return (d->xDataColumn == column || d->yDataColumn == column || d->y2DataColumn == column);
+	else
+		return (d->dataSourceCurve->xColumn() == column || d->dataSourceCurve->yColumn() == column);
+}
+
 // copy valid data from x/y data columns to x/y data vectors
 // for analysis functions
 // avgUniqueX: average y values for duplicate x values
@@ -152,9 +161,9 @@ void XYAnalysisCurve::copyData(QVector<double>& xData,
 	}
 }
 
-//##############################################################################
-//##########################  getter methods  ##################################
-//##############################################################################
+// ##############################################################################
+// ##########################  getter methods  ##################################
+// ##############################################################################
 BASIC_SHARED_D_READER_IMPL(XYAnalysisCurve, XYAnalysisCurve::DataSourceType, dataSourceType, dataSourceType)
 BASIC_SHARED_D_READER_IMPL(XYAnalysisCurve, const XYCurve*, dataSourceCurve, dataSourceCurve)
 const QString& XYAnalysisCurve::dataSourceCurvePath() const {
@@ -173,9 +182,9 @@ bool XYAnalysisCurve::saveCalculations() const {
 	return const_cast<XYAnalysisCurve*>(this)->project()->saveCalculations();
 }
 
-//##############################################################################
-//#################  setter methods and undo commands ##########################
-//##############################################################################
+// ##############################################################################
+// #################  setter methods and undo commands ##########################
+// ##############################################################################
 STD_SETTER_CMD_IMPL_S(XYAnalysisCurve, SetDataSourceType, XYAnalysisCurve::DataSourceType, dataSourceType)
 void XYAnalysisCurve::setDataSourceType(DataSourceType type) {
 	Q_D(XYAnalysisCurve);
@@ -212,7 +221,7 @@ void XYAnalysisCurve::setXDataColumn(const AbstractColumn* column) {
 		handleSourceDataChanged();
 		if (column) {
 			setXDataColumnPath(column->path());
-			connect(column->parentAspect(), &AbstractAspect::aspectAboutToBeRemoved, this, &XYAnalysisCurve::xDataColumnAboutToBeRemoved);
+			connect(column->parentAspect(), &AbstractAspect::childAspectAboutToBeRemoved, this, &XYAnalysisCurve::xDataColumnAboutToBeRemoved);
 			connect(column, SIGNAL(dataChanged(const AbstractColumn*)), this, SLOT(handleSourceDataChanged()));
 			connect(column, &AbstractAspect::aspectDescriptionChanged, this, &XYAnalysisCurve::xDataColumnNameChanged);
 			// TODO disconnect on undo
@@ -230,7 +239,7 @@ void XYAnalysisCurve::setYDataColumn(const AbstractColumn* column) {
 		handleSourceDataChanged();
 		if (column) {
 			setYDataColumnPath(column->path());
-			connect(column->parentAspect(), &AbstractAspect::aspectAboutToBeRemoved, this, &XYAnalysisCurve::yDataColumnAboutToBeRemoved);
+			connect(column->parentAspect(), &AbstractAspect::childAspectAboutToBeRemoved, this, &XYAnalysisCurve::yDataColumnAboutToBeRemoved);
 			connect(column, SIGNAL(dataChanged(const AbstractColumn*)), this, SLOT(handleSourceDataChanged()));
 			connect(column, &AbstractAspect::aspectDescriptionChanged, this, &XYAnalysisCurve::yDataColumnNameChanged);
 			// TODO disconnect on undo
@@ -248,7 +257,7 @@ void XYAnalysisCurve::setY2DataColumn(const AbstractColumn* column) {
 		handleSourceDataChanged();
 		if (column) {
 			setY2DataColumnPath(column->path());
-			connect(column->parentAspect(), &AbstractAspect::aspectAboutToBeRemoved, this, &XYAnalysisCurve::y2DataColumnAboutToBeRemoved);
+			connect(column->parentAspect(), &AbstractAspect::childAspectAboutToBeRemoved, this, &XYAnalysisCurve::y2DataColumnAboutToBeRemoved);
 			connect(column, SIGNAL(dataChanged(const AbstractColumn*)), this, SLOT(handleSourceDataChanged()));
 			connect(column, &AbstractAspect::aspectDescriptionChanged, this, &XYAnalysisCurve::y2DataColumnNameChanged);
 			// TODO disconnect on undo
@@ -272,9 +281,9 @@ void XYAnalysisCurve::setY2DataColumnPath(const QString& path) {
 	d->y2DataColumnPath = path;
 }
 
-//##############################################################################
-//#################################  SLOTS  ####################################
-//##############################################################################
+// ##############################################################################
+// #################################  SLOTS  ####################################
+// ##############################################################################
 void XYAnalysisCurve::handleSourceDataChanged() {
 	Q_D(XYAnalysisCurve);
 	d->sourceDataChangedSinceLastRecalc = true;
@@ -362,9 +371,28 @@ void XYAnalysisCurve::createDataSpreadsheet() {
 	folder()->addChild(spreadsheet);
 }
 
-//##############################################################################
-//######################### Private implementation #############################
-//##############################################################################
+void XYAnalysisCurve::handleAspectUpdated(const QString& aspectPath, const AbstractAspect* aspect) {
+	const auto column = dynamic_cast<const AbstractColumn*>(aspect);
+	if (!column)
+		return;
+
+	setUndoAware(false);
+	if (xDataColumnPath() == aspectPath)
+		setXDataColumn(column);
+	if (yDataColumnPath() == aspectPath)
+		setYDataColumn(column);
+	if (y2DataColumnPath() == aspectPath)
+		setY2DataColumn(column);
+
+	// From XYCurve
+	if (valuesColumnPath() == aspectPath)
+		setValuesColumn(column);
+	setUndoAware(true);
+}
+
+// ##############################################################################
+// ######################### Private implementation #############################
+// ##############################################################################
 XYAnalysisCurvePrivate::XYAnalysisCurvePrivate(XYAnalysisCurve* owner)
 	: XYCurvePrivate(owner)
 	, q(owner) {
@@ -406,8 +434,10 @@ void XYAnalysisCurvePrivate::recalculate() {
 	} else {
 		xColumn->invalidateProperties();
 		yColumn->invalidateProperties();
-		xVector->clear();
-		yVector->clear();
+		if (xVector)
+			xVector->clear();
+		if (yVector)
+			yVector->clear();
 	}
 
 	resetResults();
@@ -425,7 +455,7 @@ void XYAnalysisCurvePrivate::recalculate() {
 
 		if (result) {
 			// redraw the curve
-			recalcLogicalPoints();
+			recalc();
 		}
 	}
 	Q_EMIT q->dataChanged();
@@ -435,9 +465,9 @@ bool XYAnalysisCurvePrivate::preparationValid(const AbstractColumn* tmpXDataColu
 	return tmpXDataColumn && tmpYDataColumn;
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 //! Save as XML
 void XYAnalysisCurve::save(QXmlStreamWriter* writer) const {
 	Q_D(const XYAnalysisCurve);
@@ -463,7 +493,6 @@ void XYAnalysisCurve::save(QXmlStreamWriter* writer) const {
 bool XYAnalysisCurve::load(XmlStreamReader* reader, bool preview) {
 	Q_D(XYAnalysisCurve);
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -485,6 +514,10 @@ bool XYAnalysisCurve::load(XmlStreamReader* reader, bool preview) {
 			READ_COLUMN(xDataColumn);
 			READ_COLUMN(yDataColumn);
 			READ_COLUMN(y2DataColumn);
+		} else { // unknown element
+			reader->raiseUnknownElementWarning();
+			if (!reader->skipToEndElement())
+				return false;
 		}
 	}
 
