@@ -63,6 +63,7 @@ public:
 	virtual void recalculate() = 0;
 	bool resultAvailable() const;
 	virtual const Result& result() const = 0;
+	bool usingColumn(const Column*) const override;
 
 	void save(QXmlStreamWriter*) const override;
 	bool load(XmlStreamReader*, bool preview) override;
@@ -84,6 +85,7 @@ public:
 
 protected:
 	XYAnalysisCurve(const QString& name, XYAnalysisCurvePrivate*, AspectType);
+	virtual void handleAspectUpdated(const QString& aspectPath, const AbstractAspect*) override;
 
 private:
 	Q_DECLARE_PRIVATE(XYAnalysisCurve)
