@@ -1,13 +1,12 @@
 /*
-    File                 : XYCorrelationCurve.h
-    Project              : LabPlot
-    Description          : A xy-curve defined by a correlation
-    --------------------------------------------------------------------
-    SPDX-FileCopyrightText: 2018 Stefan Gerlach <stefan.gerlach@uni.kn>
-    SPDX-License-Identifier: GPL-2.0-or-later
+	File                 : XYCorrelationCurve.h
+	Project              : LabPlot
+	Description          : A xy-curve defined by a correlation
+	--------------------------------------------------------------------
+	SPDX-FileCopyrightText: 2018 Stefan Gerlach <stefan.gerlach@uni.kn>
+	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-
 #ifndef XYCORRELATIONCURVE_H
 #define XYCORRELATIONCURVE_H
 
@@ -24,27 +23,22 @@ class XYCorrelationCurve : public XYAnalysisCurve {
 
 public:
 	struct CorrelationData {
-		CorrelationData() {};
+		CorrelationData(){};
 
-		double samplingInterval{1.};			// sampling interval used when no x-axis is present
-		nsl_corr_type_type type{nsl_corr_type_linear};	// linear or circular
-		nsl_corr_norm_type normalize{nsl_corr_norm_none};	// normalization
-		bool autoRange{true};				// use all data?
-		//TODO: use Range
-		QVector<double> xRange{0., 0.};			// x range for correlation
-	};
-	struct CorrelationResult {
-		CorrelationResult() {};
-
-		bool available{false};
-		bool valid{false};
-		QString status;
-		qint64 elapsedTime{0};
+		double samplingInterval{1.}; // sampling interval used when no x-axis is present
+		nsl_corr_type_type type{nsl_corr_type_linear}; // linear or circular
+		nsl_corr_norm_type normalize{nsl_corr_norm_none}; // normalization
+		bool autoRange{true}; // use all data?
+		// TODO: use Range
+		QVector<double> xRange{0., 0.}; // x range for correlation
 	};
 
 	explicit XYCorrelationCurve(const QString& name);
 	~XYCorrelationCurve() override;
 
+	typedef XYAnalysisCurve::Result CorrelationResult;
+	virtual const XYAnalysisCurve::Result& result() const override;
+
 	void recalculate() override;
 	QIcon icon() const override;
 	void save(QXmlStreamWriter*) const override;
