@@ -81,14 +81,14 @@ QIcon XYEquationCurve::icon() const {
 	return QIcon::fromTheme(QStringLiteral("labplot-xy-equation-curve"));
 }
 
-//##############################################################################
-//##########################  getter methods  ##################################
-//##############################################################################
+// ##############################################################################
+// ##########################  getter methods  ##################################
+// ##############################################################################
 BASIC_SHARED_D_READER_IMPL(XYEquationCurve, XYEquationCurve::EquationData, equationData, equationData)
 
-//##############################################################################
-//#################  setter methods and undo commands ##########################
-//##############################################################################
+// ##############################################################################
+// #################  setter methods and undo commands ##########################
+// ##############################################################################
 STD_SETTER_CMD_IMPL_F_S(XYEquationCurve, SetEquationData, XYEquationCurve::EquationData, equationData, recalculate)
 void XYEquationCurve::setEquationData(const XYEquationCurve::EquationData& equationData) {
 	Q_D(XYEquationCurve);
@@ -97,9 +97,9 @@ void XYEquationCurve::setEquationData(const XYEquationCurve::EquationData& equat
 		exec(new XYEquationCurveSetEquationDataCmd(d, equationData, ki18n("%1: set equation")));
 }
 
-//##############################################################################
-//#################################  SLOTS  ####################################
-//##############################################################################
+// ##############################################################################
+// #################################  SLOTS  ####################################
+// ##############################################################################
 
 /*!
  * creates a new spreadsheet having the data with the results of the calculation.
@@ -129,9 +129,9 @@ void XYEquationCurve::createDataSpreadsheet() {
 	folder()->addChild(spreadsheet);
 }
 
-//##############################################################################
-//######################### Private implementation #############################
-//##############################################################################
+// ##############################################################################
+// ######################### Private implementation #############################
+// ##############################################################################
 XYEquationCurvePrivate::XYEquationCurvePrivate(XYEquationCurve* owner)
 	: XYCurvePrivate(owner)
 	, xColumn(new Column(QStringLiteral("x"), AbstractColumn::ColumnMode::Double))
@@ -155,7 +155,7 @@ void XYEquationCurvePrivate::recalculate() {
 			// invalid number of points provided
 			xVector->clear();
 			yVector->clear();
-			recalcLogicalPoints();
+			recalc();
 			Q_EMIT q->dataChanged();
 			return;
 		}
@@ -189,13 +189,13 @@ void XYEquationCurvePrivate::recalculate() {
 	xColumn->invalidateProperties();
 	yColumn->invalidateProperties();
 
-	recalcLogicalPoints();
+	recalc();
 	Q_EMIT q->dataChanged();
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 //! Save as XML
 void XYEquationCurve::save(QXmlStreamWriter* writer) const {
 	Q_D(const XYEquationCurve);
@@ -222,7 +222,6 @@ void XYEquationCurve::save(QXmlStreamWriter* writer) const {
 bool XYEquationCurve::load(XmlStreamReader* reader, bool preview) {
 	Q_D(XYEquationCurve);
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -246,6 +245,10 @@ bool XYEquationCurve::load(XmlStreamReader* reader, bool preview) {
 			READ_STRING_VALUE("min", equationData.min);
 			READ_STRING_VALUE("max", equationData.max);
 			READ_INT_VALUE("count", equationData.count, int);
+		} else { // unknown element
+			reader->raiseUnknownElementWarning();
+			if (!reader->skipToEndElement())
+				return false;
 		}
 	}
 
