@@ -4,7 +4,7 @@
 	Description          : A xy-curve defined by an interpolation
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2016-2021 Stefan Gerlach <stefan.gerlach@uni.kn>
-	SPDX-FileCopyrightText: 2016-2017 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2016-2023 Alexander Semke <alexander.semke@web.de>
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -27,9 +27,9 @@
 extern "C" {
 #include "backend/nsl/nsl_diff.h"
 #include "backend/nsl/nsl_int.h"
+}
 #include <gsl/gsl_interp.h>
 #include <gsl/gsl_spline.h>
-}
 
 #include <QElapsedTimer>
 #include <QIcon>
@@ -59,9 +59,9 @@ QIcon XYInterpolationCurve::icon() const {
 	return QIcon::fromTheme(QStringLiteral("labplot-xy-interpolation-curve"));
 }
 
-//##############################################################################
-//##########################  getter methods  ##################################
-//##############################################################################
+// ##############################################################################
+// ##########################  getter methods  ##################################
+// ##############################################################################
 BASIC_SHARED_D_READER_IMPL(XYInterpolationCurve, XYInterpolationCurve::InterpolationData, interpolationData, interpolationData)
 
 const XYAnalysisCurve::Result& XYInterpolationCurve::result() const {
@@ -69,18 +69,18 @@ const XYAnalysisCurve::Result& XYInterpolationCurve::result() const {
 	return d->interpolationResult;
 }
 
-//##############################################################################
-//#################  setter methods and undo commands ##########################
-//##############################################################################
+// ##############################################################################
+// #################  setter methods and undo commands ##########################
+// ##############################################################################
 STD_SETTER_CMD_IMPL_F_S(XYInterpolationCurve, SetInterpolationData, XYInterpolationCurve::InterpolationData, interpolationData, recalculate)
 void XYInterpolationCurve::setInterpolationData(const XYInterpolationCurve::InterpolationData& interpolationData) {
 	Q_D(XYInterpolationCurve);
 	exec(new XYInterpolationCurveSetInterpolationDataCmd(d, interpolationData, ki18n("%1: set options and perform the interpolation")));
 }
 
-//##############################################################################
-//######################### Private implementation #############################
-//##############################################################################
+// ##############################################################################
+// ######################### Private implementation #############################
+// ##############################################################################
 XYInterpolationCurvePrivate::XYInterpolationCurvePrivate(XYInterpolationCurve* owner)
 	: XYAnalysisCurvePrivate(owner)
 	, q(owner) {
@@ -172,6 +172,7 @@ bool XYInterpolationCurvePrivate::recalculateSpecific(const AbstractColumn* tmpX
 	///////////////////////////////////////////////////////////
 	int status = 0;
 
+	gsl_set_error_handler_off();
 	gsl_interp_accel* acc = gsl_interp_accel_alloc();
 	gsl_spline* spline = nullptr;
 	switch (type) {
@@ -220,6 +221,16 @@ bool XYInterpolationCurvePrivate::recalculateSpecific(const AbstractColumn* tmpX
 		double x = xmin + i * (xmax - xmin) / (npoints - 1);
 		(*xVector)[(int)i] = x;
 
+		// make sure the value for x determined above is within the ranges to avoid subtle issues
+		// related to the representation of float numbers
+		if (i == 0 && x < xmin) {
+			x = xmin;
+			(*xVector)[(int)i] = xmin;
+		} else if (i == npoints - 1 && x > xmax) {
+			x = xmax;
+			(*xVector)[(int)i] = x;
+		}
+
 		// find index a,b for interval [x[a],x[b]] around x[i] using bisection
 		if (type == nsl_interp_type_cosine || type == nsl_interp_type_exponential || type == nsl_interp_type_pch) {
 			while (b - a > 1) {
@@ -377,9 +388,9 @@ bool XYInterpolationCurvePrivate::recalculateSpecific(const AbstractColumn* tmpX
 	return true;
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 //! Save as XML
 void XYInterpolationCurve::save(QXmlStreamWriter* writer) const {
 	Q_D(const XYInterpolationCurve);
@@ -426,7 +437,6 @@ void XYInterpolationCurve::save(QXmlStreamWriter* writer) const {
 bool XYInterpolationCurve::load(XmlStreamReader* reader, bool preview) {
 	Q_D(XYInterpolationCurve);
 
-	KLocalizedString attributeWarning = ki18n("Attribute '%1' missing or empty, default value is used");
 	QXmlStreamAttributes attribs;
 	QString str;
 
@@ -470,6 +480,10 @@ bool XYInterpolationCurve::load(XmlStreamReader* reader, bool preview) {
 				d->xColumn = column;
 			else if (column->name() == QLatin1String("y"))
 				d->yColumn = column;
+		} else { // unknown element
+			reader->raiseUnknownElementWarning();
+			if (!reader->skipToEndElement())
+				return false;
 		}
 	}
 
@@ -492,7 +506,7 @@ bool XYInterpolationCurve::load(XmlStreamReader* reader, bool preview) {
 		static_cast<XYCurvePrivate*>(d_ptr)->xColumn = d->xColumn;
 		static_cast<XYCurvePrivate*>(d_ptr)->yColumn = d->yColumn;
 
-		recalcLogicalPoints();
+		recalc();
 	}
 
 	return true;
