@@ -9,10 +9,13 @@
 */
 
 #include "GuiTools.h"
+#include "backend/core/Settings.h"
 #include "backend/worksheet/plots/cartesian/Symbol.h"
 
-#include <KI18n/KLocalizedString>
+#include <KConfigGroup>
+#include <KLocalizedString>
 
+#include <QActionGroup>
 #include <QApplication>
 #include <QColor>
 #include <QComboBox>
@@ -24,8 +27,12 @@
 #include <QScreen>
 
 #ifdef HAVE_POPPLER
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <poppler-qt6.h>
+#else
 #include <poppler-qt5.h>
 #endif
+#endif
 
 #include <array>
 
@@ -41,6 +48,10 @@ static std::array<QColor, colorsCount> colors = {
 	QColor(128, 128, 128), QColor(160, 160, 160), QColor(195, 195, 195) // grey
 };
 
+bool GuiTools::isDarkMode() {
+	return (QApplication::palette().color(QPalette::Base).lightness() < 128);
+}
+
 /*!
 	fills the ComboBox \c combobox with the six possible Qt::PenStyles, the color \c color is used.
 */
@@ -157,7 +168,7 @@ void GuiTools::updateBrushStyles(QComboBox* comboBox, const QColor& color) {
 										   i18n("Backward Diag. Lines"),
 										   i18n("Forward Diag. Lines"),
 										   i18n("Crossing Diag. Lines")};
-	const QColor& borderColor = DARKMODE ? Qt::white : Qt::black;
+	const QColor& borderColor = GuiTools::isDarkMode() ? Qt::white : Qt::black;
 	for (int i = 0; i < 15; i++) {
 		pm.fill(Qt::transparent);
 		pa.begin(&pm);
@@ -237,7 +248,7 @@ void GuiTools::highlight(QWidget* widget, bool invalid) {
 void GuiTools::addSymbolStyles(QComboBox* cb) {
 	QPainter pa;
 	QPen pen(Qt::SolidPattern, 0);
-	const QColor& color = DARKMODE ? Qt::white : Qt::black;
+	const QColor& color = GuiTools::isDarkMode() ? Qt::white : Qt::black;
 	pen.setColor(color);
 	pa.setPen(pen);
 
@@ -262,7 +273,7 @@ void GuiTools::addSymbolStyles(QComboBox* cb) {
 }
 
 QString GuiTools::openImageFile(const QString& className) {
-	KConfigGroup conf(KSharedConfig::openConfig(), className);
+	KConfigGroup conf = Settings::group(className);
 	const QString& dir = conf.readEntry(QLatin1String("LastImageDir"), QString());
 
 	QString formats;
@@ -318,6 +329,7 @@ QImage GuiTools::importPDFFile(const QString& fileName) {
 
 	return image;
 #else
+	Q_UNUSED(fileName)
 	return {};
 #endif
 }
@@ -352,6 +364,23 @@ QImage GuiTools::imageFromPDFData(const QByteArray& data, double zoomFactor) {
 
 	return image;
 #else
+	Q_UNUSED(data)
+	Q_UNUSED(zoomFactor)
 	return {};
 #endif
 }
+
+/*
+ * replaces the file extension in the path name \c path with \c extension or adds it if not extension available yet.
+ */
+QString GuiTools::replaceExtension(const QString& path, const QString& extension) {
+	const int lastSeparatorIndex = path.lastIndexOf(QDir::separator());
+	const int index = path.lastIndexOf(QLatin1Char('.'));
+	QString newPath;
+	if (index < lastSeparatorIndex) // dot in front of the last separator, is part of the path and not of the file name -> no extension available yet
+		newPath = path + extension;
+	else
+		newPath = path.left(index) + extension;
+
+	return newPath;
+}
