@@ -21,6 +21,8 @@ class QAction;
 
 class GuiTools {
 public:
+	static bool isDarkMode();
+
 	static void updateBrushStyles(QComboBox*, const QColor&);
 	static void updatePenStyles(QComboBox*, const QColor&);
 	static void updatePenStyles(QMenu*, QActionGroup*, const QColor&);
@@ -37,6 +39,8 @@ public:
 	static QString openImageFile(const QString&);
 	static QImage importPDFFile(const QString&);
 	static QImage imageFromPDFData(const QByteArray&, double zoomFactor = 1.);
+
+	static QString replaceExtension(const QString& fileName, const QString& extension);
 };
 
 #endif // GUITOOLS_H
