@@ -8,12 +8,14 @@
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
+
 #ifndef MAINWIN_H
 #define MAINWIN_H
 
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
 
 #include <KXmlGuiWindow>
+#include <QStringLiteral>
 #include <QTimer>
 
 class AbstractAspect;
@@ -22,33 +24,20 @@ class Folder;
 class ProjectExplorer;
 class Project;
 class Worksheet;
-class Note;
-class Workbook;
-class Datapicker;
 class Spreadsheet;
-class Matrix;
 class GuiObserver;
 class CursorDock;
+class ContentDockWidget;
 class MemoryWidget;
-class CartesianPlot;
-class InfoElementDialog;
-
-#ifdef HAVE_CANTOR_LIBS
-class CantorWorksheet;
-#endif
-
-class ImportDatasetWidget;
-class TreeModel;
 // class WelcomeScreenHelper;
+class WorksheetPreviewWidget;
 
 class QDockWidget;
 class QDragEnterEvent;
 class QDropEvent;
-class QMdiArea;
-class QMdiSubWindow;
 class QStackedWidget;
 class QToolButton;
-class QQuickWidget;
+// class QQuickWidget;
 
 class KColorSchemeManager;
 class KHamburgerMenu;
@@ -56,9 +45,24 @@ class KRecentFilesAction;
 class KToggleAction;
 class KToggleFullScreenAction;
 
+namespace ads {
+class CDockManager;
+class CDockWidget;
+}
+
 #ifdef HAVE_KUSERFEEDBACK
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+#include <KUserFeedbackQt6/Provider>
+#else
 #include <KUserFeedback/Provider>
 #endif
+#endif
+
+#ifdef HAVE_PURPOSE
+namespace Purpose {
+class Menu;
+}
+#endif
 
 #ifdef HAVE_TOUCHBAR
 class KDMacTouchBar;
@@ -77,7 +81,8 @@ public:
 	void addAspectToProject(AbstractAspect*);
 	static void updateLocale();
 
-	enum class LoadOnStart { Nothing, NewProject, NewProjectWorksheet, NewProjectSpreadsheet, LastProject, WelcomeScreen };
+	enum class LoadOnStart { NewProject, LastProject, WelcomeScreen };
+	enum class NewProject { WithSpreadsheet, WithWorksheet, WithSpreadsheetWorksheet, WithNotebook };
 	enum class TitleBarMode { ShowFilePath, ShowFileName, ShowProjectName };
 
 #ifdef HAVE_KUSERFEEDBACK
@@ -87,15 +92,19 @@ public:
 #endif
 
 private:
-	QMdiArea* m_mdiArea{nullptr};
+	ads::CDockManager* m_dockManagerContent{nullptr};
+	ads::CDockManager* m_dockManagerMain{nullptr};
 	KColorSchemeManager* m_schemeManager{nullptr};
-	QMdiSubWindow* m_currentSubWindow{nullptr};
+	ContentDockWidget* m_currentDock{nullptr}; // Currently selected dock
 	Project* m_project{nullptr};
 	AspectTreeModel* m_aspectTreeModel{nullptr};
 	ProjectExplorer* m_projectExplorer{nullptr};
-	QDockWidget* m_projectExplorerDock{nullptr};
-	QDockWidget* m_propertiesDock{nullptr};
+	WorksheetPreviewWidget* m_worksheetPreviewWidget{nullptr};
+	ads::CDockWidget* m_projectExplorerDock{nullptr};
+	ads::CDockWidget* m_propertiesDock{nullptr};
+	ads::CDockWidget* m_worksheetPreviewDock{nullptr};
 	AbstractAspect* m_currentAspect{nullptr};
+	ads::CDockWidget* m_currentAspectDock{nullptr};
 	Folder* m_currentFolder{nullptr};
 	QString m_undoViewEmptyLabel;
 	bool m_suppressCurrentSubWindowChangedEvent{false};
@@ -103,14 +112,12 @@ private:
 	bool m_projectClosing{false};
 	bool m_autoSaveActive{false};
 	QTimer m_autoSaveTimer;
-	bool m_showWelcomeScreen{false};
-	bool m_saveWelcomeScreen{true};
+	// bool m_showWelcomeScreen{false};
+	// bool m_saveWelcomeScreen{true};
 	int undoStackIndexLastSave{0};
 	MemoryWidget* m_memoryInfoWidget{nullptr};
-	QMdiSubWindow* m_welcomeWindow{nullptr};
-	QQuickWidget* m_welcomeWidget{nullptr};
-	// 	WelcomeScreenHelper* m_welcomeScreenHelper{nullptr};
-	ImportDatasetWidget* m_importDatasetWidget{nullptr};
+	// QQuickWidget* m_welcomeWidget{nullptr};
+	// WelcomeScreenHelper* m_welcomeScreenHelper{nullptr};
 	QString m_lastOpenFileFilter;
 	const Worksheet* m_lastWorksheet{nullptr};
 	const Spreadsheet* m_lastSpreadsheet{nullptr};
@@ -141,7 +148,7 @@ private:
 	QAction* m_importLabPlotAction;
 	QAction* m_importOpjAction;
 	QAction* m_exportAction;
-	QAction* m_closeAction;
+	// QAction* m_closeAction;
 	QAction* m_newFolderAction;
 	QAction* m_newWorkbookAction;
 	QAction* m_newSpreadsheetAction;
@@ -149,8 +156,6 @@ private:
 	QAction* m_newWorksheetAction;
 	QAction* m_newNotesAction;
 	QAction* m_newLiveDataSourceAction;
-	QAction* m_newSqlDataSourceAction;
-	QAction* m_newScriptAction;
 	QAction* m_newProjectAction;
 	QAction* m_openProjectAction;
 	QAction* m_historyAction;
@@ -158,18 +163,18 @@ private:
 	QAction* m_redoAction;
 	QAction* m_closeWindowAction;
 	QAction* m_closeAllWindowsAction;
-	QAction* m_tileWindowsAction;
-	QAction* m_cascadeWindowsAction;
 	QAction* m_nextWindowAction;
 	QAction* m_prevWindowAction;
 	QAction* m_newDatapickerAction;
 
 	// toggling dock widgets, status bar and full screen
-	QAction* m_toggleProjectExplorerDockAction;
-	QAction* m_togglePropertiesDockAction;
-	KToggleAction* m_toggleStatusBarAction;
-	QAction* m_toggleMemoryInfoAction;
-	KToggleFullScreenAction* m_toggleFullScreenAction;
+	QAction* m_projectExplorerDockAction;
+	QAction* m_propertiesDockAction;
+	QAction* m_worksheetPreviewAction;
+	KToggleAction* m_statusBarAction;
+	QAction* m_memoryInfoAction;
+	KToggleFullScreenAction* m_fullScreenAction;
+	QAction* m_configureCASAction;
 
 	// window visibility
 	QAction* m_visibilityFolderAction;
@@ -180,20 +185,32 @@ private:
 	QMenu* m_visibilityMenu{nullptr};
 	QMenu* m_newMenu{nullptr};
 	QMenu* m_importMenu{nullptr};
+	QMenu* m_newNotebookMenu{nullptr};
 	KHamburgerMenu* m_hamburgerMenu{nullptr};
 
+#ifdef HAVE_PURPOSE
+	QAction* m_shareAction{nullptr};
+	Purpose::Menu* m_shareMenu{nullptr};
+	void fillShareMenu();
+#endif
+
 	// Docks
+	ads::CDockWidget* cursorDock{nullptr};
+
 	QStackedWidget* stackedWidget{nullptr};
-	QDockWidget* cursorDock{nullptr};
 	CursorDock* cursorWidget{nullptr};
 
 	void initActions();
 	void initMenus();
 	bool warnModified();
-	void activateSubWindowForAspect(const AbstractAspect*) const;
+	void activateSubWindowForAspect(const AbstractAspect*);
 	bool save(const QString&);
 	// 	void toggleShowWidget(QWidget* widget, bool showToRight);
 	// 	void toggleHideWidget(QWidget* widget, bool hideToLeft);
+	// 	QQuickWidget* createWelcomeScreen();
+	// 	void resetWelcomeScreen();
+	void initDocks();
+	void restoreDefaultDockState() const;
 
 	Spreadsheet* activeSpreadsheet() const;
 
@@ -207,15 +224,17 @@ protected:
 
 private Q_SLOTS:
 	void initGUI(const QString&);
-	// 	QQuickWidget* createWelcomeScreen();
-	// 	void resetWelcomeScreen();
-	void createMdiArea();
+	void changeVisibleAllDocks(bool);
+	void activateNextDock();
+	void activatePreviousDock();
+	void dockWidgetRemoved(ads::CDockWidget*);
+	void dockFocusChanged(ads::CDockWidget* old, ads::CDockWidget* now);
 	void updateGUI();
 	void updateGUIOnProjectChanges();
 	void undo();
 	void redo();
 
-	bool newProject();
+	bool newProject(bool createInitialContent = true);
 	void openProject();
 	void openProject(const QString&);
 	void openRecentProject(const QUrl&);
@@ -240,10 +259,10 @@ private Q_SLOTS:
 	void colorSchemeChanged(QAction*);
 	void openDatasetExample();
 
-	// Cantor
 #ifdef HAVE_CANTOR_LIBS
-	void newCantorWorksheet(QAction* action);
-	void cantorSettingsDialog();
+	void newNotebook();
+	void casSettingsDialog();
+	void updateNotebookActions();
 #endif
 
 	void newFolder();
@@ -253,7 +272,6 @@ private Q_SLOTS:
 	void newWorksheet();
 	void newNotes();
 	void newDatapicker();
-	// TODO: void newScript();
 	void newLiveDataSource();
 
 	void createContextMenu(QMenu*) const;
@@ -263,13 +281,12 @@ private Q_SLOTS:
 	void handleAspectAboutToBeRemoved(const AbstractAspect*);
 	void handleAspectRemoved(const AbstractAspect*, const AbstractAspect*, const AbstractAspect*);
 	void handleCurrentAspectChanged(AbstractAspect*);
-	void handleCurrentSubWindowChanged(QMdiSubWindow*);
 	void handleShowSubWindowRequested();
 
 	void handleSettingsChanges();
 
-	void setMdiWindowVisibility(QAction*);
-	void updateMdiWindowVisibility() const;
+	void setDockVisibility(QAction*);
+	void updateDockWindowVisibility() const;
 	void toggleDockWidget(QAction*);
 	void toggleStatusBar(bool);
 	void toggleMenuBar(bool);
@@ -277,10 +294,17 @@ private Q_SLOTS:
 	void toggleFullScreen(bool);
 	void projectExplorerDockVisibilityChanged(bool);
 	void propertiesDockVisibilityChanged(bool);
+	void worksheetPreviewDockVisibilityChanged(bool);
 	void cursorDockVisibilityChanged(bool);
 	void propertiesExplorerRequested();
 
+	void focusCursorDock();
+
 	void cartesianPlotMouseModeChanged(CartesianPlot::MouseMode);
+
+#ifdef HAVE_PURPOSE
+	void shareActionFinished(const QJsonObject& output, int error, const QString& message);
+#endif
 };
 
 #endif
