@@ -35,7 +35,8 @@ private:
 	void retranslateUi();
 
 private Q_SLOTS:
-	void interfaceChanged(int);
+	void loadOnStartChanged();
+	void newProjectChanged();
 	void autoSaveChanged(bool);
 	void changed();
 
