@@ -9,12 +9,12 @@
 */
 
 #include "SettingsWorksheetPage.h"
+#include "backend/core/Settings.h"
 #include "kdefrontend/widgets/ThemesComboBox.h"
 #include "tools/TeXRenderer.h"
 
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <KSharedConfig>
 
 /**
  * \brief Page for the 'General' settings of the Labplot settings dialog.
@@ -58,7 +58,7 @@ void SettingsWorksheetPage::applySettings() {
 	if (!m_changed)
 		return;
 
-	KConfigGroup group = KSharedConfig::openConfig()->group(QLatin1String("Settings_Worksheet"));
+	KConfigGroup group = Settings::group(QStringLiteral("Settings_Worksheet"));
 	if (m_cbThemes->currentText() == i18n("Default"))
 		group.writeEntry(QLatin1String("Theme"), QString());
 	else
@@ -86,7 +86,7 @@ void SettingsWorksheetPage::restoreDefaults() {
 }
 
 void SettingsWorksheetPage::loadSettings() {
-	const KConfigGroup group = KSharedConfig::openConfig()->group(QLatin1String("Settings_Worksheet"));
+	const KConfigGroup group = Settings::group(QStringLiteral("Settings_Worksheet"));
 	m_cbThemes->setItemText(0, group.readEntry(QLatin1String("Theme"), ""));
 	ui.chkPresenterModeInteractive->setChecked(group.readEntry(QLatin1String("PresenterModeInteractive"), false));
 	ui.chkDoubleBuffering->setChecked(group.readEntry(QLatin1String("DoubleBuffering"), true));
@@ -108,7 +108,7 @@ void SettingsWorksheetPage::loadSettings() {
 
 		if (index != -1) {
 			// one of the tex engines was found -> automatically save it in the settings without any user action
-			KConfigGroup group = KSharedConfig::openConfig()->group(QLatin1String("Settings_Worksheet"));
+			KConfigGroup group = Settings::group(QStringLiteral("Settings_Worksheet"));
 			group.writeEntry(QLatin1String("LaTeXEngine"), ui.cbTexEngine->itemData(index));
 		}
 	} else
