@@ -3,8 +3,7 @@
 	Project              : LabPlot
 	Description          : import SQL database dialog
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2016 Ankit Wagadre <wagadre.ankit@gmail.com>
-	SPDX-FileCopyrightText: 2016-2019 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2016-2023 Alexander Semke <alexander.semke@web.de>
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -13,9 +12,9 @@
 
 #include "kdefrontend/datasources/ImportDialog.h"
 
+class ImportSQLDatabaseWidget;
 class MainWin;
 class QStatusBar;
-class ImportSQLDatabaseWidget;
 
 class ImportSQLDatabaseDialog : public ImportDialog {
 	Q_OBJECT
@@ -24,7 +23,7 @@ public:
 	explicit ImportSQLDatabaseDialog(MainWin*);
 	~ImportSQLDatabaseDialog() override;
 
-	void importTo(QStatusBar*) const override;
+	bool importTo(QStatusBar*) const override;
 	QString selectedObject() const override;
 
 private:
