@@ -3,12 +3,13 @@
 	Project              : LabPlot
 	Description          : widget for image properties
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2019-2022 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2019-2023 Alexander Semke <alexander.semke@web.de>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "ImageDock.h"
+#include "backend/core/Settings.h"
 #include "backend/worksheet/Image.h"
 #include "backend/worksheet/Worksheet.h"
 #include "kdefrontend/GuiTools.h"
@@ -16,6 +17,10 @@
 #include "kdefrontend/ThemeHandler.h"
 #include "kdefrontend/widgets/LineWidget.h"
 
+#include <KConfig>
+#include <KConfigGroup>
+#include <KLocalizedString>
+
 #include <QCompleter>
 // see https://gitlab.kitware.com/cmake/cmake/-/issues/21609
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
@@ -25,8 +30,7 @@
 #endif
 #include <QPageSize>
 
-#include <KConfig>
-#include <KLocalizedString>
+#include <gsl/gsl_const_cgs.h>
 
 /*!
   \class ImageDock
@@ -38,9 +42,8 @@
 ImageDock::ImageDock(QWidget* parent)
 	: BaseDock(parent) {
 	ui.setupUi(this);
-	m_leName = ui.leName;
-	m_teComment = ui.teComment;
-	m_teComment->setFixedHeight(1.2 * m_leName->height());
+	setBaseWidgets(ui.leName, ui.teComment);
+	setVisibilityWidgets(ui.chbVisible);
 
 	ui.bOpen->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
@@ -88,9 +91,7 @@ ImageDock::ImageDock(QWidget* parent)
 
 	// SLOTs
 	// General
-	connect(ui.leName, &QLineEdit::textChanged, this, &ImageDock::nameChanged);
 	connect(ui.chbEmbedded, &QCheckBox::clicked, this, &ImageDock::embeddedChanged);
-	connect(ui.teComment, &QTextEdit::textChanged, this, &ImageDock::commentChanged);
 	connect(ui.bOpen, &QPushButton::clicked, this, &ImageDock::selectFile);
 	connect(ui.leFileName, &QLineEdit::returnPressed, this, &ImageDock::fileNameChanged);
 	connect(ui.leFileName, &QLineEdit::textChanged, this, &ImageDock::fileNameChanged);
@@ -114,8 +115,8 @@ ImageDock::ImageDock(QWidget* parent)
 	connect(ui.dtePositionXLogical, &UTCDateTimeEdit::mSecsSinceEpochUTCChanged, this, &ImageDock::positionXLogicalDateTimeChanged);
 	connect(ui.sbPositionYLogical, QOverload<double>::of(&NumberSpinBox::valueChanged), this, &ImageDock::positionYLogicalChanged);
 
+	connect(ui.chbLock, &QCheckBox::clicked, this, &ImageDock::lockChanged);
 	connect(ui.chbBindLogicalPos, &QCheckBox::clicked, this, &ImageDock::bindingChanged);
-	connect(ui.chbVisible, &QCheckBox::clicked, this, &ImageDock::visibilityChanged);
 }
 
 void ImageDock::setImages(QList<Image*> list) {
@@ -126,27 +127,6 @@ void ImageDock::setImages(QList<Image*> list) {
 
 	updateLocale();
 
-	// if there are more than one image in the list, disable the name and comment field in the tab "general"
-	if (list.size() == 1) {
-		ui.lName->setEnabled(true);
-		ui.leName->setEnabled(true);
-		ui.lComment->setEnabled(true);
-		ui.teComment->setEnabled(true);
-
-		ui.leName->setText(m_image->name());
-		ui.teComment->setText(m_image->comment());
-	} else {
-		ui.lName->setEnabled(false);
-		ui.leName->setEnabled(false);
-		ui.lComment->setEnabled(false);
-		ui.teComment->setEnabled(false);
-
-		ui.leName->setText(QString());
-		ui.teComment->setText(QString());
-	}
-	ui.leName->setStyleSheet(QString());
-	ui.leName->setToolTip(QString());
-
 	QList<Line*> lines;
 	for (auto* image : m_imageList)
 		lines << image->borderLine();
@@ -158,11 +138,10 @@ void ImageDock::setImages(QList<Image*> list) {
 
 	// init connections
 	// General
-	connect(m_image, &Image::aspectDescriptionChanged, this, &ImageDock::aspectDescriptionChanged);
 	connect(m_image, &Image::fileNameChanged, this, &ImageDock::imageFileNameChanged);
 	connect(m_image, &Image::embeddedChanged, this, &ImageDock::imageEmbeddedChanged);
 	connect(m_image, &Image::opacityChanged, this, &ImageDock::imageOpacityChanged);
-	connect(m_image, &Image::visibleChanged, this, &ImageDock::imageVisibleChanged);
+	connect(m_image, &Image::lockChanged, this, &ImageDock::imageLockChanged);
 
 	// Size
 	connect(m_image, &Image::widthChanged, this, &ImageDock::imageWidthChanged);
@@ -191,7 +170,7 @@ void ImageDock::updateLocale() {
 }
 
 void ImageDock::updateUnits() {
-	const KConfigGroup group = KSharedConfig::openConfig()->group(QStringLiteral("Settings_General"));
+	const KConfigGroup group = Settings::group(QStringLiteral("Settings_General"));
 	BaseDock::Units units = (BaseDock::Units)group.readEntry("Units", static_cast<int>(Units::Metric));
 	if (units == m_units)
 		return;
@@ -203,18 +182,18 @@ void ImageDock::updateUnits() {
 		// convert from imperial to metric
 		m_worksheetUnit = Worksheet::Unit::Centimeter;
 		suffix = QStringLiteral(" cm");
-		ui.sbWidth->setValue(ui.sbWidth->value() * 2.54);
-		ui.sbHeight->setValue(ui.sbHeight->value() * 2.54);
-		ui.sbPositionX->setValue(ui.sbPositionX->value() * 2.54);
-		ui.sbPositionY->setValue(ui.sbPositionY->value() * 2.54);
+		ui.sbWidth->setValue(ui.sbWidth->value() * GSL_CONST_CGS_INCH);
+		ui.sbHeight->setValue(ui.sbHeight->value() * GSL_CONST_CGS_INCH);
+		ui.sbPositionX->setValue(ui.sbPositionX->value() * GSL_CONST_CGS_INCH);
+		ui.sbPositionY->setValue(ui.sbPositionY->value() * GSL_CONST_CGS_INCH);
 	} else {
 		// convert from metric to imperial
 		m_worksheetUnit = Worksheet::Unit::Inch;
 		suffix = QStringLiteral(" in");
-		ui.sbWidth->setValue(ui.sbWidth->value() / 2.54);
-		ui.sbHeight->setValue(ui.sbHeight->value() / 2.54);
-		ui.sbPositionX->setValue(ui.sbPositionX->value() / 2.54);
-		ui.sbPositionY->setValue(ui.sbPositionY->value() / 2.54);
+		ui.sbWidth->setValue(ui.sbWidth->value() / GSL_CONST_CGS_INCH);
+		ui.sbHeight->setValue(ui.sbHeight->value() / GSL_CONST_CGS_INCH);
+		ui.sbPositionX->setValue(ui.sbPositionX->value() / GSL_CONST_CGS_INCH);
+		ui.sbPositionY->setValue(ui.sbPositionY->value() / GSL_CONST_CGS_INCH);
 	}
 
 	ui.sbWidth->setSuffix(suffix);
@@ -447,11 +426,10 @@ void ImageDock::rotationChanged(int value) {
 		image->setRotationAngle(value);
 }
 
-void ImageDock::visibilityChanged(bool state) {
+void ImageDock::lockChanged(bool locked) {
 	CONDITIONAL_LOCK_RETURN;
-
 	for (auto* image : m_imageList)
-		image->setVisible(state);
+		image->setLock(locked);
 }
 
 //*************************************************************
@@ -524,9 +502,9 @@ void ImageDock::imageRotationAngleChanged(qreal angle) {
 	ui.sbRotation->setValue(angle);
 }
 
-void ImageDock::imageVisibleChanged(bool on) {
+void ImageDock::imageLockChanged(bool on) {
 	CONDITIONAL_LOCK_RETURN;
-	ui.chbVisible->setChecked(on);
+	ui.chbLock->setChecked(on);
 }
 
 //*************************************************************
@@ -541,6 +519,7 @@ void ImageDock::load() {
 	ui.leFileName->setText(m_image->fileName());
 	ui.chbEmbedded->setChecked(m_image->embedded());
 	embeddedChanged(ui.chbEmbedded->checkState());
+	ui.chbLock->setChecked(m_image->isLocked());
 	ui.chbVisible->setChecked(m_image->isVisible());
 
 	// Size
