@@ -3,13 +3,15 @@
 	Project              : LabPlot
 	Description          : widget for spreadsheet properties
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2010-2019 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2010-2023 Alexander Semke <alexander.semke@web.de>
 	SPDX-FileCopyrightText: 2012-2013 Stefan Gerlach <stefan.gerlach@uni-konstanz.de>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "SpreadsheetDock.h"
+#include "backend/core/AspectTreeModel.h"
+#include "backend/core/Project.h"
 #include "backend/datapicker/DatapickerCurve.h"
 #include "backend/spreadsheet/Spreadsheet.h"
 #include "commonfrontend/spreadsheet/SpreadsheetView.h"
@@ -30,22 +32,31 @@
 SpreadsheetDock::SpreadsheetDock(QWidget* parent)
 	: BaseDock(parent) {
 	ui.setupUi(this);
-	m_leName = ui.leName;
-	m_teComment = ui.teComment;
-	ui.teComment->setFixedHeight(1.2 * ui.leName->height());
+	setBaseWidgets(ui.leName, ui.teComment);
 
-	connect(ui.leName, &QLineEdit::textChanged, this, &SpreadsheetDock::nameChanged);
-	connect(ui.teComment, &QTextEdit::textChanged, this, &SpreadsheetDock::commentChanged);
 	connect(ui.sbColumnCount, QOverload<int>::of(&QSpinBox::valueChanged), this, &SpreadsheetDock::columnCountChanged);
 	connect(ui.sbRowCount, QOverload<int>::of(&QSpinBox::valueChanged), this, &SpreadsheetDock::rowCountChanged);
 	connect(ui.cbShowComments, &QCheckBox::toggled, this, &SpreadsheetDock::commentsShownChanged);
+	connect(ui.cbShowSparklines, &QCheckBox::toggled, this, &SpreadsheetDock::sparklinesShownChanged);
 
-	auto* templateHandler = new TemplateHandler(this, TemplateHandler::ClassName::Spreadsheet);
-	ui.gridLayout->addWidget(templateHandler, 11, 0, 1, 4);
+	connect(ui.cbLinkingEnabled, &QCheckBox::toggled, this, &SpreadsheetDock::linkingChanged);
+	connect(ui.cbLinkedSpreadsheet, &TreeViewComboBox::currentModelIndexChanged, this, &SpreadsheetDock::linkedSpreadsheetChanged);
+
+	auto* templateHandler = new TemplateHandler(this, QLatin1String("Spreadsheet"));
+	ui.gridLayout->addWidget(templateHandler, 17, 0, 1, 4);
 	templateHandler->show();
 	connect(templateHandler, &TemplateHandler::loadConfigRequested, this, &SpreadsheetDock::loadConfigFromTemplate);
 	connect(templateHandler, &TemplateHandler::saveConfigRequested, this, &SpreadsheetDock::saveConfigAsTemplate);
 	connect(templateHandler, &TemplateHandler::info, this, &SpreadsheetDock::info);
+
+	// tooltip texts
+	QString info = i18n("Enable linking to synchronize the number of rows with another spreadsheet");
+	ui.lLinkingEnabled->setToolTip(info);
+	ui.cbLinkingEnabled->setToolTip(info);
+
+	info = i18n("Spreadsheet to synchronize the number of rows with");
+	ui.lLinkedSpreadsheet->setToolTip(info);
+	ui.cbLinkedSpreadsheet->setToolTip(info);
 }
 
 /*!
@@ -66,30 +77,28 @@ void SpreadsheetDock::setSpreadsheets(const QList<Spreadsheet*> list) {
 		}
 	}
 
-	if (list.size() == 1) {
-		ui.leName->setEnabled(true);
-		ui.teComment->setEnabled(true);
+	auto* model = aspectModel();
+	model->setSelectableAspects({AspectType::Spreadsheet});
+	model->enableNumericColumnsOnly(true);
+	// model->enableNonEmptyNumericColumnsOnly(true);
 
-		ui.leName->setText(m_spreadsheet->name());
-		ui.teComment->setText(m_spreadsheet->comment());
-	} else {
-		// disable the fields "Name" and "Comment" if there are more than one spreadsheet
-		ui.leName->setEnabled(false);
-		ui.teComment->setEnabled(false);
+	ui.cbLinkedSpreadsheet->setTopLevelClasses({AspectType::Folder, AspectType::Workbook, AspectType::Spreadsheet});
+	ui.cbLinkedSpreadsheet->setModel(model);
 
-		ui.leName->setText(QString());
-		ui.teComment->setText(QString());
-	}
-	ui.leName->setStyleSheet(QString());
-	ui.leName->setToolTip(QString());
+	// don't allow to select self spreadsheet!
+	QList<const AbstractAspect*> aspects;
+	for (auto* sh : m_spreadsheetList)
+		aspects << sh;
+	ui.cbLinkedSpreadsheet->setHiddenAspects(aspects);
 
 	// show the properties of the first Spreadsheet in the list
 	this->load();
 
 	// undo functions
-	connect(m_spreadsheet, &AbstractAspect::aspectDescriptionChanged, this, &SpreadsheetDock::aspectDescriptionChanged);
 	connect(m_spreadsheet, &Spreadsheet::rowCountChanged, this, &SpreadsheetDock::spreadsheetRowCountChanged);
 	connect(m_spreadsheet, &Spreadsheet::columnCountChanged, this, &SpreadsheetDock::spreadsheetColumnCountChanged);
+	connect(m_spreadsheet, &Spreadsheet::linkingChanged, this, &SpreadsheetDock::spreadsheetLinkingChanged);
+	connect(m_spreadsheet, &Spreadsheet::linkedSpreadsheetChanged, this, &SpreadsheetDock::spreadsheetLinkedSpreadsheetChanged);
 	// TODO: show comments
 
 	ui.lDimensions->setVisible(!nonEditable);
@@ -100,6 +109,8 @@ void SpreadsheetDock::setSpreadsheets(const QList<Spreadsheet*> list) {
 	ui.lFormat->setVisible(!nonEditable);
 	ui.lShowComments->setVisible(!nonEditable);
 	ui.cbShowComments->setVisible(!nonEditable);
+	ui.lShowSparklines->setVisible(!nonEditable);
+	ui.cbShowSparklines->setVisible(!nonEditable);
 }
 
 //*************************************************************
@@ -128,6 +139,42 @@ void SpreadsheetDock::commentsShownChanged(bool state) {
 	for (auto* spreadsheet : m_spreadsheetList)
 		static_cast<SpreadsheetView*>(spreadsheet->view())->showComments(state);
 }
+/*!
+  enable/disable the sparkline header in the views of the selected spreadsheets.
+*/
+void SpreadsheetDock::sparklinesShownChanged(bool state) {
+	CONDITIONAL_LOCK_RETURN;
+
+	for (auto* spreadsheet : m_spreadsheetList)
+		static_cast<SpreadsheetView*>(spreadsheet->view())->showSparkLines(state);
+}
+
+void SpreadsheetDock::linkingChanged(bool linking) {
+	ui.sbRowCount->setEnabled(!linking);
+	ui.lLinkedSpreadsheet->setVisible(linking);
+	ui.cbLinkedSpreadsheet->setVisible(linking);
+
+	CONDITIONAL_LOCK_RETURN;
+
+	for (auto* spreadsheet : m_spreadsheetList)
+		spreadsheet->setLinking(linking);
+}
+
+void SpreadsheetDock::linkedSpreadsheetChanged(const QModelIndex& index) {
+	// combobox was potentially red-highlighted because of a missing column
+	// remove the highlighting when we have a valid selection now
+	auto* aspect{static_cast<AbstractAspect*>(index.internalPointer())};
+	if (aspect) {
+		auto* cb{dynamic_cast<TreeViewComboBox*>(QObject::sender())};
+		if (cb)
+			cb->setStyleSheet(QString());
+		auto* sh = dynamic_cast<Spreadsheet*>(aspect);
+		if (sh) {
+			for (auto* spreadsheet : m_spreadsheetList)
+				spreadsheet->setLinkedSpreadsheet(sh);
+		}
+	}
+}
 
 //*************************************************************
 //******** SLOTs for changes triggered in Spreadsheet *********
@@ -147,6 +194,21 @@ void SpreadsheetDock::spreadsheetShowCommentsChanged(bool checked) {
 	ui.cbShowComments->setChecked(checked);
 }
 
+void SpreadsheetDock::spreadsheetShowSparklinesChanged(bool checked) {
+	CONDITIONAL_LOCK_RETURN;
+	ui.cbShowSparklines->setChecked(checked);
+}
+
+void SpreadsheetDock::spreadsheetLinkingChanged(bool linking) {
+	CONDITIONAL_LOCK_RETURN;
+	ui.cbLinkingEnabled->setChecked(linking);
+}
+
+void SpreadsheetDock::spreadsheetLinkedSpreadsheetChanged(const Spreadsheet* spreadsheet) {
+	CONDITIONAL_LOCK_RETURN;
+	ui.cbLinkedSpreadsheet->setAspect(spreadsheet);
+}
+
 //*************************************************************
 //******************** SETTINGS *******************************
 //*************************************************************
@@ -156,17 +218,14 @@ void SpreadsheetDock::load() {
 
 	auto* view = static_cast<SpreadsheetView*>(m_spreadsheet->view());
 	ui.cbShowComments->setChecked(view->areCommentsShown());
+	ui.cbShowSparklines->setChecked(view->areSparkLinesShown());
+	ui.cbLinkedSpreadsheet->setAspect(m_spreadsheet->linkedSpreadsheet());
+	ui.cbLinkingEnabled->setChecked(m_spreadsheet->linking());
+	linkingChanged(m_spreadsheet->linking()); // call this to update the widgets
 }
 
 void SpreadsheetDock::loadConfigFromTemplate(KConfig& config) {
-	// extract the name of the template from the file name
-	QString name;
-	const int index = config.name().lastIndexOf(QLatin1Char('/'));
-	if (index != -1)
-		name = config.name().right(config.name().size() - index - 1);
-	else
-		name = config.name();
-
+	auto name = TemplateHandler::templateName(config);
 	const int size = m_spreadsheetList.size();
 	if (size > 1)
 		m_spreadsheet->beginMacro(i18n("%1 spreadsheets: template \"%2\" loaded", size, name));
@@ -182,22 +241,25 @@ void SpreadsheetDock::loadConfigFromTemplate(KConfig& config) {
 	loads saved spreadsheet properties from \c config.
  */
 void SpreadsheetDock::loadConfig(KConfig& config) {
-	KConfigGroup group = config.group("Spreadsheet");
+	KConfigGroup group = config.group(QStringLiteral("Spreadsheet"));
 
-	ui.sbColumnCount->setValue(group.readEntry("ColumnCount", m_spreadsheet->columnCount()));
-	ui.sbRowCount->setValue(group.readEntry("RowCount", m_spreadsheet->rowCount()));
+	ui.sbColumnCount->setValue(group.readEntry(QStringLiteral("ColumnCount"), m_spreadsheet->columnCount()));
+	ui.sbRowCount->setValue(group.readEntry(QStringLiteral("RowCount"), m_spreadsheet->rowCount()));
 
 	auto* view = static_cast<SpreadsheetView*>(m_spreadsheet->view());
-	ui.cbShowComments->setChecked(group.readEntry("ShowComments", view->areCommentsShown()));
+	ui.cbShowComments->setChecked(group.readEntry(QStringLiteral("ShowComments"), view->areCommentsShown()));
+	ui.cbShowSparklines->setChecked(group.readEntry(QStringLiteral("ShowSparklines"), view->areSparkLinesShown()));
 }
 
 /*!
 	saves spreadsheet properties to \c config.
  */
 void SpreadsheetDock::saveConfigAsTemplate(KConfig& config) {
-	KConfigGroup group = config.group("Spreadsheet");
-	group.writeEntry("ColumnCount", ui.sbColumnCount->value());
-	group.writeEntry("RowCount", ui.sbRowCount->value());
-	group.writeEntry("ShowComments", ui.cbShowComments->isChecked());
+	KConfigGroup group = config.group(QStringLiteral("Spreadsheet"));
+	group.writeEntry(QStringLiteral("ColumnCount"), ui.sbColumnCount->value());
+	group.writeEntry(QStringLiteral("RowCount"), ui.sbRowCount->value());
+	group.writeEntry(QStringLiteral("ShowComments"), ui.cbShowComments->isChecked());
+	group.writeEntry(QStringLiteral("ShowSparklines"), ui.cbShowSparklines->isChecked());
+
 	config.sync();
 }
