@@ -11,12 +11,14 @@
 
 #ifndef SPREADSHEETDOCK_H
 #define SPREADSHEETDOCK_H
+#include "backend/core/AspectTreeModel.h"
 #include "kdefrontend/dockwidgets/BaseDock.h"
 #include "ui_spreadsheetdock.h"
 
 class KConfig;
 class Spreadsheet;
 class AbstractAspect;
+class AspectTreeModel;
 
 class SpreadsheetDock : public BaseDock {
 	Q_OBJECT
@@ -38,11 +40,17 @@ private Q_SLOTS:
 	void rowCountChanged(int);
 	void columnCountChanged(int);
 	void commentsShownChanged(bool);
+	void sparklinesShownChanged(bool);
+	void linkingChanged(bool);
+	void linkedSpreadsheetChanged(const QModelIndex& index);
 
 	// SLOTs for changes triggered in Spreadsheet
 	void spreadsheetRowCountChanged(int);
 	void spreadsheetColumnCountChanged(int);
 	void spreadsheetShowCommentsChanged(bool);
+	void spreadsheetShowSparklinesChanged(bool);
+	void spreadsheetLinkingChanged(bool);
+	void spreadsheetLinkedSpreadsheetChanged(const Spreadsheet*);
 
 	// save/load template
 	void loadConfigFromTemplate(KConfig&);
@@ -50,6 +58,8 @@ private Q_SLOTS:
 
 Q_SIGNALS:
 	void info(const QString&);
+
+	friend class SpreadsheetTest;
 };
 
 #endif // SPREADSHEETDOCK_H
